#ifndef _OME_API_H
//! \cond
#define _OME_API_H "$Id: OMEapi.h 340 2020-05-19 18:43:48Z geoff $"
//! \endcond

#include <OMEioObjects.h>

/** \defgroup OMEapi OME External Application Program Interface
 * @{
 */

/*!
\brief establishes and maintains a connection with a FARGOS/VISTA
Object Management Environment.

All session key negotiation, encoding, compression and encryption is performed
by an OMEconnection object.  This class is normally never accessed directly;
instead, the class OMEapi provides the standard high-level interface.
*/
class OMEconnection {
private:
	OMEioDescriptor	*descriptor;
	OMEstring	destination;
	OMEstring	secret;
	OMEtype		authenticationData;
	OMEoid		*thisObject;
	OMEtype		remoteOID;
	OMEtype		peerOID;
	int		encodeVersion;
	int		encryptCipherID;
	int		encryptKeyID;
	int		decryptCipherID;
	int		decryptKeyID;
	OMEstring	userInfo;
	OMEstring	VISTAdomainName;

	void freeEncryptionResources();

	int sendString(const OMEstring &data, int *errRet = nullptr);

	OMEstring *receiveString(int *errRet = nullptr);

	OMEstring *encodeData(const OMEtype &messData);

	OMEtype *decodeData(const OMEstring &data);

	int negotiateKey();
public:

	OMEconnection(const OMEoid &objId, const OMEstring &destination, const OMEtype &authentication);

	~OMEconnection();

	int establishConnection();

	int sendMessage(const OMEtype &dest, int *errRet = nullptr);

	int receiveMessage(OMEtype &result, int *errRet = nullptr);

	int getPeerObject(OMEtype &obj) {
		obj = peerOID;
		return (0);
	}

	OMEioDescriptor *getDescriptor() const {
		return (descriptor);
	}

	int getHandle() const {
		if (descriptor == nullptr) return (-1);
		return (descriptor->ioHandle);
	}
}; // end class OMEconnection


/*!
\brief The standard mechanism for attaching external processes
to a FARGOS/VISTA Object Management Environment.

New or pre-existing applications can access and be integrated into a
FARGOS/VISTA Object Management Environment using the OMEapi.
*/
class OMEapi {
private:
	static int	connectionCount;
	OMEoid		*thisObject;
	OMEstring	destination;
	OMEconnection	*conn;
	OMEassoc	defaultContext;
	OMEstring	userInfo;
public:
    static OMEassoc *makeACL(const OMEstring &userName);
	/*!
	Constructs a new OMEapi connection and defines the access control
	list and transport address of the remote peer.  An actual connection
	attempt is made by establishConnection().  Any stream-oriented
	transport supported by OMEioDescriptor can be used.
	*/
	OMEapi(const OMEassoc &acl, const char *dest) NONNULL_CLASS_PARAMETERS(3);

	OMEapi(const OMEassoc &acl, const OMEstring &dest);

	~OMEapi();

	/*!
	Attempt to establish a peer connection to the remote FARGOS/VISTA
	Object Management Environment.
	\retval -1 is returned if the connection attempt was not successful.
	\retval 0 indicates success.
	*/
	int establishConnection(OMEtype &authentication);

	/*!
	Sends a message to a remote object that is accessible via the set
	of interconnected FARGOS/VISTA Object Management Environment peers.
	\param dest identifies the target of the message.  Usually this is
	an object Id, but it can be a string name that identifies a named
	service.  Name services are resolved to object Ids by the connected
	FARGOS/VISTA Object Management Environment.
	\param methodName identifies the name of the method to be invoked.
	\param args is an array of arguments for the method.  The first
	argument, if any, starts at subscript 0.  The count of the number
	of arguments is performed automatically, so there is no
	need nor provision for providing this information.
	\param fromObj is an optional argument, which defaults to the object
	Id assigned to this instance of the OMEapi (which can be retrieved
	by getThisObject() ).
	\param context is an optional associative array that enables a
	specific thread context to be set for the method invocation.  If
	not provided, the default context (set by setContext() ) is used.
	\retval -1 is returned on error.
	\retval 0 indicates success.
	*/
	int invokeMethod(const OMEtype &dest, const OMEtype &methodName,
		const OMEtype &args, const OMEtype *fromObj = nullptr,
		const OMEassoc *context  = nullptr);

	/*! \brief Receives a message from a remote object that is accessible
     * via the set of interconnected FARGOS/VISTA Object Management
     * Environment peers.
	 * \param methodName identifies the name of the method which is
     * to be invoked.
	 * \param argv is an array of arguments for the method.  The first
	 * argument, if any, starts at subscript 0.  The OMEtype::indexExists()
	 * function can be used to determine if an argument is present.
	 * \param fromObj is an optional argument. If provided, the object id of
	 * the sender will be saved; otherwise, this information will be discarded.
	 * \param destObj is an optional argument.  If provided, the object Id
	 * of the intended target will be saved; otherwise, this information will
	 * be discarded.  Usually, \a destObj will be identical to that
	 * returned by getThisObject(); however, since it is possible to have a
	 * given OMEapi() connection maintain access to multiple objects, some
	 * applications will need this information to select a specific object.
	 * \param context is an optional associative array that enables a
	 * specific thread context to be set for the method invocation.  If
	 * not provided, the default context (set by setContext() ) is used.
	 * \param user is an optional argument. If provided, information about
	 * the identity of the user requesting the operation will be stored.
	 * \retval -1 is returned on error.
	 * \retval 0 indicates success.
	 */
	int importInvocation(OMEtype &methodName, OMEtype &argv,
		OMEtype *fromObj = nullptr, OMEtype *destObj = nullptr,
		OMEtype *context = nullptr, OMEtype *user = nullptr);

	/*!
	Sets the default thread context
	*/
	void setContext(const OMEassoc &env) { defaultContext = env; }

	void setUserInfo();

	/*!
	\return
	The Id of the peer object (typically a PeerConnection)
	that maintains the other end of the connection is stored in the
	\arg obj argument, which is passed by reference.
	\retval -1 if not connected.
	\retval 0 indicates success.
	*/
	int getPeerObject(OMEtype &obj) {
		if (conn == nullptr) return (-1);
		return (conn->getPeerObject(obj));
	}

	/*!
	\return 
	The object Id of the globally unique Id that identifies
	this instance of the OMEapi is stored in the \arg obj which is
	passed by reference.  This can be provided to any entity attached
	to the set of interconnected FARGOS/VISTA Object Management
	Environments.
	\retval -1 if not connected.
	\retval 0 indicates success.
	*/
	int getThisObject(OMEtype &obj) {
		if (conn == nullptr) return (-1);
		obj = *thisObject;
		return (0);
	}

	/*!
	\return If connected, the pointer to the OMEioDescriptor that provides
	I/O transport facilities for this OMEapi object is returned.
	\retval 0 if not connected, a null pointer is returned.
	*/
	OMEioDescriptor *getDescriptor() const {
		if (conn == nullptr) return (nullptr);
		return (conn->getDescriptor());
	}

	/*!
	Similar to getDescriptor(), getHandle() returns the file descriptor
	or handle maintained by the underlying native operating system
	for an open I/O port/connection.
	\return
	If connected, the file descriptor/handle is returned.
	\retval -1 is returned if not connected.
	*/
	int getHandle() const {
		if (conn == nullptr) {
            return (-1);
        }
		return (conn->getHandle());
	}
}; // end class OMEapi

/** @}
 */

#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
