#ifndef _OME_ASSOC_H
//! \cond
#define _OME_ASSOC_H "$Id: OMEassoc.h 312 2020-03-21 21:25:35Z geoff $"
//! \endcond

/*! \file
 * \brief OME associative array implementation.
 */

#include <map>

#include <OMErefCount.h>
#include <OMEstring.h>

/** \addtogroup OMEtypes
 * @{
 */

/*! \brief  Holds the string / data tuple for an element of
 * an OMEassoc sparse array.
 */
class OMEassocKeyPair
{
    friend class OMEassocStorage;

private:
    class OMEstring *key;
    class OMEbaseType *value;

    OMEassocKeyPair(OMEstring *k, OMEbaseType *v)
    {
        key = k;
        value = v;
    }

    // delete implementation in OMEassoc.cpp since OMEbaseType not known
    ~OMEassocKeyPair();

    bool operator==(const OMEassocKeyPair &arg) const;

    bool operator!=(const OMEassocKeyPair &arg) const;

    template <typename STREAMTYPE>
    STREAMTYPE &outputOnStream(STREAMTYPE &outputOnStream, int_fast16_t indent = 0,
                               uint8_t includeTypePrefix = OME_DEFAULT_COMPLEX_OUTPUT_MODE) const;
}; // end class OMEassocKeyPair

typedef class OMEassocKeyPair *OME_KEY_PAIR;

// OMEassocStorage is used internally by OMEassoc, thus all of its
// members are private and OMEassoc is listed as a friend.

class OMEassocStorage final : public OMEreferenceCount
{
    friend class OMEassoc;
    friend class OMEreferenceToData<OMEassocStorage>;

public:
    typedef uint32_t ASSOC_HASH_KEY_t;

private:
    typedef _STD map<ASSOC_HASH_KEY_t, OME_KEY_PAIR> AssocArrayType_t;
    AssocArrayType_t assocArray;

    // default initializer
    OMEassocStorage();

    // destructor
    ~OMEassocStorage();

    virtual OMEreferenceCount *deepCopy() const override;

    bool operator==(const OMEassocStorage &arg) const;

    bool operator!=(const OMEassocStorage &arg) const;

    OMEbaseType &operator[](const ASSOC_HASH_KEY_t);

    const OMEbaseType &operator[](const ASSOC_HASH_KEY_t) const;

    bool deleteIndex(const ASSOC_HASH_KEY_t);

    const OMEstring &getKeyForIndex(const ASSOC_HASH_KEY_t i) const;

    bool indexExists(const ASSOC_HASH_KEY_t i) const;

    ASSOC_HASH_KEY_t nextIndex(const ASSOC_HASH_KEY_t currentSubscript) const;

    OMEbaseType &operator[](const OMEstring &);

    const OMEbaseType &operator[](const OMEstring &) const;

    bool deleteIndex(const OMEstring &);

    bool indexExists(const OMEstring &) const;

    uint_fast32_t elementCount() const OME_ALWAYS_INLINE
    {
        return (assocArray.size());
    }

    bool isEmpty() const OME_ALWAYS_INLINE
    {
        return (assocArray.empty());
    }

    template <typename STREAMTYPE>
    STREAMTYPE &outputOnStream(STREAMTYPE &outputOnStream, int_fast16_t indent = 0,
        uint8_t includeTypePrefix = OME_DEFAULT_COMPLEX_OUTPUT_MODE) const;
}; // end class OMEassocStorage

class OME_DLL_EXPORT OMEassoc final : public OMEreferenceToData<OMEassocStorage>
{
private:
    explicit OMEassoc(OMEassocStorage *s)
    {
        // only used by deepCopy()
        data = s;
        // TODO:  may add unwanted reference since storage objects have 
        // initial reference count of 1
        // addReadOnlyReference();
    }

public:
    typedef OMEassocStorage::ASSOC_HASH_KEY_t ASSOC_HASH_KEY_t;
    // default initializer
    OMEassoc()
    {
        data = new OMEassocStorage();
    }

    // copy initializer
    explicit OMEassoc(const OMEassoc &org) : OMEreferenceToData<OMEassocStorage>(org.data)
    {
//        data = org.data;
//        addReadOnlyReference();
    }

    // assignment
    OMEassoc &operator=(const OMEassoc &arg)
    {
        if (this->data == arg.data) {
            //std::cout << "OMEassoc assignment to self, ref=" << totalReferences() << "\n";
            return (*this); // asignment to self...
        }
        arg.data->addReadOnlyReference();
        dropReference(arg.data);
        return (*this);
    }

    ~OMEassoc() {}

    // COMPARISON
    bool operator==(const OMEassoc &arg) const
    {
        if (data == arg.data) { // points at same object
            return (true);  // fast path
        }
        // evaluate by comparing content
        return (*data == *arg.data);
    }

    bool operator!=(const OMEassoc &arg) const
    {
        if (data == arg.data) { // points at same object
            return (false); // fast path
        }
        // evaluate by comparing content
        return (*data != *arg.data);
    }

    OMEbaseType &operator[](const ASSOC_HASH_KEY_t i)
    {
        getUniqueReference();
        return ((*data)[i]);
    }

    const OMEbaseType &operator[](const ASSOC_HASH_KEY_t i) const
    {
        // read-only
        const OMEassocStorage &a = *data;
        return (a[i]);
    }

    OMEbaseType &operator[](const OMEstring &key)
    {
        getUniqueReference();
        return ((*data)[key]);
    }

    const OMEbaseType &operator[](const OMEstring &key) const
    {
        // read-only
        const OMEassocStorage &a = *data;
        return (a[key]);
    }

    const OMEstring &getKeyForIndex(const ASSOC_HASH_KEY_t i) const
    {
        return (data->getKeyForIndex(i));
    }

    bool deleteIndex(const ASSOC_HASH_KEY_t i)
    {
        getUniqueReference();
        return (data->deleteIndex(i));
    }

    bool forceDeleteIndex(const ASSOC_HASH_KEY_t i)
    {
        return (data->deleteIndex(i));
    }

    bool deleteIndex(const OMEstring &key)
    {
        getUniqueReference();
        return (data->deleteIndex(key));
    }

    bool forceDeleteIndex(const OMEstring &key)
    {
        // don't do getUniqueReference()
        return (data->deleteIndex(key));
    }

    bool indexExists(const ASSOC_HASH_KEY_t i) const
    {
        return (data->indexExists(i));
    }

    bool indexExists(const OMEstring &key) const
    {
        return (data->indexExists(key));
    }

    ASSOC_HASH_KEY_t nextIndex(const ASSOC_HASH_KEY_t currentSubscript) const
    {
        return (data->nextIndex(currentSubscript));
    }

    uint_fast32_t elementCount() const OME_ALWAYS_INLINE
    {
        return (data->elementCount());
    }

    bool isEmpty() const OME_ALWAYS_INLINE
    {
        return (data->isEmpty());
    }

    OMEassoc *deepCopy() const
    {

        OMEassocStorage *storageCopy = static_cast<OMEassocStorage *>(data->deepCopy());
        OMEassoc *copy = new OMEassoc(storageCopy);
        return (copy);
    }

    /*! \brief Output an OMEassoc object to an output stream.
    * \param outputStream is a reference to an output stream.
    * \param indent specifies the number of spaces to indent before
    * beginning output.  If negative, no indent is performed
    * for the first line, but subsequent lines are indented
    * based on the absolute value of \a indent.
    * \param includeTypePrefix is a mask of flags indicating if type names
    * should prefix data elements, if strings should be enclosed in quotes,
    * etc.
    */
    template <typename STREAMTYPE>
    STREAMTYPE &outputOnStream(STREAMTYPE &outputStream, int_fast16_t indent = 0,
        uint8_t includeTypePrefix = OME_DEFAULT_COMPLEX_OUTPUT_MODE) const
    {
        data->outputOnStream(outputStream, indent, includeTypePrefix);
        return (outputStream);
    }
}; // end class OMEassoc

#define OME_ASSOC_SUB(v) static_cast<OMEassoc::ASSOC_HASH_KEY_t>(v)

/*! \brief Output an OMEassoc object to an output stream.
 */
template <typename STREAMTYPE>
inline STREAMTYPE &operator<<(STREAMTYPE &outputStream, const OMEassoc &data)
{
    data.outputOnStream(outputStream, 0);
    return (outputStream);
}

/** @}
 */

#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
