#ifndef _OME_DEBUG_INFO_H
//! \cond
#define _OME_DEBUG_INFO_H "$Id: OMEdebugInfo.h 312 2020-03-21 21:25:35Z geoff $"
//! \endcond

/*! \file
 * \brief OME debug and profiling interfaces.
 */

#include <utils/shared_vars/shared_variable.hpp>

#    include <OMEtype.h>

/** \addtogroup OMEtypes
 * @{
 */

#    define OMEdebugMethodInvocations (2 << 0)
#    define OMEdebugObjectCreate (2 << 1)
#    define OMEdebugObjectDelete (2 << 2)
#    define OMEdebugThreadDelete (2 << 3)
#    define OMEdebugIncludeArguments (2 << 4)
#    define OMEdebugEvents (2 << 5)
#    define OMEdebugExceptions (2 << 6)
#    define OMEdebugBlockedInvocations (2 << 7)
#    define OMEdebugOIL2instructions (2 << 9)
#    define OMEdebugBadParameter (2 << 10)
#    define OMEdebugProfileCounters (2 << 11)


#    define __OME_LOG_LEVEL_BASE 20
#    define OMEdebugLogLevel0 (1 << (__OME_LOG_LEVEL_BASE + 0))
#    define OMEdebugLogLevel1 (1 << (__OME_LOG_LEVEL_BASE + 1))
#    define OMEdebugLogLevel2 (1 << (__OME_LOG_LEVEL_BASE + 2))
#    define OMEdebugLogLevel3 (1 << (__OME_LOG_LEVEL_BASE + 3))

// two aliases for everything...
#    define OMEdebugEverything (~0)
#    define OMEdebugAlways (~0)
#    define OMEdebugNever (0)


extern OME_DLL_EXPORT int OMEsetDebugFlag(const char *flagName,
        bool setFlag = 1) NONNULL_PARAMETERS(1);

extern OME_DLL_EXPORT void OMEinitDebugFlag(const char *paramLine = nullptr);


/*! \brief Base class for OME profiling variable.
 */
class OMEprofileRecord {
protected:
    static OMEprofileRecord *startOfList;

    OMEprofileRecord    *next;


    static void addToList(OMEprofileRecord *rec) {
        rec->next = startOfList;
        startOfList = rec;
    }

    static void removeFromList(OMEprofileRecord *recToRemove) {
        if (startOfList == recToRemove) {
            startOfList = recToRemove->next; // fast path...
        } else {
            OMEprofileRecord *last = nullptr;
            OMEprofileRecord *rec = startOfList;
            while (rec != nullptr) {
                if (rec == recToRemove) {
                    break;
                }
                last = rec;
                rec = rec->next;
            }
            if (last == nullptr) {
                startOfList = rec->next;
            } else {
                last->next = rec->next;
            }
        }
    }


public:
    OMEprofileRecord() {
        next = nullptr;
    }

    virtual ~OMEprofileRecord() {
       removeFromList(this);
    }

    virtual const char *getName(uint_fast32_t *retNameLen=nullptr) const = 0;

    virtual const OMEtype getValue() const = 0;

    virtual void setValue(const OMEtype &newV) = 0;

    static OMEarray *listDefinedCounters();

    static OMEassoc *getCounters();

    static const OMEtype getValueOfCounter(const char *name) NONNULL_PARAMETERS(1);

    /*!  \brief Write a counter's value to an output stream.
     */
    template <typename STREAMTYPE> static STREAMTYPE &outputCounters(STREAMTYPE &o)
    {
        if (startOfList == nullptr) {
            o << "No FARGOS/VISTA OME counters are defined.\n";
            return (o);
        }

        o << "Counter\t\t\t\tValue\n";

        for (OMEprofileRecord *rec = startOfList; rec != nullptr; rec = rec->next) {
            uint_fast32_t s_len;
            const char *cp = rec->getName(&s_len);
            // NOTE:  l must be a signed integer
            int_fast32_t l = static_cast<int_fast32_t>(s_len);

            o << cp;
            l = (4 * 8) - l; // 8 spaces/tab
            do {
                o << "\t";
                l -= 8;
            }
            while (l > 0);
            o << rec->getValue() << "\n";
        }
        return (o);
    }
};


/*! \brief Base class for application profiling counters or
 * numerical system information.
 *
 * The OMEprofileCounter class enables arbitrary performance and
 * informative data to be made available to management applications.
 * Each variable should be assigned a unique text name to enable
 * its identification.
 */
template <typename NUMTYPE>
class OME_DLL_EXPORT OMEprofileCounter : public OMEprofileRecord, public SMV_StandaloneNumeric<NUMTYPE>
{
public:


/*!  \brief Initialize a counter with a potentially external source.
 *
 * \param varName is a pointer to a null-terminated C string that
 * identifies the name of the variable.
 * \param initVal is the initial value of the counter.
 * \param mgr points to the manager for the shared memory segment 
 * exposing the counter.
 */
    explicit OMEprofileCounter(const char *varName, NUMTYPE initVal=0,
            SharedMemoryVariableManager *mgr=&DEFAULT_sharedMemoryVariableManager) NONNULL_CLASS_PARAMETERS(2) :
        SMV_StandaloneNumeric<NUMTYPE>(varName, mgr, initVal)
    {
        addToList(this);
    }

/*! \brief Initialize a counter with a constant value.
 *
 * \param varName is a pointer to a null-terminated C string that
 * identifies the name of the variable.
 * \param parentNode points to the parent node used to name
 * the variable.
 * \param initVal provides the initial value of the variable.
 */
    explicit OMEprofileCounter(const char *varName,
            SharedMemoryVariableNode *parentNode, NUMTYPE initVal=0) NONNULL_CLASS_PARAMETERS(2) :
        SMV_StandaloneNumeric<NUMTYPE>(varName, parentNode, initVal)
    {
        addToList(this);
    }

    virtual ~OMEprofileCounter() {}

    virtual const char *getName(uint_fast32_t *retNameLen=nullptr) const override {
        const char *result = SMV_StandaloneNumeric<NUMTYPE>::getName(retNameLen);
        return (result);
    }

    OMEprofileCounter<NUMTYPE> &operator=(const NUMTYPE v) {
        static_cast<SMV_StandaloneNumeric<NUMTYPE> &>(*this) = v;
        return (*this);
    }

    virtual const OMEtype getValue() const override
    {
        OMEtype result;
        result = static_cast<const NUMTYPE>(*this);
        return (result);
    }

    virtual void setValue(const OMEtype &newV) override
    {
        static_cast<SMV_StandaloneNumeric<NUMTYPE> &>(*this) = static_cast<const NUMTYPE>(newV);
    }

 /*! \brief A convenience function to increment a counter.
 * \note only makes sense if the counter holds an integer value.
 */
    virtual void incValue(NUMTYPE amt=1)
    {
        *this += amt;
    }

    //  virtual OMEprofileCounter &operator++(); // prefix ++



}; // end class OMEprofileCounter<NUMTYPE>

/*! \brief Base type for system information represented as strings.
 */
template <size_t MAXLEN>
class OME_DLL_EXPORT OMEprofileString :
    public OMEprofileRecord,
    public SMV_StandaloneString<MAXLEN>
{
public:
/*! \brief Initialize a profiling string, possibly with an initial value.
 *
 * \param varName is a pointer to a null-terminated C string that
 * identifies the name of the variable.
 * \param initVal provides the initial value of the variable.
 * \param mgr points to the manager of the shared memory segment used to
 * export the variable.
 */
    explicit OMEprofileString(const char *varName, const char *initVal, SharedMemoryVariableManager *mgr=&DEFAULT_sharedMemoryVariableManager) :
        SMV_StandaloneString<MAXLEN>(varName, mgr)
    {
        if (initVal != nullptr) {
            static_cast<SMV_StandaloneString<MAXLEN> &>(*this) = initVal;
        }
        addToList(this);
    }

/*! \brief Initialize a string with a constant value.
 *
 * \param varName is a pointer to a null-terminated C string that
 * identifies the name of the variable.
 * \param parentNode points to the parent node used to name
 * the variable.
 * \param initVal provides the initial value of the variable.
 */
    explicit OMEprofileString(const char *varName, SharedMemoryVariableNode *parentNode,
            const char *initVal=nullptr) :
        SMV_StandaloneString<MAXLEN>(varName, parentNode)
    {
        if (initVal != nullptr) {
            static_cast<SMV_StandaloneString<MAXLEN> &>(*this) = initVal;
        }
        addToList(this);
    }

    virtual ~OMEprofileString() {}

    virtual const char *getName(uint_fast32_t *retNameLen=nullptr) const override {
        const char *result = SMV_StandaloneString<MAXLEN>::getName(retNameLen);
        return (result);
    }

    virtual const OMEtype getValue() const override
    {
        OMEtype result;
        result = static_cast<const char *>(*this);
        return (result);
    }

    OMEprofileString<MAXLEN> &operator=(const char *v) {
        static_cast<SMV_StandaloneString<MAXLEN> &>(*this) = v;
        return (*this);
    }

    virtual void setValue(const OMEtype &newV) override
    {
        static_cast<SMV_StandaloneString<MAXLEN> &>(*this) = static_cast<const char *>(newV);
    }

}; // end class OMEprofileString<MAXLEN>


extern OME_DLL_EXPORT void OMEdisplayModuleComments();

extern "C" OME_DLL_EXPORT void OMEnoteModuleComment(const char *commentText);

extern "C" OME_DLL_EXPORT int32_t OMEshutdownRequested();

extern OME_DLL_EXPORT SMV_StandaloneNumeric<uint32_t> OMEdebugFlag;
extern OME_DLL_EXPORT SMV_StandaloneNumeric<uint32_t> OMEstopFlag;
extern OME_DLL_EXPORT SMV_StandaloneString<128> OMEstopReason;
extern OME_DLL_EXPORT SMV_StandaloneString<128> OMEadminStopReason;
extern OME_DLL_EXPORT SMV_StandaloneString<128> OMEadminProcessLabel;

//! Specifies FARGOS/VISTA Version major number
extern OME_DLL_EXPORT OMEprofileCounter<uint32_t> OMEmajorVersion;
//! Specifies FARGOS/VISTA Version minor number
extern OME_DLL_EXPORT OMEprofileCounter<uint32_t> OMEminorVersion;
//! Specifies FARGOS/VISTA Version release number
extern OME_DLL_EXPORT OMEprofileCounter<uint32_t> OMEreleaseVersion;
//! Specifies name of underlying native operating system
extern OME_DLL_EXPORT OMEprofileString<32> OMEvistaOS;
//! Specifies name of the underlying CPU architecture
extern OME_DLL_EXPORT OMEprofileString<32> OMEvistaCPU;

#    define OME_DEBUG_LEVEL_FLAG(l) (2 << (OME_LOG_LEVEL_BASE + l))

#    define NOTE_DEFAULT_NLM(s) __defaultMess = s

//#    define __CREATE_NLM __nlmRec = new OMEnlm(__defaultMess, &param, &__messID, &__catName, &paramInfo)

#    define DEFINE_NLM_MESSAGE(Lvl, MessID, AppName) \
    if (((Lvl) == ~0) || ((Lvl) &OMEdebugFlag)) { \
        OMEtype __messID, __nlmMessage; \
        OMEarray param, paramInfo; \
        OMEstring __defaultMess, __catName; \
        uint32_t _paramSub = 0; \
        __messID = MessID; \
        __catName = AppName

#    define AND_OUTPUT_NLM(o, t) \
    OMEnlm __nlmRec(__defaultMess, &param, &__messID, &__catName, &paramInfo); \
    o << __nlmRec << t; \
    }

#    define AND_ASSIGN_INTO(v) \
    OMEnlm __nlmRec(__defaultMess, &param, &__messID, &__catName, &paramInfo); \
    v = __nlmRec; \
    }

#    define NEXT_NLM_PARAM param[_paramSub++]

/** @}
 */

#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
