#ifndef _OME_NLM_H
//! \cond
#define _OME_NLM_H "$Id: OMEnlm.h 312 2020-03-21 21:25:35Z geoff $"
//! \endcond

/*! \file
 * \brief OME native language message implementation.
 */

/** \addtogroup OMEtypes
 * @{
 */


#    include <OMEarray.h>

// OMEnlmStorage is used internally by OMEnlm, thus all of its
// members are private and OMEnlm is listed as a friend.

/*! \brief Referenced-counted storage for an OME Native Language Message.
 */
class OMEnlmStorage final : public OMEreferenceCount
{
    friend class OMEnlm;
    friend class OMEreferenceToData<OMEnlmStorage>;

private:
    class OMEnlmData *messageData;
    OMEarray messageArguments;
    OMEarray argumentInfo;
    // possible extras: debug filter flag
    int debugLevel;

    // default initializer
    OMEnlmStorage();

    explicit OMEnlmStorage(const class OMEstring &defaultMess,
            OMEarray *args = nullptr,
            const class OMEbaseType *id = nullptr,
            const class OMEstring *catName = nullptr,
            OMEarray *extras = nullptr);

    // destructor
    ~OMEnlmStorage();

    OMEstring *encodeNLM(const class OMEencodeBuffer *bfr) const;

    // SHOULD NEVER BE CALLED:  NLM's are immutable...
    virtual OMEreferenceCount *deepCopy() const override
    {
        std::cerr << "OMEnlmStorage::deepCopy called!\n";
        return ((OMEnlmStorage *) this);
    }

    bool operator==(const OMEnlmStorage &arg) const;

    bool operator!=(const OMEnlmStorage &arg) const;

    void getMessageInfo(OMEarray &list) const;

    //  OMEbaseType &operator[](const uint32_t);
    const OMEbaseType &operator[](const OMEarray::ARRAY_SUBSCRIPT_t i) const
    {
        return (messageArguments[i]);
    }

    //  int deleteIndex(const uint32_t);
    bool indexExists(const OMEarray::ARRAY_SUBSCRIPT_t i) const
    {
        return (messageArguments.indexExists(i));
    }

    OMEarray::ARRAY_SUBSCRIPT_t nextIndex(const OMEarray::ARRAY_SUBSCRIPT_t currentSubscript) const
    {
        return (messageArguments.nextIndex(currentSubscript));
    }

    uint_fast32_t elementCount() const
    {
        return (messageArguments.elementCount());
    }

    bool isEmpty() const
    {
        return (messageArguments.isEmpty());
    }

    const OMEstring &getMessageText(const OMEstring *lang = nullptr) const;


    template <typename STREAMTYPE> STREAMTYPE &outputOnStream(STREAMTYPE &outputStream, int_fast16_t indent = 0,
            uint8_t includeTypePrefix = OME_DEFAULT_COMPLEX_OUTPUT_MODE,
            const OMEstring *lang = nullptr) const;
}; // end class OMEnlmStorage

/*! \brief Public interface to an OME Native Language Message.
 */
class OME_DLL_EXPORT OMEnlm final : public OMEreferenceToData<OMEnlmStorage>
{
private:
    explicit OMEnlm(OMEnlmStorage *s)
    {
        // only used by deepCopy()
        data = s;
        // addReadOnlyReference();
    }

public:
    static OMEnlm *decodeNLM(uint32_t ver, const OMEstring *encodedData,
                             size_t *offset);

    template <typename STREAMTYPE> static STREAMTYPE &outputFormattedData(STREAMTYPE &o,
            const class OMEbaseType &data,
            const class OMEbaseType &annotation,
            const char *fieldModifier, int modifierLen);

    // NO DEFAULT INITIALIZER
    // copy initializer
    OMEnlm(const OMEnlm &org) : OMEreferenceToData<OMEnlmStorage>(org.data)
    {
//        data = org.data;
//        addReadOnlyReference();
    }

    explicit OMEnlm(const OMEstring &defaultMess, OMEarray *args = nullptr,
                    const OMEbaseType *id = nullptr, const OMEstring *catName = nullptr,
                    OMEarray *extra = nullptr)
    {
        data = new OMEnlmStorage(defaultMess, args, id, catName, extra);
//        addReadOnlyReference();
    }

    explicit OMEnlm(const char *m)
    {
        OMEstring s(m);

        data = new OMEnlmStorage(s);
//        addReadOnlyReference();
    }

    // assignment
    OMEnlm &operator=(const OMEnlm &arg)
    {
        if (this->data == arg.data) {
            std::cerr << "OMEnlm assignment to self, ref=" << totalReferences() << "\n";
            return (*this); // asignment to self...
        }
        arg.data->addReadOnlyReference();
        dropReference();
        data = arg.data;
        return (*this);
    }

    ~OMEnlm() {}

    OMEstring *encodeNLM(const class OMEencodeBuffer *bfr) const
    {
        return (data->encodeNLM(bfr));
    };

    // COMPARISON
    bool operator==(const OMEnlm &arg) const
    {
        if (data == arg.data) {
            return (true); // fast path
        }
        return (*data == *arg.data);
    }

    bool operator!=(const OMEnlm &arg) const
    {
        if (data == arg.data) {
            return (false); // fast path
        }
        return (*data != *arg.data);
    }

    void getMessageInfo(OMEarray &list) const
    {
        data->getMessageInfo(list);
    }

#    ifdef DO_WRITEABLE
    OMEbaseType &operator[](const OMEarray::ARRAY_SUBSCRIPT_t i)
    {
        getUniqueReference();
        return ((*data)[i]);
    }
#    endif

    const OMEbaseType &operator[](const OMEarray::ARRAY_SUBSCRIPT_t i) const
    {
        const OMEnlmStorage &a = *data;
        return (a[i]);
    }
#    ifdef DO_WRITEABLE
    int deleteIndex(const OMEarray::ARRAY_SUBSCRIPT_t i)
    {
        return (data->deleteIndex(i));
    }
#    endif
    bool indexExists(const OMEarray::ARRAY_SUBSCRIPT_t i) const
    {
        return (data->indexExists(i));
    }

    OMEarray::ARRAY_SUBSCRIPT_t nextIndex(const OMEarray::ARRAY_SUBSCRIPT_t currentSubscript) const
    {
        return (data->nextIndex(currentSubscript));
    }

    uint_fast32_t elementCount() const
    {
        return (data->elementCount());
    }

    bool isEmpty() const
    {
        return (data->isEmpty());
    }

    OMEnlm *deepCopy() const
    {

        OMEnlmStorage *storageCopy = static_cast<OMEnlmStorage *>(data->deepCopy());
        OMEnlm *copy = new OMEnlm(storageCopy);
        return (copy);
    }

    /*! \brief Output an OMEnlm object to an output stream.
    * \param outputStream is a reference to an output stream.
    * \param indent specifies the number of spaces to indent before
    * beginning output.  If negative, no indent is performed
    * for the first line, but subsequent lines are indented
    * based on the absolute value of \a indent.
    * \param includeTypePrefix is a mask of flags indicating if type names
    * should prefix data elements, if strings should be enclosed in quotes,
    * etc.
    * \param lang is an optional argument that indicates the intended
    * output language.
    */
    template <typename STREAMTYPE> STREAMTYPE &outputOnStream(STREAMTYPE &outputStream, int_fast16_t indent = 0, uint8_t includeTypePrefix = OME_DEFAULT_COMPLEX_OUTPUT_MODE,
            const OMEstring *lang = nullptr) const
    {
        data->outputOnStream(outputStream, indent, includeTypePrefix, lang);
        return (outputStream);
    }
}; // end class OMEnlm

/*! \brief Output an OMEnlm object to an output stream.
 */
template <typename STREAMTYPE> inline STREAMTYPE &operator<<(STREAMTYPE &outputStream, const OMEnlm &data)
{
    data.outputOnStream(outputStream, 0);
    return (outputStream);
}

/** @}
 */

#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
