#ifndef _OME_SET_H
#    define _OME_SET_H "$Id: OMEset.h 312 2020-03-21 21:25:35Z geoff $"

/*! \file
 * \brief OME set implementation.
 */
/** \addtogroup OMEtypes
 * @{
 */

#    include <deque>

#    include <OMErefCount.h>

typedef class OMEbaseType *OMEbaseTypeP;

// OMEsetStorage is used internally by OMEset, thus all of its
// members are private and OMEset is listed as a friend.

class OMEsetStorage final : public OMEreferenceCount
{
    friend class OMEset;
    friend class OMEreferenceToData<OMEsetStorage>;

private:
    _STD deque<OMEbaseTypeP> dataSet;

    // default initializer
    OMEsetStorage();

    // destructor
    ~OMEsetStorage();

    virtual OMEreferenceCount *deepCopy() const override;

    OMEsetStorage &operator+=(const OMEbaseType &);

    OMEsetStorage &operator-=(const OMEbaseType &);

    OMEsetStorage &operator|=(const OMEbaseType &);

    OMEsetStorage &operator&=(const OMEbaseType &);

    bool operator==(const OMEsetStorage &arg) const;

    bool operator!=(const OMEsetStorage &arg) const;

    uint32_t elementCount() const OME_ALWAYS_INLINE
    {
        return (dataSet.size());
    }

    uint32_t isEmpty() const OME_ALWAYS_INLINE
    {
        return (dataSet.empty());
    }

    bool elementExists(const OMEbaseType &elem) const;

    template <typename STREAMTYPE> STREAMTYPE &outputOnStream(STREAMTYPE &outputStream,
            int_fast16_t indent = 0, uint8_t includeTypePrefix = OME_DEFAULT_COMPLEX_OUTPUT_MODE) const;
}; // end class OMEsetStorage

class OME_DLL_EXPORT OMEset final : public OMEreferenceToData<OMEsetStorage>
{
private:
    _STD deque<OMEbaseTypeP>::const_iterator *iterator;


    explicit OMEset(OMEsetStorage *s)
    {
        // only used by deepCopy()
        iterator = nullptr;
        data = s;
    }

public:
    // default initializer
    OMEset()
    {
        iterator = nullptr;
        data = new OMEsetStorage();
    }

    // copy initializer
    explicit OMEset(const OMEset &org) : OMEreferenceToData<OMEsetStorage>(org.data)
    {
        iterator = nullptr;
//        data = org.data;
//        addReadOnlyReference();
    }

    ~OMEset()
    {
        OME_DELETE_OBJECT(iterator);
    }

    // ASSIGNMENT
    OMEset &operator=(const OMEset &arg)
    {
        if (this->data == arg.data) {
            std::cerr << "OMEset assignment to self, ref=" << totalReferences() << "\n";
            return (*this); // asignment to self...
        }
        arg.data->addReadOnlyReference();
        OME_DELETE_OBJECT(iterator);
        iterator = nullptr;

        dropReference(arg.data);
        return (*this);
    }

    // COMPARISON
    bool operator==(const OMEset &arg) const
    {
        if (data == arg.data) {
            return (true); // fast path
        }
        return (*data == *arg.data);
    }

    bool operator!=(const OMEset &arg) const
    {
        if (data == arg.data) {
            return (false); // fast path
        }
        return (*data != *arg.data);
    }

    bool operator==(const OMEbaseType &arg) const;

    bool operator!=(const OMEbaseType &arg) const;

    OMEset *deepCopy() const
    {
        OMEsetStorage *storageCopy = static_cast<OMEsetStorage *>(data->deepCopy());
        OMEset *copy = new OMEset(storageCopy);
        return (copy);
    }

    OMEset &operator+=(const OMEbaseType &item)
    {
        getUniqueReference();
        OME_DELETE_OBJECT(iterator);
        iterator = nullptr;
        *data += item;
        return (*this);
    }

    OMEset &operator-=(const OMEbaseType &item)
    {
        getUniqueReference();
        OME_DELETE_OBJECT(iterator);
        iterator = nullptr;
        *data -= item;
        return (*this);
    }

    OMEset &operator|=(const OMEbaseType &item)
    {
        // set union
        getUniqueReference();
        OME_DELETE_OBJECT(iterator);
        iterator = nullptr;
        *data |= item;
        return (*this);
    }

    OMEset &operator&=(const OMEbaseType &item)
    {
        // set intersection
        getUniqueReference();
        OME_DELETE_OBJECT(iterator);
        iterator = nullptr;
        *data &= item;
        return (*this);
    }

    OMEset *operator+(const OMEbaseType &item) const
    {
        OMEset *result = new OMEset(*this);
        *result += item;
        return (result);
    }

    OMEset *operator-(const OMEbaseType &item) const
    {
        OMEset *result = new OMEset(*this);
        *result -= item;
        return (result);
    }

    OMEset *operator|(const OMEbaseType &item) const
    {
        // set union
        OMEset *result = new OMEset(*this);
        *result |= item;
        return (result);
    }

    OMEset *operator&(const OMEbaseType &item) const
    {
        // set intersection
        OMEset *result = new OMEset(*this);
        *result &= item;
        return (result);
    }

    OMEbaseType *nextElement(); // iterate

    void resetIterator(); // set back to beginning

    uint32_t elementCount() const OME_ALWAYS_INLINE
    {
        return (data->elementCount());
    }

    uint32_t isEmpty() const OME_ALWAYS_INLINE
    {
        return (data->isEmpty());
    }

    bool elementExists(const OMEbaseType &elem) const OME_ALWAYS_INLINE
    {
        return (data->elementExists(elem));
    }

    /*! \brief Output an OMEset to an output stream.
    * \param outputStream is a reference to an output stream.
    * \param indent specifies the number of spaces to indent before
    * beginning output.  If negative, no indent is performed
    * for the first line, but subsequent lines are indented
    * based on the absolute value of \a indent.
    * \param includeTypePrefix is a mask of flags indicating if type names
    * should prefix data elements, if strings should be enclosed in quotes,
    * etc.
    */
    template <typename STREAMTYPE> STREAMTYPE &outputOnStream(STREAMTYPE &outputStream, int_fast16_t indent = 0,
            uint8_t includeTypePrefix = OME_DEFAULT_COMPLEX_OUTPUT_MODE) const
    {
        data->outputOnStream(outputStream, indent, includeTypePrefix);
        return (outputStream);
    }
}; // end class OMEset

/*! \brief Write an OMEset to an output stream.
 */
template <typename STREAMTYPE> inline STREAMTYPE &operator<<(STREAMTYPE &outputStream, const OMEset &data)
{
    data.outputOnStream(outputStream, 0);
    return (outputStream);
}

/** @}
 */
#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
