#ifndef _OME_THREAD_H
//! \cond
#define _OME_THREAD_H "$Id: OMEthread.h 453 2020-07-23 20:20:44Z geoff $"
//! \endcond

#include <OMEruntime.h>
#include <OMEtype.h>
#include <OMEmethod.h>
#include <OMEclass.h>
#include <OMEobject.h>
#include <OMEthreadCntrl.h>
#include <OMEoid.h>

/** \defgroup OMEthread OME Thread Classes
 * @{
 */

enum OMEthreadStates {
    OME_THREAD_INIT,        //!< thread is being initialized
    OME_THREAD_SUSPEND,     //!< thread is suspended
	OME_THREAD_RUNNING,     //!< thread is active
    OME_THREAD_KILLED,      //!< thread has been killed
    OME_THREAD_TERMINATED   //!< thread is terminated
};


/*! \internal
 * \brief Represents the call stack of an OME thread.
 */
class OMEcallStack {
private:
	OMEtype			objectData;
public:
	const OMEinvocationData	*methodBody;
	int			    invocationLevel;
	OMEtype			thisMethod;
	OMEtype			argc;
	OMEtype			argv;
	uint32_t		programCounter;
	uint32_t		stackPointer;
	void			*threadStack;
	uint32_t		stackLength;
	OMEcallStack	*next;

	OMEcallStack(const OMEoid &obj, const OMEinvocationData *routine,
		int level, const OMEstring &methodNm, const uint32_t srcArgc,
		const OMEarray &srcArgv);

	~OMEcallStack();

	OME_FAST_CALL const OMEtype &getThisObject() const { return (objectData); }

	void dropThisObjectReference();

	bool validReferenceToObject() { return (objectData.type == OME_OID); }
}; // end class OMEcallStack

/*! \brief Public interface to an OME thread.
 */
class OMEthread final : public OMEobject {
friend class OMEobjectInstance;		// for access to constructor
friend class OMEthreadExecQueue;	// for access to destructor
private:
	OMEtype			*replyOID;
	OMEthreadController	*threadController;
	OMEoid			*notifyWhenDead;
	OMEtype			userInfo;
	bool			modifiedObject;
	bool			preemptable;
	bool			resultPending;
public:
	static void defineOMEthreadClass();

	OMEcallStack	*callStack;
	OMEthreadStates	threadState;
	int			    threadPriority;
	OMEtype			threadContext;
	OMEtype			fromObject;
	OMEtype			replyResult;
	OMEthread		*fromThreadInstance;
	OMEtype			threadErrorCode;
	// for debugging assistance...
	int			    sourceLineNumber;
	const char		*sourceFileName;
	class OMEtickInterval	*tickInterval;
private:
	// fromThread is used to extract threadContext information...
	OMEthread(const OMEinvocationData *routine, const int level,
		const OMEoid &obj,
		const OMEstring &methodName, const uint32_t srcArgc, 
		const OMEarray &srcArgv, const OMEtype *fromObj = nullptr,
		const OMEthread *fromThread = nullptr,
        const OMEoid *notifyObj = nullptr);

	OMEthread(const OMEstring &user, const OMEstring &pw);

	virtual ~OMEthread();

    void createReplyOID();

	OME_FAST_CALL void setReplyResult(const OMEtype &arg, bool sameThread = false);
public:

	// virtual from OMEobject
	virtual bool isThread() const override	{ return (true); }

	bool inCalledMethod() const OME_PURE_FUNCTION;

	inline void clearResultPending() OME_ALWAYS_INLINE { resultPending = false; }

	inline void setResultPending() OME_ALWAYS_INLINE { resultPending = true; }

	inline bool replyStillPending() const OME_ALWAYS_INLINE { return (resultPending); }

    inline void setTickInterval(class OMEtickInterval *interval) OME_ALWAYS_INLINE { tickInterval = interval; }

	inline void noteThreadState(const OMEthreadStates s) OME_ALWAYS_INLINE { threadState = s; }

	inline OMEthreadStates getThreadState() const OME_ALWAYS_INLINE { return (threadState); }

	inline void noteThreadPriority(int p) OME_ALWAYS_INLINE { threadPriority  = p; }

	inline int getThreadPriority() const OME_ALWAYS_INLINE { return (threadPriority); }

	const OMEtype &getReplyOID() {
        if (OME_EXPECT_FALSE(replyOID == nullptr)) {
            createReplyOID();
        }
        return (*replyOID);
    }

	virtual OME_FAST_CALL OMEthread *invokeMethod(int *errRet, const OMEstring &methodName,
		const uint32_t hashIndx, const OMEoid &objRefData,
		const uint32_t argc, const OMEarray &argv, const uint32_t signature,
		const OMEtype *fromObject = nullptr,
        const OMEthread *fromThread = nullptr,
		const OMEoid *notify = nullptr, const int inheritanceLevel = -1,
		const int mustBeAtLevel = -1) override;

	OME_FAST_CALL void callRoutine(const OMEinvocationData *routine, const int level,
		const OMEstring &methodName, const int srcArgc, 
		const OMEarray &srcArgv);

	void exitRoutine();

	const OMEtype &getUserID() const OME_ALWAYS_INLINE { return (userInfo); }

	const OMEstring *getUserName() const;

	OME_FAST_CALL int setUserID(const OMEstring &userID, const OMEstring &pwInfo);

	const OMEtype &getThisObject() const OME_ALWAYS_INLINE { return (callStack->getThisObject()); }

	const OMEoid &getThisOID() const OME_ALWAYS_INLINE {
		return (*(callStack->getThisObject().value.oid));
	}

		
	void dropThisObjectReference() OME_ALWAYS_INLINE { callStack->dropThisObjectReference(); }

	bool validReferenceToObject() OME_ALWAYS_INLINE { return (callStack->validReferenceToObject()); }

	OME_FAST_CALL void *getInstanceData() const;

	OME_FAST_CALL void setPC(uint32_t pc) OME_ALWAYS_INLINE {
		callStack->programCounter = pc;
	}

	uint32_t getPC() const OME_ALWAYS_INLINE {
		return (callStack->programCounter);
	}

	void setErrorCode(OMEtype &rc) OME_ALWAYS_INLINE { threadErrorCode = rc; }

	OMEtype &getErrorCode() OME_ALWAYS_INLINE { return (threadErrorCode); }

	const OMEtype &getReplyResult() const OME_ALWAYS_INLINE { return (replyResult); }

	OME_FAST_CALL int returnReply(const OMEtype &result);

	OME_FAST_CALL int returnReply(const uint32_t result);

	OME_FAST_CALL int returnReply(const int32_t result);

	OME_FAST_CALL int returnReply(const float result);

	OME_FAST_CALL int returnReply(const OMEoid &result);

	OME_FAST_CALL void threadNowTerminated(uint32_t exitStatus, bool dontDelete = false);

	void noteObjectModified() OME_ALWAYS_INLINE { modifiedObject = true; }

	inline void noteLineNumber(int line, const char *fileName = nullptr) OME_ALWAYS_INLINE {
		sourceLineNumber = line;
		if (OME_EXPECT_TRUE(fileName != nullptr)) {
			sourceFileName = fileName;
		}
	}

	void setAsPreemptable();

	OMEthread *releaseThread();

	void suspendThread();

	void terminateThread();

	void yieldSlice();

	void executeSlice();

	uint32_t getMethodType() const OME_ALWAYS_INLINE {
		if (OME_EXPECT_TRUE(threadController != nullptr)) {
			return(callStack->methodBody->invocationMode);
		}
		return (0);	// unknown...
	}

	OME_FAST_CALL int sendMessage(const OMEstring &methodName,
        const uint32_t hashIndx,
		const uint32_t argc, const OMEarray &argv,
		const OMEtype &destObj,
		const OMEtype *fromObj = nullptr) const;

	// convenience function for use by OIL2 methods
	OME_FAST_CALL int sendMessage(const OMEtype &methodName,
		const uint32_t hashIndx, const OMEtype &argv,
		const OMEtype &destObj, const OMEtype &fromObj,
		const OMEtype &timeout);

	OME_FAST_CALL int sendMessageWithSetArgument(const OMEtype &methodName,
		const uint32_t hashIndx,
		const OMEtype &argSet, const OMEtype &destObj,
		const OMEtype &fromObj, const int32_t timeout);

	OME_FAST_CALL int sendMessageAndWait(const OMEstring &methodName,
		const uint32_t hashIndx, const uint32_t argc,
		const OMEarray &argv, const OMEtype &destObj);

	OME_FAST_CALL int callMethodWithSetArgument(const OMEtype &methodName,
		const uint32_t hashIndx, const OMEtype &argSet);

	OME_FAST_CALL int generateException(uint_fast32_t argc, const OMEarray &argv) const;

	// convenience interfaces...
	OME_FAST_CALL int generateException(const char *exceptionType) const;

	OME_FAST_CALL int generateException(const char *exceptionType,
		const OMEstring &methName) const;

	void outputOnStream(std::ostream &o=std::cout) const;
}; // end class OMEthread

typedef OME_CORE_CALL int (*zeroFP)(OMEthread *, OMEtype &);

typedef OME_CORE_CALL int (*oneFP)(OMEthread *, OMEtype &, const OMEtype &);

typedef OME_CORE_CALL int (*twoFP)(OMEthread *, OMEtype &, const OMEtype &,
	const OMEtype &);

typedef OME_CORE_CALL int (*threeFP)(OMEthread *, OMEtype &, const OMEtype &,
	const OMEtype &, const OMEtype &);

typedef OME_CORE_CALL int (*fourFP)(OMEthread *, OMEtype &, const OMEtype &,
	const OMEtype &, const OMEtype &, const OMEtype &);

typedef OME_CORE_CALL int (*fiveFP)(OMEthread *, OMEtype &, const OMEtype &,
	const OMEtype &, const OMEtype &, const OMEtype &, const OMEtype &);

typedef OME_CORE_CALL int (*sixFP)(OMEthread *, OMEtype &, const OMEtype &,
	const OMEtype &, const OMEtype &, const OMEtype &, const OMEtype &,
	const OMEtype &);

typedef OME_CORE_CALL int (*sevenFP)(OMEthread *, OMEtype &, const OMEtype &,
	const OMEtype &, const OMEtype &, const OMEtype &, const OMEtype &,
	const OMEtype &, const OMEtype &);

typedef OME_CORE_CALL int (*eightFP)(OMEthread *, OMEtype &, const OMEtype &,
	const OMEtype &, const OMEtype &, const OMEtype &, const OMEtype &,
	const OMEtype &, const OMEtype &, const OMEtype &);

typedef OME_CORE_CALL int (*atLeastOneFP)(OMEthread *, OMEtype &,
	const OMEtype &, ...);

/*! \brief Record to define the implementation of an OIL2 function.
 */
class OMEdefOIL2func {
private:
	static OMEdefOIL2func	*listStart;
	OMEdefOIL2func	*next;

	inline void registerFunction() {
		next = listStart;
		listStart = this;
	}
public:
	static OME_FAST_CALL const OMEdefOIL2func *findFunction(const OMEstring &fName);

	OMEstring	functionName;
	void		*function;
	int		argCount;

	OMEdefOIL2func(const char *funcName, zeroFP f, int args) : functionName(funcName) {
//		functionName = funcName;
		function = (void *) f;
		argCount = args;
		registerFunction();
	}

	OMEdefOIL2func(const char *funcName, oneFP f, int args) : functionName(funcName) {
//		functionName = funcName;
		function = (void *) f;
		argCount = args;
		registerFunction();
	}

	OMEdefOIL2func(const char *funcName, twoFP f, int args) : functionName(funcName) {
//		functionName = funcName;
		function = (void *) f;
		argCount = args;
		registerFunction();
	}

	OMEdefOIL2func(const char *funcName, threeFP f, int args) : functionName(funcName) {
//		functionName = funcName;
		function = (void *) f;
		argCount = args;
		registerFunction();
	}

	OMEdefOIL2func(const char *funcName, fourFP f, int args) : functionName(funcName) {
//		functionName = funcName;
		function = (void *) f;
		argCount = args;
		registerFunction();
	}

	OMEdefOIL2func(const char *funcName, fiveFP f, int args) : functionName(funcName) {
//		functionName = funcName;
		function = (void *) f;
		argCount = args;
		registerFunction();
	}

	OMEdefOIL2func(const char *funcName, sixFP f, int args) : functionName(funcName) {
//		functionName = funcName;
		function = (void *) f;
		argCount = args;
		registerFunction();
	}

	OMEdefOIL2func(const char *funcName, sevenFP f, int args) : functionName(funcName) {
//		functionName = funcName;
		function = (void *) f;
		argCount = args;
		registerFunction();
	}

	OMEdefOIL2func(const char *funcName, eightFP f, int args) : functionName(funcName) {
//		functionName = funcName;
		function = (void *) f;
		argCount = args;
		registerFunction();
	}

	~OMEdefOIL2func()	{}
}; // end class OMEdefOIL2func

OME_FAST_CALL OMEtype &OMEgetObjectCreatorOID(const OMEtype &userInfo) OME_CONST_FUNCTION; // in OMEcreate.cpp

OME_FAST_CALL OMEtype &OILnextIterator(int *rc, OMEtype &setVar); // special runtime

OME_FAST_CALL bool OMEokToAssignTypes(uint32_t sourceType, uint32_t targetType, OMEthread *t = nullptr);

// call at exit to cleanup
void unregisterAllServices();

/** @}
 */
#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
