#ifndef _ASSIGN_TO_ZERO_HPP_
//! \cond
#define _ASSIGN_TO_ZERO_HPP_ "$Id: assignToZero.hpp 454 2020-07-23 20:22:23Z geoff $"
//! \endcond

/** \addtogroup FARGOSbase
 * @{
 */


// subgroup
/** \defgroup assignToZero Set Value of Element to Zero
 *
 * \brief The functions within this group are used to assign a value of 0
 * to different numerical types.  Used in templated code where the nature
 * of the type to be used is unknown and might be a vector element instead
 * of a trivial fundamental type.
 *
 * For most of these inlined routines, the code amounts to nothing
 * more than:
 * \code{.cpp}
 * n = 0;
 * \endcode
 *
 * While the use of inlines prevents incurring actual function call overhead,
 * the programmer effort involved in typing \b assignToZero(n)
 * rather than simply \b n=0 is much greater.
 * The reason for assignToZero()'s existence is for usage
 * within templated routines that might need to deal with non-scalar
 * fundamental types, such as a 4-element vector of integers or a 64-element
 * vector of doubles.
 *
 * \note Although the use of a reference argument would seem to force
 * the element of interest into main memory, this is compensated for by
 * having forced the routines to always be inlined.  In reality, their
 * true utility arises from assigning registers.
 */

/** \addtogroup assignToZero
 * @{
 */
/* implementations of assignToZero for common
 * fundamental signed numeric types
 */

//! \brief Assigns 0 to a 32-bit integer
extern void assignToZero(int32_t &n) OME_ALWAYS_INLINE;
inline void assignToZero(int32_t &n) {
    n = 0;
}

//! \brief Assigns 0 to an 64-bit integer
extern void assignToZero(int64_t &n) OME_ALWAYS_INLINE;
inline void assignToZero(int64_t &n) {
    n = 0;
}

//! \brief Assigns 0 to a float
extern void assignToZero(float &n) OME_ALWAYS_INLINE;
inline void assignToZero(float &n) {
    n = 0;
}

//! \brief Assigns 0 to a double
extern void assignToZero(double &n) OME_ALWAYS_INLINE;
inline void assignToZero(double &n) {
    n = 0;
}
/** @} */

/** @} */

#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
