#ifndef _COMPILER_HINTS_H_
//! \cond
#define _COMPILER_HINTS_H_ "$Id: compiler_hints.h 472 2020-07-25 21:03:30Z geoff $"
//! \endcond

/* Copyright (C) 2001 - 2019 FARGOS Development, LLC
 * This file is useable by C and C++ programs; avoid C++-specific features
 * like "//" comments.
 */
/*! \file
 * \brief Compiler-specific macros to provide performance-related hints.
 *
 * \note This file contains compiler-specific macros to provide hints
 * for higher-performance.  Correctness should not be affected if a compiler
 * fails to provide support for a given feature.
 *
 * To generate list of predefined macros:
 * \code
 *   $(CXX) -dM -E -x c /dev/null
 * \endcode
 * \copyright 2001 - 2019 FARGOS Development, LLC
 */

/*! \def OME_DEPRECATED_BECAUSE
 * \brief Mark interface as deprecated and provide reason or
 * suggested alternative.
 *
 * \param msg specifies a string of informational content to be emitted
 * with the warning message
 */

/*! \def OME_DEPRECATED
 * \brief Mark interface as deprecated
 *
 * \note In general, one should use OME_DEPRECATED_BECAUSE()
 * instead to provide an explanation as to why the function or variable
 * has been deprecated and suggest an alternative.
*/

/*! \def OME_USAGE_WARNING
 * \brief Emit a warning if the element is used.
 *
 * \param msg specifies a string of informational content to be emitted
 * with the warning message
 *
 * \sa OME_USAGE_ERROR
 */

/*! \def OME_USAGE_ERROR
 * \brief Emit an error if the element is used.
 *
 * \param msg specifies a string of informational content to be emitted
 * with the error message
 *
 * \sa OME_USAGE_WARNING
 */


/*! \def NONNULL_RETURN
 * \brief Mark a function as never returning a null pointer.
 *
 * \sa NONNULL_PARAMETERS
 * \sa NONNULL_CLASS_PARAMETERS
 */

/*! \def OME_EXPECT
 * \brief Annotation macro for conditional expression to
 * indicate expected Boolean outcome.
 *
 * \param expr specifies the expression being evaluated
 * \param expected is a Boolean that indicates the expected
 * result of the expression
 *
 * Useful to give the compiler a hint as to the expected outcome
 * of an conditional expression to assist with layout of the object code.
 *
 * \sa OME_EXPECT_TRUE
 * \sa OME_EXPECT_FALSE
 */

/*! \def OME_EXPECT_FALSE
 * \brief Annotation macro for conditional expression expected
 * to be false.
 *
 * \param expr specifies the expression being evaluated
 */

/*! \def OME_EXPECT_TRUE
 * \brief Annotation macro for conditional expression expected
 * to be true.
 *
 * \param expr specifies the expression being evaluated
 */

/*! \def OME_UNUSED
 * \brief Tell the compiler it is known that an element is not used.
 *
 * This rarely used attribute is intended to silence compiler complaints.
 */

/*! \def OME_USED
 * \brief Tell the compiler an element is used to prevent it from
 * being optimized away.
 */

/*! \def NONNULL_PARAMETERS
 * \brief Mark parameters to a function as not permitting null pointers.
 *
 * The arguments to this macro are a comma-separated list of integers,
 * starting from 1, that identify the parameter positions of interest.
 *
 * \sa NONNULL_CLASS_PARAMETERS
 */

/*! \def NONNULL_CLASS_PARAMETERS
 * \brief Mark a function as never returning a null pointer.
 *
 * The arguments to this macro are a comma-separated list of integers,
 * starting from 2, that identify the parameter positions of interest.
 * Parameter position 1 refers to the \c this pointer.
 *
 * \sa NONNULL_PARAMETERS
 */

/*! \def OME_ALWAYS_INLINE
 * \brief Tell the compiler to alway inline a function, regardless of
 * optimization level.
 *
 * This attribute is best used with trivial routines for which it would be
 * a waste to ever incur the overhead of a function call,
 * even in unoptimized code.
 *
 * \sa OME_ALWAYS_OPTIMIZE
 */

/*! \def OME_NEVER_INLINE
 * \brief Tell the compiler to not inline a function, regardless of
 * optimization level.
 *
 * This attribute is best used with routines for which there should be
 * a single implementation.
 */

/*! \def OME_ALWAYS_OPTIMIZE
 * \brief Mark a function to be compiled with a specific level of
 * optimization.
 *
 * \param level is a string that specifies the optimization level,
 * such as \c "-O3".
 *
 * \note Usually this is used to force high-levels of optimization
 * on core functions, but it can be used to reduce optimization.
 *
 * \sa OME_ALWAYS_INLINE
 */

/*! \def OME_PREFETCH
 * \brief Macro to request prefetch
 *
 * \param addr is the address of the memory block
 * \param rw is 0 for read-only, 1 for to be updated
 * \param locality is an integer value 0 to 3, where 0 means no further use 
 * and 3 for retain if at all possible.
 */

/*! \def DO_PREFETCH
 * \brief Prefetch an address line for read.
 *
 * \param a specifies the address line to be fetched
 *
 * \sa OME_PREFETCH
 */


/*! \def OME_ALWAYS_FLATTEN
 * \brief Annotation macro to request compiler to always inline
 * body and any called routines
 *
 * \sa OME_ALWAYS_INLINE
 */

/*! \def OME_COLD_ROUTINE
 * \brief Annotation macro to tell compiler a routine may never be used
 */

/*! \def OME_INIT_PRIORITY
 * \brief Override initialization priority of a C++ variable.
 *
 * \param n specifies the priority; lower values have higher priority.
 */

/*! \def OME_WEAK
 * \brief Force generation of weak symbol
 */

/*! \def OME_PURE_FUNCTION
 * \brief Mark as an idempotent function that can access global variables.
 *
 * For a given set of argument values, the function must always return
 * the same value.  It may access data other than that provided by the
 * arguments, such as looking up an entry in table, but the external
 * data cannot have a variable influence on the result to be returned.
 * The intent is to indicate to the compiler when a single call to the routine
 * can be made; this is used for common sub-expression elimination and
 * potentially hoisting a call out of a loop body.
 *
 * \warning A common mistake is to declare as pure a function that takes
 * an argument which is a pointer to a memory address that remains
 * constant but the pointed-at content changes between calls.
 * A good example would be an input buffer: the buffer address does not
 * change, but new content read from disk, the network or the keyboard
 * does alter the contents of the buffer.
 * The function is not idempotent because
 * the result varies depending on the contents of the pointed-at buffer.
 */

/*! \def OME_CONST_FUNCTION
 * \brief Mark as an idempotent function that \b only accesses arguments --
 * no global data.
 *
 * This is similar to declaring a C++ function as \c constexpr.
 * Few functions will qualify for this treatment.
 *
 * \sa OME_PURE_FUNCTION
 */

/*! \def AN_ENUM
 * \brief Label an enum expression to avoid ambiguity
 */

/*! \def OME_YIELD_THREAD
 * \brief Macro for platform-specific yield of thread's time slice.
 */

/*! \def OME_PREFETCH_OBJECT_VTABLE
 * \brief Prefetch the v-table entry for an object.
 *
 * \param obj is the pointer to the object.
 * \param relativeFunction is an integer corresponding
 * to the relative index of the virtual function associated
 * with the class.
 *
 * Sometimes the easiest mechanism to determine the \a relativeFunction
 * value is by disassembling the source.  An alternative would be to
 * scan the v-table for the function of interest to derive
 * the relative index.
 *
 * \note While the value of the v-table pointer will be identical among all
 * of an object's peers, the v-table pointer itself will be local to
 * each object that has virtual functions.  Prefetching the object
 * pointer itself should be sufficient to obtain the v-table
 * pointer, but it does not perform the needed indirection.
 *
 * \sa DO_PREFETCH
 */

/*! \def VIRTUAL_OVERRIDE
 * \brief Generates \c override if the compiler supports it.
 */

/*! \def CONSTEXPR
 * \brief Generates \c constexpr if the compiler supports it.
 */

/*! \def EXPLICIT_CONVERSION 
 * \brief Generates \c explicit if the compiler supports it.
 */

/*! \def NOEXCEPT_SPECIFIER
 * \brief Generates \c noexcept if the compiler supports it.
 */

#ifdef __cplusplus
#  include <typeinfo>
#  include <stdlib.h> /* needed for free() */
#endif

#ifndef _WIN32
#    include <unistd.h> /* needed for correct selection of thread yield */
#endif
#ifndef _MSC_VER /* Not Microsoft Visual C++ */
#    include <cxxabi.h>
#endif
#include <string.h>
#include <stdint.h>

#ifdef _MSC_VER /* Microsoft Visual C++ */
#    ifndef _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES
#    define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1 /* For Windows, rename standard routines */
#    endif
#    ifndef _CRT_SECURE_NO_WARNINGS
#    define _CRT_SECURE_NO_WARNINGS 1
#    endif
#    ifndef _CRT_NONSTDC_NO_WARNINGS
#    define _CRT_NONSTDC_NO_WARNINGS 1
#    endif

#    define OME_FAST_CALL __fastcall

#    pragma warning(disable : 4996) /* complaints regarding deprecated POSIX routines */

#endif

#if __clang__ > 0 /* clang */
/* clang supports a limited number of gcc attributes
*/
#    define OME_DEPRECATED __attribute__((deprecated))
#    define OME_DEPRECATED_BECAUSE(msg) __attribute__((deprecated(msg)))
#    define OME_USAGE_WARNING(msg) __attribute__((warning(msg)))
#    define OME_USAGE_ERROR(msg) __attribute__((error(msg)))
#    define CONSTEXPR constexpr
#    define AN_ENUM(x) x
/* Could be __Nonnull */
#    define NONNULL_PARAMETERS(...) __attribute__((nonnull(__VA_ARGS__)))
/* flag for now -- "this" pointer is hidden parameter #1 */
#    define NONNULL_CLASS_PARAMETERS(...) __attribute__((nonnull(__VA_ARGS__)))
/* Could be __Nonnull */
#    define NONNULL_RETURN __attribute__((returns_nonnull))

#    define OME_USED __attribute__((used))
#    define OME_ALWAYS_INLINE __attribute__((always_inline))
#    define OME_INIT_PRIORITY(n) __attribute__((init_priority(n)))
#    define OME_PURE_FUNCTION __attribute__((pure))
#    define OME_CONST_FUNCTION __attribute__((const))
#    define OME_EXPECT(expr, expected) __builtin_expect((expr), expected)

#    define OME_PREFETCH(addr, rw, locality) __builtin_prefetch(addr, rw, locality)

/* not really the effect we want; silences compiler complaints
   * rather than forcing symbol to be retained */
#    define OME_UNUSED __attribute__((unused))

#elif __GNUC__ > 0 /* GNU C++ Compiler */

#    define NONNULL_PARAMETERS(...) __attribute__((nonnull(__VA_ARGS__)))
/* flag for now -- "this" pointer is hidden parameter #1 */
#    define NONNULL_CLASS_PARAMETERS(...) __attribute__((nonnull(__VA_ARGS__)))

#    define NONNULL_RETURN __attribute__((returns_nonnull))

#    ifdef HAVE_GCCVISIBILITYPATCH
#    define OME_DLL_EXPORT __attribute__((visibility("default")))
#    define OME_DLL_LOCAL __attribute__((visibility("hidden")))
#    endif

#    ifdef _MSC_VER /* Microsoft Visual C++ */

#    ifdef BUILDING_DLL
#        define OME_DLL_EXPORT __declspec(dllexport)
#    else
#        define OME_DLL_EXPORT __declspec(dllimport)
#    endif
#    define OME_DLL_LOCAL
//#define OME_FAST_CALL __fastcall
#    endif // _MSC_VER

/* original check:  ifndef __MIPSEL__ and ifndef sun
 */

#    define OME_EXPECT(expr, expected) __builtin_expect((expr), expected)

#    define OME_PREFETCH(addr, rw, locality) __builtin_prefetch(addr, rw, locality)

#    define OME_ALWAYS_INLINE __attribute__((always_inline))

#    define OME_NEVER_INLINE __attribute__((noinline))

#    define OME_ALWAYS_FLATTEN __attribute__((flatten))

#    define OME_COLD_ROUTINE __attribute__((cold))

#    define OME_INIT_PRIORITY(n) __attribute__((init_priority(n)))

#    define OME_DEPRECATED __attribute__((deprecated))

#    define OME_USAGE_WARNING(msg) __attribute__((warning(msg)))

#    define OME_USAGE_ERROR(msg) __attribute__((error(msg)))

#    define OME_USED __attribute__((used))

#    define OME_UNUSED __attribute__((unused))

/*! \brief Force generation of weak symbol
 */
#define OME_WEAK __attribute__((weak))

#    define OME_PURE_FUNCTION __attribute__((pure))

#    define OME_CONST_FUNCTION __attribute__((const))

#    define AN_ENUM(x) x

/* Note: 64-bit API already uses registers rather than the stack, so this
 * should be inhibited or at least adjusted for 64-bit compilation
 */
#    if __LP64__ == 0
#if __i386__
#    define OME_FAST_CALL __attribute__((__cdecl__)) __attribute__((__regparm__(3)))
#endif
#    endif

#    if (__GNUC__ > 4) || ((__GNUC__ == 4) && (__GNUC_MINOR__ >= 7))
#    define OME_DEPRECATED_BECAUSE(msg) __attribute__((deprecated(msg)))

#    define OME_ALWAYS_OPTIMIZE(level) __attribute__((optimize(level)))
#    else
/* compiler is too old... */
#    define OME_DEPRECATED_BECAUSE(msg) __attribute__((deprecated))
#    endif

#endif /* end GNUC */

#ifdef _MSC_VER /* Microsoft Visual C++ */
/* override was present in Microsoft Visual C++ 2005 */
#    if _MSC_VER >= 1400
#    define VIRTUAL_OVERRIDE override
#    endif
/* constexpr recognized in Visual Studio 2015 */
#    if _MSV_VER >= 1900
#    define CONSTEXPR constexpr
#    endif
/*! \brief Label an enum expression to avoid ambiguity */
#    define AN_ENUM(x) ((uint32_t)(x))
#endif

#if __cplusplus >= 201100
#    ifndef EXPLICIT_CONVERSION
#    define EXPLICIT_CONVERSION explicit
#    endif
#    ifndef CONSTEXPR
#    define CONSTEXPR constexpr
#    endif
#    ifndef VIRTUAL_OVERRIDE
#    define VIRTUAL_OVERRIDE override
#    endif
#    ifndef NOEXCEPT_SPECIFIER
#    define NOEXCEPT_SPECIFIER noexcept
#    endif
#endif

/* If noexcept keyword is not supported, then hide it */
#ifndef NOEXCEPT_SPECIFIER
#    define NOEXCEPT_SPECIFIER
#endif
/* If override keyword is not supported, then hide it */
#ifndef VIRTUAL_OVERRIDE
#    define VIRTUAL_OVERRIDE
#endif
/* If explicit keyword is not supported, then hide it */
#ifndef EXPLICIT_CONVERSION
#    define EXPLICIT_CONVERSION
#endif
/* If constexpr keyword is not supported, then use default or const */
#ifndef CONSTEXPR
#    ifdef _GLIBCXX_USE_CONSTEXPR
#    define CONSTEXPR _GLIBCXX_USE_CONSTEXPR /* use whatever the config wants */
#    else
#    define CONSTEXPR const
#    endif
#endif /* CONSTEXPR */

#ifndef OME_DLL_EXPORT
#    define OME_DLL_EXPORT /* nothing */
#endif

#ifndef OME_FAST_CALL
#    define OME_FAST_CALL /* nothing */
#endif

#ifndef OME_CORE_CALL
#    define OME_CORE_CALL /* nothing */
#endif

#ifndef OME_EXPECT
#    define OME_EXPECT(expr, expected) (expr)
#endif
#ifndef OME_PREFETCH
#    define OME_PREFETCH(addr, rw, locality) /* nothing -- make it go away */
#endif

#ifndef OME_FAST_CALL
#    define OME_FAST_CALL
#endif
#ifndef OME_ALWAYS_INLINE
#    define OME_ALWAYS_INLINE
#endif
#ifndef OME_NEVER_INLINE
#    define OME_NEVER_INLINE
#endif
#ifndef OME_ALWAYS_FLATTEN
#    define OME_ALWAYS_FLATTEN
#endif
#ifndef OME_COLD_ROUTINE
#    define OME_COLD_ROUTINE
#endif
#ifndef OME_ALWAYS_OPTIMIZE
#    define OME_ALWAYS_OPTIMIZE(level)
#endif
#ifndef OME_INIT_PRIORITY
#    define OME_INIT_PRIORITY(n)
#endif
#ifndef OME_DEPRECATED
#    define OME_DEPRECATED
#endif
#ifndef OME_DEPRECATED_BECAUSE
#    define OME_DEPRECATED_BECAUSE(msg)
#endif
#ifndef OME_USAGE_WARNING
#    define OME_USAGE_WARNING(msg)
#endif
#ifndef OME_USAGE_ERROR
#    define OME_USAGE_ERROR(msg)
#endif
#ifndef OME_USED
#    define OME_USED
#endif
#ifndef OME_UNUSED
#    define OME_UNUSED
#endif
#ifndef OME_WEAK
#define OME_WEAK
#endif
#ifndef OME_PURE_FUNCTION
#    define OME_PURE_FUNCTION
#endif
#ifndef OME_CONST_FUNCTION
#    define OME_CONST_FUNCTION
#endif
#ifndef NONNULL_PARAMETERS
#    define NONNULL_PARAMETERS(...)
#endif
#ifndef NONNULL_CLASS_PARAMETERS
#    define NONNULL_CLASS_PARAMETERS(...)
#endif
#ifndef NONNULL_RETURN
#    define NONNULL_RETURN
#endif

#define OME_EXPECT_FALSE(expr) OME_EXPECT(expr, false)
#define OME_EXPECT_TRUE(expr) OME_EXPECT(expr, true)

#ifdef _WIN32
#    define OME_YIELD_THREAD() SwitchToThread()
#else
#    ifdef _POSIX_PRIORITY_SCHEDULING
#    define OME_YIELD_THREAD() sched_yield()
#    endif /* _POSIX_PRIORITY_SCHEDULING */
#endif     /* _WIN32 */

#ifndef OME_YIELD_THREAD /* not yet defined... */
/*! \brief Macro for platform-specific yield of thread's time slice.
 */
#    define OME_YIELD_THREAD() \
    do { \
        struct timespec nano_delay = {0, 0}; \
        nanosleep(&nano_delay, 0); \
    } while (0)
#endif

/* If desired to use CPU prefetch instructions, enable the following: */
#ifdef __GNUC__ /* GNU C++ Compiler */
#    define USE_GCC_PREFETCH 1
#ifdef MemoryBarrier
#undef MemoryBarrier
#endif
#    define MemoryBarrier() __sync_synchronize()
#else
#    define USE_GCC_PREFETCH 0
#endif

/*! \brief Definition for target system's cache line length.
 *
 * Note:  Assume 64 bytes per cache line, which is appropriate for modern CPUs.
 */
#define CACHE_LINE_LENGTH 64

/*! \brief Maximum number of prefetch requests supported by the hardware.
 */
#define MAX_PREFETCH_REQUESTS 8

#ifndef DO_PREFETCH
#    if USE_GCC_PREFETCH == 1
/* use CPU prefetch instructions */
#    define DO_PREFETCH(a) OME_PREFETCH(a, 0, 1)
#    else
/* trigger read of cache line */
#    define DO_PREFETCH(a) p_fetch += *a
#    endif

#    ifndef _MSC_VER
#    define OME_PREFETCH_OBJECT_VTABLE(obj, relativeFunction) \
    do {                            /* open scope */ \
        unsigned char ***vTablePtr = (unsigned char ***) (obj); /* get hidden v-table pointer */ \
        unsigned char **vTableBase = *vTablePtr;        /* read v-table pointer */ \
        unsigned char **funcPtrAddr = vTableBase + relativeFunction; \
        OME_PREFETCH(funcPtrAddr, 0, 3); /* read-only, retain */ \
    } while (0)
#    else
#    define OME_PREFETCH_OBJECT_VTABLE(obj, relativeFunction) \
    do { \
    } while (0)
#    endif

extern inline int prefetch_memory_block(const void *addr,
                                        const unsigned int len) OME_ALWAYS_INLINE;

/*! \brief Prefetch a memory block into the CPU's cache.
 *
 * \param addr points to the base of the memory block
 * \param len is the number of bytes in the block
 *
 * Note: the AMD Opteron/Athlon's have 64-byte cache lines and
 * can perform a maximum of 8 outstanding prefetch requests.
 * These CPU-specific attributes are represented by the use of the
 * CACHE_LINE_LENGTH and MAX_PREFETCH_REQUESTS.  With their normal values,
 * a maximum amount of 64 * 8 = 512 bytes can be prefetched by this call.
 *
 * \return The return value is undefined and logically this routine should be
 * thought of as being of type void. If compiler-specific directives are
 * used to perform the prefetching, then a constant value of zero is returned;
 * otherwise the return value is derived from words used to trigger the
 * reads of various cache lines.
 */
inline int prefetch_memory_block(const void *addr,
                                 const unsigned int len)
{
    enum {
        CACHE_LINE_LENGTH_IN_WORDS = (CACHE_LINE_LENGTH / sizeof(int_fast32_t))
    };
    int cache_bytes = len - sizeof(int_fast32_t);
    if (cache_bytes <= 0) {
        return (0);    /* not enough to cache */
    }
    int cache_lines = cache_bytes / CACHE_LINE_LENGTH;
    if (cache_lines >= MAX_PREFETCH_REQUESTS) {
        cache_lines = MAX_PREFETCH_REQUESTS - 1;
    }

#    if USE_GCC_PREFETCH == 0
    int p_fetch = 0;
#    endif
    /* cast as integer pointer for speed */
    const int_fast32_t *a = (const int_fast32_t *)(addr);

    /* With a switch() statement using a dense set of values, most compilers
     * should generate an efficient jump table.
     * The various cases fall through to the subsequent case, which
     * serves to unroll the loop.
     */
    switch (cache_lines) {
    default:
    case 7:
        DO_PREFETCH(a);
        a += CACHE_LINE_LENGTH_IN_WORDS;
    case 6:
        DO_PREFETCH(a);
        a += CACHE_LINE_LENGTH_IN_WORDS;
    case 5:
        DO_PREFETCH(a);
        a += CACHE_LINE_LENGTH_IN_WORDS;
    case 4:
        DO_PREFETCH(a);
        a += CACHE_LINE_LENGTH_IN_WORDS;
    case 3:
        DO_PREFETCH(a);
        a += CACHE_LINE_LENGTH_IN_WORDS;
    case 2:
        DO_PREFETCH(a);
        a += CACHE_LINE_LENGTH_IN_WORDS;
    case 1:
        DO_PREFETCH(a);
        a += CACHE_LINE_LENGTH_IN_WORDS;
    case 0:
        DO_PREFETCH(a);
    }
#    if USE_GCC_PREFETCH == 1
    return (0);
#    else
    return (p_fetch);
#    endif
}
#endif /* #ifndef DO_PREFETCH */


/*! \brief Safe \c strcpy() routine that will not copy more than \c l bytes
 * and always ensures that a null is present to mark the end of the string.
 *
 * \param d speciries the destination to which the string will be copied
 * \param s specifies the source of the string content
 * \param l specifies the length of the destination
 *
 * \retval 0 indicates the string had to be truncated
 * \retval 1 indicates the string was copied in its entirety.
 */
#define safe_strcpy(d, s, l) ((memccpy(d, s, 0, (l)) == 0) ? (*(d + (l) -1) = 0) : 1)

#ifdef __cplusplus

/*! \brief Templated C++-only routine similar to \c memccpy() that is
 * suitable for handling short strings that are being copied into buffers
 * whose size is known at compile time.  The maximum number of characters
 * to be copied from the source is specified as the template parameter.
 *
 * When used as intended, the tail recursion will be
 * flattened into a sequence of test and jumps and register pressure
 * will be reduced.  The number of instructions generated will be
 * proportional to the length of the string.  On hardware that supports
 * a load-from-immediate-offset, no register is needed to traverse
 * the length of the string, nor do the pointers to the source and
 * destination need to be modified.
 *
 * \arg N is the template parameter that specifies the maximum number
 * of bytes that can be copied from the source buffer.
 *
 * \param bfr points to the output buffer.
 * \param src points to the source buffer.
 * \param stopAt specifies a character that marks the end of the
 * source string.  It defaults to the null character.
 * \param altStopAt specifies an alternate character that can be tested
 * as a stop character in addition to a null.  The default is a null
 * character.  If both \a stopAt and \a altStopAt are the same character,
 * the redundant test will be optimized away at compile-time.
 *
 * \note If the string is only terminated by something other than a
 * null character,
 * specify the same stop character for both \a stopAt and \a altStopAt
 * to remove the unneeded/undesired test for a null character.
 * When there is only a 1-bit difference between the two values,
 * the optimizer can reduce this to a single test against the other bits.
 * One common scenario that fits this pattern is an ASCII space (0x20)
 * and a null: the resulting test is against 0xdf.
 *
 * \return A pointer to the end of the destination buffer is returned;
 * this will be 1 byte past last character that was copied.
 * The end of the buffer might be less than \a bfr + \a N if any of the stop
 * characters were seen.
 *
 * No trailing null or stop character is added to mark the end of the string.
 * See copyShortStringUntil<>() for a routine which enforces returning a
 * null-terminated string.
 */
#pragma GCC diagnostic push 
#pragma GCC diagnostic ignored "-Wmissing-attributes"
template <uint_fast8_t N> char *do_copyShortStringUntil(char *bfr, const char *src, const char stopAt = '\0', const char altStopAt = '\0') OME_ALWAYS_INLINE OME_ALWAYS_FLATTEN OME_ALWAYS_OPTIMIZE("-O3") NONNULL_PARAMETERS(1, 2) NONNULL_RETURN;

template <> inline char *do_copyShortStringUntil<0>(char *bfr, const char *src, const char stopAt, const char altStopAt) OME_ALWAYS_INLINE OME_ALWAYS_FLATTEN OME_ALWAYS_OPTIMIZE("-O3") NONNULL_PARAMETERS(1, 2) NONNULL_RETURN;

// halt recursion with zero length string
template <> inline char *do_copyShortStringUntil<0>(char *bfr, const char *src, const char stopAt, const char altStopAt)
{
    return (bfr);
}

template <uint_fast8_t N> inline char *do_copyShortStringUntil(char *bfr, const char *src, const char stopAt, const char altStopAt)
{
    char c = *src;
    /* Set expectation bias based upon how close the end we are;
     * the closer to the end, the higher the probability that one
     * will have seen a stop character.
     * For now, evaluate in terms of distance remaining, which will
     * be ineffective with large buffers.  Ideally, determined
     * based on actual size of buffer, perhaps via second template
     * argument.
     */
    enum { AVE_LEN_LEFT = 2 };
    if (OME_EXPECT(((c == stopAt) || (c == altStopAt)), (N <= AVE_LEN_LEFT))) {
        return (bfr);
    }
    *bfr = c;
    return (do_copyShortStringUntil < N - 1 > (bfr + 1, src + 1, stopAt));
}

#pragma GCC diagnostic pop

/*! \brief Templated C++-only variant of safe_strcpy() suitable for
 * handling small-sized strings that are being copied into buffers
 * whose size is known at compile time.
 *
 * When used as intended and inlined, the tail recursion will be
 * flattened into a sequence of test and jumps and register pressure
 * will be reduced.
 *
 * \arg N is the template parameter that specifies the maximum number of
 * bytes that can be taken from the source buffer.
 * \param bfr points to the output buffer.  The output buffer must be
 * at least \a N bytes long.  If truncateIfNeeded is false, then the
 * output buffer must be at least \a N + 1 bytes long to hold the null marker.
 * \param src points to the source buffer.
 * \param stopAt specifies a character that marks the end of the source string.
 * It defaults to the null character.
 * \param altStopAt specifies an alternate character that can be tested
 * as a stop character.  It defaults to the null character.
 * \param truncateIfNeeded is a Boolean that indicates if the destination
 * buffer only has \a N characters and \a N characters were copied from
 * the source, then the last character must be replaced by a null.
 * This is not an issue if the destination buffer can hold at
 * least \a N + 1 characters.
 *
 * \return Always returns the start of the output buffer.
 */
template <uint_fast8_t N> char *copyShortStringUntil(char *bfr, const char *src, const char stopAt = '\0', const char altStopAt = '\0', const bool truncateIfNeeded = false) OME_ALWAYS_INLINE OME_ALWAYS_FLATTEN OME_ALWAYS_OPTIMIZE("-O3") NONNULL_PARAMETERS(1, 2) NONNULL_RETURN;

template <uint_fast8_t N> inline char *copyShortStringUntil(char *bfr, const char *src, const char stopAt, const char altStopAt, const bool truncateIfNeeded)
{
    // expand templated implementation to copy string
    char *end = do_copyShortStringUntil<N>(bfr, src, stopAt, altStopAt);
    // NOTE: this test gets dropped if truncateIfNeeded is false at compile time
    if ((truncateIfNeeded == true) && (end == (bfr + N))) { // copied buffer until full
        end -= 1;
    }
    *end = '\0';
    return (bfr);
}

/*! \brief Similar to copyShortStringUntil<>(), but returns the number
 * of bytes before the trailing null.  This would be the equivalent
 * of \c strlen(bfr).
 *
 * \arg N is the template parameter that specifies the maximum number of
 * bytes that can be taken from the source buffer.
 * \param bfr points to the output buffer.  The output buffer must be
 * at least \a N bytes long.  If \a truncateIfNeeded is false, then the
 * output buffer must be at least \a N + 1 bytes long to hold the null marker.
 * \param src points to the source buffer.
 * \param stopAt specifies a character that marks the end of the source string.
 * It defaults to the null character.
 * \param altStopAt specifies an alternate character that can be tested
 * as a stop character.  It defaults to the null character.
 * \param truncateIfNeeded is a Boolean that indicates if the destination
 * buffer only has \a N characters and \a N characters were copied from
 * the source, then the last character must be replaced by a null.
 * This is not an issue if the destination buffer can hold at
 * least \a N + 1 characters.
 *
 * \return The length of the copied string is returned.  The value is
 * the equivalent of strlen(bfr).  Performance-senstive applications can
 * use this to avoid the need to make a \c strlen() call of their own.
 */
template <uint_fast8_t N> uint_fast32_t copyShortStringUntilAndReturnLength(char *bfr, const char *src, const char stopAt = '\0', const char altStopAt = '\0', const bool truncateIfNeeded = false) OME_ALWAYS_INLINE OME_ALWAYS_FLATTEN OME_ALWAYS_OPTIMIZE("-O3") NONNULL_PARAMETERS(1, 2);

template <uint_fast8_t N> inline uint_fast32_t copyShortStringUntilAndReturnLength(char *bfr, const char *src, const char stopAt, const char altStopAt, const bool truncateIfNeeded)
{
    // expand templated implementation to copy string
    char *end = do_copyShortStringUntil<N>(bfr, src, stopAt, altStopAt);
    // NOTE: this test gets dropped if truncateIfNeeded is false at compile time
    if ((truncateIfNeeded == true) && (end == (bfr + N))) { // copied buffer until full
        end -= 1;
    }
    *end = '\0';
    // compute length and return
    return (static_cast<uint_fast32_t>(end - bfr));
}


/*! \brief Demangles a C++ encoded type name into its human-readable name.
 *
 * \param name specifies the encoded type name.
 * \param bfr points to an output buffer into which the demangled name
 * will be placed.
 * \param bfrLen specifies the length of the output buffer.
 *
 * \return A pointer to the demangled type name in \a bfr is returned.
 */
const char *demangleEncodedTypeName(const char *name, char *bfr, uint32_t bfrLen) NONNULL_PARAMETERS(1, 2) NONNULL_RETURN;
inline const char *demangleEncodedTypeName(const char *name, char *bfr, uint32_t bfrLen)
{
#    ifndef _WIN32
    int result;
    char *readableName = abi::__cxa_demangle(name, 0, 0, &result);
    safe_strcpy(bfr, readableName, bfrLen);
    ::free(readableName);
#    else
#    if 0 /* if DbgHelp library is around... */
    UnDecorateSymbolName(name, bfr, bfrLen, 0);
#    else
    safe_strcpy(bfr, name, bfrLen);
#    endif
#    endif
    return (bfr);
}

/*! \brief Templated convenience function that obtains the demangled type name
 * of a C++ type.
 *
 * \arg TYPENAME is the template parameter that specifies the type.
 * \param bfr points to an output buffer into which the demangled name
 * will be placed.
 * \param bfrLen specifies the length of the output buffer.
 *
 * \return A pointer to the demangled type name in \a bfr is returned.
 *
 * Most usage scenarios find this routine to be the easiest to use when
 * the type is known, such as via an incoming template argument.
 * A buffer is created to hold the demangled name and the type of interest
 * is used the template parameter:
 * \code
 *     char bfr[256];
 *     char *typeName = demangledTypeName<uint64_t>(bfr, sizeof(bfr));
 * \endcode
 */
template <typename TYPENAME>
inline const char *demangledTypeName(char *bfr, uint32_t bfrLen)
{
    return (demangleEncodedTypeName(typeid(TYPENAME).name(), bfr, bfrLen));
}

/*! \brief Convenience macro to return the demangled type provided
 * by a \c std::typeinfo value.
 *
 * \param typeid_var specifies the \c std::typeinfo content, typically
 * obtained from the \c typeid() operator.
 * \param bfr specifies the output buffer into which the demangled name
 * will be placed.  It is expected to be passed as \c char array,
 * not a pointer to a buffer.
 *
 * No length is specified as it is taken from the \c sizeof(bfr).
 *
 * \return A pointer to the demangled type name in \a bfr is returned.
 *
 * This macro is useful in code that has no idea what it was provided.
 *
 * \code
 *     char bfr[256];
 *     const char *typeName = demangledEncodedTypeName(typeid(*somePointer), bfr);
 * \endcode
 */
#define demangledTypeIdName(typeid_var, bfr) demangleEncodedTypeName((typeid_var).name(), bfr, sizeof(bfr))

#endif /* only for C++ */

/* vim: set expandtab shiftwidth=4 tabstop=4: */
#endif
