#ifndef _MEMCHR_SSE2_H_
//! \cond
#define _MEMCHR_SSE2_H_ "$Id: memchrSSE2.h 316 2020-03-26 16:18:40Z geoff $"
//! \endcond

/*
    fast memchr and strlen using SSE2 for gcc 4.x / Visual Studio 2008(32bit/64bit)

    Copyright (C) 2008 MITSUNARI Shigeo at Cybozu Labs, Inc.
    license:new BSD license
*/
/*! \file
 * \brief Fast memchr() and strlen() using SSE2 instructions.
 * \copyright 2008 MITSUNARI Shigeo at Cybozu Labs, Inc.
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <memory.h>
#include <vector>

#ifdef _WIN32
    #include <intrin.h>
    #define _memchr_bsf(x) (_BitScanForward(&x, x), x)
#else
    #include <xmmintrin.h>
    #define _memchr_bsf(x) __builtin_ctz(x)
#endif

inline void *memchrSSE2(const void *ptr, int c, size_t len)
{
    const char *p = reinterpret_cast<const char*>(ptr);
    if (len >= 16) {
        __m128i c16 = _mm_set1_epi8(static_cast<char>(c));
        /* 16 byte alignment */
        size_t ip = reinterpret_cast<size_t>(p);
        size_t n = ip & 15;
        if (n > 0) {
            ip &= ~15;
            __m128i x = *(const __m128i*)ip;
            __m128i a = _mm_cmpeq_epi8(x, c16);
            unsigned long mask = _mm_movemask_epi8(a);
            mask &= 0xffffffffUL << n;
            if (mask) {
                return (void*)(ip + _memchr_bsf(mask));
            }
            n = 16 - n;
            len -= n;
            p += n;
        }
        while (len >= 32) {
            __m128i x = *(const __m128i*)&p[0];
            __m128i y = *(const __m128i*)&p[16];
            __m128i a = _mm_cmpeq_epi8(x, c16);
            __m128i b = _mm_cmpeq_epi8(y, c16);
            unsigned long mask = (_mm_movemask_epi8(b) << 16) | _mm_movemask_epi8(a);
            if (mask) {
                return (void*)(p + _memchr_bsf(mask));
            }
            len -= 32;
            p += 32;
        }
    }
    while (len > 0) {
        if (*p == c) return (void*)p;
        p++;
        len--;
    }
    return 0;
}

inline size_t strlenSSE2(const char *p)
{
    const char *const top = p;
    __m128i c16 = _mm_set1_epi8(0);
    /* 16 byte alignment */
    size_t ip = reinterpret_cast<size_t>(p);
    size_t n = ip & 15;
    if (n > 0) {
        ip &= ~15;
        __m128i x = *(const __m128i*)ip;
        __m128i a = _mm_cmpeq_epi8(x, c16);
        unsigned long mask = _mm_movemask_epi8(a);
        mask &= 0xffffffffUL << n;
        if (mask) {
            return _memchr_bsf(mask) - n;
        }
        p += 16 - n;
    }
    /*
        thanks to egtra-san
    */
    assert((reinterpret_cast<size_t>(p) & 15) == 0);
    if (reinterpret_cast<size_t>(p) & 31) {
        __m128i x = *(const __m128i*)&p[0];
        __m128i a = _mm_cmpeq_epi8(x, c16);
        unsigned long mask = _mm_movemask_epi8(a);
        if (mask) {
            return p + _memchr_bsf(mask) - top;
        }
        p += 16;
    }
    assert((reinterpret_cast<size_t>(p) & 31) == 0);
    for (;;) {
        __m128i x = *(const __m128i*)&p[0];
        __m128i y = *(const __m128i*)&p[16];
        __m128i a = _mm_cmpeq_epi8(x, c16);
        __m128i b = _mm_cmpeq_epi8(y, c16);
        unsigned long mask = (_mm_movemask_epi8(b) << 16) | _mm_movemask_epi8(a);
        if (mask) {
            return p + _memchr_bsf(mask) - top;
        }
        p += 32;
    }
}

/* macro unneeded from this point onwards */
#undef _memchr_bsf

#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
