#ifndef _TEXT2INT_TBL_H_
//! \cond
#define _TEXT2INT_TBL_H_ "$Id: text2int_tbl.h 454 2020-07-23 20:22:23Z geoff $"
//! \endcond

/* Copyright (C) 2010 - 2020 FARGOS Development, LLC
 * Author:  Geoff Carpenter -- http://www.fargos.net/gcc.html
 */

/*! \file
 * \brief Internal tables used for extreme performance text/number conversion
 * routines.
 *
 * \author Geoff Carpenter
 * \copyright 2010 - 2020 FARGOS Development, LLC
 */

/** \addtogroup FARGOSbase
 * @{
 */

/*! \addtogroup FARGOSnumeric 
 * @{
 */


#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/*! \brief Table of powers-of-10 constants as 32-bit unsigned integers.
 *
 * \sa powerOf10Constant()
 */
const uint32_t powersOf10[10] = {
    1, 10, 100, 1000, 10000, 100000,
    1000000, 10000000, 100000000, 1000000000U
};

/*! \brief Table of powers-of-10 constants as 64-bit unsigned integers.
 *
 * \sa powerOf10Constant()
 */
const uint64_t powersOf10_64bit[16] = {
    1, 10, 100, 1000, 10000, 100000,
    1000000, 10000000, 100000000, 1000000000U,
    static_cast<uint64_t>(1000000000U) * 10,
    static_cast<uint64_t>(1000000000U) * 100,
    static_cast<uint64_t>(1000000000U) * 1000,
    static_cast<uint64_t>(1000000000U) * 10000,
    static_cast<uint64_t>(1000000000U) * 100000,
    static_cast<uint64_t>(1000000000U) * 1000000
};

/*! \brief Table of powers-of-10 constants as floating-point values.
 *
 * \sa powerOf10Constant()
 */
const double floatingPowersOf10[10] = {
    1, 10, 100, 1000, 10000, 100000,
    1000000, 10000000, 100000000, 1000000000
};


/*! \brief Table of reciprocal powers-of-10 divisors.
 *
 * It is often an order-of-magnitude faster to multiply by a
 * reciprocal than perform a division, so this table of reciprocals
 * is used to divide numbers by various powers-of-10.
 *
 * \sa powerOf10Constant()
 */
const double reciprocalPowersOf10[16] = {
    1, 0.1, 0.01, 0.001, 0.0001,
    0.00001, 0.000001, 0.0000001,
    0.00000001, 0.000000001,
    0.0000000001,
    0.00000000001,
    0.000000000001,
    0.0000000000001,
    0.00000000000001,
    0.000000000000001,
};


/*! \brief Convert ASCII digit into corresponding 32-bit value.
 *
 * \warning numbers greater than 2^32 won't be converted correctly.
 * Use the text2int64_table[] if numbers >= 5 billion are to be converted.
 *
 * \sa text2int64_table
 */
const uint32_t text2int_table[256][10] = {
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 0 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 1 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 2 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 3 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 4 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 5 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 6 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 7 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 8 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 9 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 10 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 11 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 12 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 13 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 14 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 15 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 16 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 17 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 18 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 19 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 20 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 21 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 22 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 23 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 24 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 25 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 26 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 27 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 28 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 29 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 30 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 31 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 32 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 33 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 34 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 35 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 36 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 37 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 38 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 39 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 40 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 41 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 42 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 43 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 44 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 45 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 46 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 47 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 48  = "0"*/
    {
        1, 10, 100, 1000, 10000, 100000,  /* 49  = "1"*/
        1000000, 10000000, 100000000, 1000000000U
    },
    {
        2, 20, 200, 2000, 20000, 200000,  /* 50  = "2"*/
        2000000, 20000000, 200000000, 2000000000U
    },
    {
        3, 30, 300, 3000, 30000, 300000,  /* 51  = "3"*/
        3000000, 30000000, 300000000, 3000000000U
    },
    {
        4, 40, 400, 4000, 40000, 400000,  /* 52  = "4"*/
        4000000, 40000000, 400000000, 4000000000U
    },
    {
        5, 50, 500, 5000, 50000, 500000,  /* 53  = "5"*/
        5000000, 50000000, 500000000, ~0U
    },
    {
        6, 60, 600, 6000, 60000, 600000,  /* 54  = "6"*/
        6000000, 60000000, 600000000, ~0U
    },
    {
        7, 70, 700, 7000, 70000, 700000,  /* 55  = "7"*/
        7000000, 70000000, 700000000, ~0U
    },
    {
        8, 80, 800, 8000, 80000, 800000,  /* 56  = "8"*/
        8000000, 80000000, 800000000, ~0U
    },
    {
        9, 90, 900, 9000, 90000, 900000,  /* 57  = "9"*/
        9000000, 90000000, 900000000, ~0U
    },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 58 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 59 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 60 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 61 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 62 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 63 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 64 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 65 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 66 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 67 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 68 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 69 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 70 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 71 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 72 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 73 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 74 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 75 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 76 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 77 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 78 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 79 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 80 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 81 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 82 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 83 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 84 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 85 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 86 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 87 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 88 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 89 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 90 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 91 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 92 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 93 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 94 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 95 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 96 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 97 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 98 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 99 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 100 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 101 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 102 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 103 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 104 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 105 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 106 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 107 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 108 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 109 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 110 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 111 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 112 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 113 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 114 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 115 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 116 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 117 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 118 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 119 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 120 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 101 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 122 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 123 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 124 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 125 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 126 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 127 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 128 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 129 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 130 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 131 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 132 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 133 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 134 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 135 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 136 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 137 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 138 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 139 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 140 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 141 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 142 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 143 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 144 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 145 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 146 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 147 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 148 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 149 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 150 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 151 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 152 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 153 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 154 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 155 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 156 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 157 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 158 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 159 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 160 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 161 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 162 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 163 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 164 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 165 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 166 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 167 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 168 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 169 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 170 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 171 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 172 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 173 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 174 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 175 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 176 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 177 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 178 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 179 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 180 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 181 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 182 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 183 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 184 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 185 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 186 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 187 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 188 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 189 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 190 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 191 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 192 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 193 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 194 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 195 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 196 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 197 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 198 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 199 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 200 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 201 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 202 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 203 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 204 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 205 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 206 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 207 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 208 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 209 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 210 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 211 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 212 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 213 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 214 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 215 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 216 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 217 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 218 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 219 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 220 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 201 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 222 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 223 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 224 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 225 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 226 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 227 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 228 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 229 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 230 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 231 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 232 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 233 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 234 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 235 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 236 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 237 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 238 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 239 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 240 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 241 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 242 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 243 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 244 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 245 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 246 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 247 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 248 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 249 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 250 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 251 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 252 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 253 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* 254 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }    /* 255 */
};

/*! \brief Convert ASCII digit into corresponding 64-bit value.
 *
 */
const uint64_t text2int64_table[256][20] = {
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 0 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 1 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 2 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 3 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 4 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 5 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 6 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 7 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 8 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 9 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 10 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 11 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 12 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 13 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 14 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 15 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 16 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 17 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 18 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 19 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 20 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 21 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 22 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 23 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 24 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 25 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 26 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 27 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 28 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 29 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 30 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 31 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 32 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 33 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 34 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 35 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 36 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 37 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 38 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 39 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 40 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 41 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 42 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 43 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 44 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 45 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 46 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 47 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 48 */
    {
        1, 10, 100, 1000, 10000, 100000, /* 49 = '1' */
        1000000, 10000000, 100000000, 1000000000,
        10000000000U, 100000000000U, 1000000000000U, 10000000000000U,
        100000000000000U, 1000000000000000U, 10000000000000000U,
        100000000000000000U, 1000000000000000000U, 10000000000000000000U
    },
    {
        2, 20, 200, 2000, 20000, 200000, /* 50 = '2' */
        2000000, 20000000, 200000000, 2000000000U,
        20000000000U, 200000000000U, 2000000000000U, 20000000000000U,
        200000000000000U, 2000000000000000U, 20000000000000000U,
        200000000000000000U, 2000000000000000000U, 1553255926290448384U
    },
    {
        3, 30, 300, 3000, 30000, 300000, /* 51 = '3' */
        3000000, 30000000, 300000000, 3000000000U,
        30000000000U, 300000000000U, 3000000000000U, 30000000000000U,
        300000000000000U, 3000000000000000U, 30000000000000000U,
        300000000000000000U, 3000000000000000000U, 11553255926290448384U
    },
    {
        4, 40, 400, 4000, 40000, 400000, /* 52 = '4' */
        4000000, 40000000, 400000000, 4000000000U,
        40000000000U, 400000000000U, 4000000000000U, 40000000000000U,
        400000000000000U, 4000000000000000U, 40000000000000000U,
        400000000000000000U, 4000000000000000000U, 3106511852580896768U
    },
    {
        5, 50, 500, 5000, 50000, 500000, /* 53 = '5' */
        5000000, 50000000, 500000000, 5000000000U,
        50000000000U, 500000000000U, 5000000000000U, 50000000000000U,
        500000000000000U, 5000000000000000U, 50000000000000000U,
        500000000000000000U, 5000000000000000000U, 13106511852580896768U
    },
    {
        6, 60, 600, 6000, 60000, 600000, /* 54 = '6' */
        6000000, 60000000, 600000000, 6000000000U,
        60000000000U, 600000000000U, 6000000000000U, 60000000000000U,
        600000000000000U, 6000000000000000U, 60000000000000000U,
        600000000000000000U, 6000000000000000000U, 4659767778871345152U
    },
    {
        7, 70, 700, 7000, 70000, 700000, /* 55 = '7' */
        7000000, 70000000, 700000000, 7000000000U,
        70000000000U, 700000000000U, 7000000000000U, 70000000000000U,
        700000000000000U, 7000000000000000U, 70000000000000000U,
        700000000000000000U, 7000000000000000000U, 14659767778871345152U
    },
    {
        8, 80, 800, 8000, 80000, 800000, /* 56 = '8' */
        8000000, 80000000, 800000000, 8000000000U,
        80000000000U, 800000000000U, 8000000000000U, 80000000000000U,
        800000000000000U, 8000000000000000U, 80000000000000000U,
        800000000000000000U, 8000000000000000000U, 6213023705161793536U
    },
    {
        9, 90, 900, 9000, 90000, 900000, /* 57 = '9' */
        9000000, 90000000, 900000000, 9000000000U,
        90000000000U, 900000000000U, 9000000000000U, 90000000000000U,
        900000000000000U, 9000000000000000U, 90000000000000000U,
        900000000000000000U, 9000000000000000000U, ~0U
    },
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 58 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 59 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 60 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 61 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 62 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 63 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 64 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 65 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 66 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 67 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 68 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 69 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 70 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 71 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 72 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 73 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 74 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 75 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 76 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 77 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 78 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 79 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 80 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 81 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 82 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 83 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 84 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 85 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 86 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 87 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 88 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 89 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 90 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 91 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 92 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 93 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 94 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 95 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 96 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 97 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 98 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 99 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 100 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 101 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 102 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 103 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 104 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 105 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 106 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 107 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 108 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 109 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 110 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 111 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 112 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 113 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 114 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 115 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 116 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 117 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 118 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 119 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 120 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 121 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 122 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 123 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 124 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 125 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 126 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 127 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 128 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 129 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 130 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 131 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 132 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 133 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 134 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 135 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 136 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 137 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 138 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 139 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 140 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 141 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 142 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 143 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 144 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 145 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 146 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 147 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 148 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 149 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 150 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 151 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 152 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 153 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 154 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 155 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 156 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 157 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 158 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 159 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 160 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 161 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 162 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 163 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 164 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 165 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 166 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 167 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 168 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 169 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 170 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 171 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 172 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 173 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 174 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 175 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 176 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 177 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 178 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 179 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 180 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 181 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 182 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 183 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 184 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 185 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 186 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 187 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 188 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 189 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 190 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 191 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 192 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 193 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 194 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 195 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 196 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 197 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 198 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 199 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 200 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 201 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 202 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 203 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 204 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 205 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 206 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 207 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 208 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 209 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 210 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 211 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 212 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 213 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 214 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 215 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 216 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 217 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 218 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 219 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 220 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 221 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 222 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 223 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 224 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 225 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 226 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 227 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 228 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 229 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 230 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 231 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 232 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 233 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 234 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 235 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 236 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 237 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 238 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 239 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 240 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 241 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 242 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 243 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 244 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 245 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 246 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 247 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 248 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 249 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 250 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 251 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 252 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 253 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, /* 254 */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}  /* 255 */
};


//! \brief Table of uppercase hexadecimal characters for each byte value.
extern const char byteAsLowercaseHexadecimal[256][3];

//! \brief Table of uppercase hexadecimal characters for each byte value.
extern const char byteAsUppercaseHexadecimal[256][3];

//! \brief Table of 2-character strings representing 00-99.
extern const char numberAsTwoDigits[100][3];

//! \brief Table of word-aligned 4-character groups representing 0000-9999.
extern const union Aligned4Chars {
    char    digits[4];
    uint32_t    val;
    struct {
        char        _zero[2];
        uint16_t    two_digits;
    } lastTwoDigits;
    struct {
        char        _zero[1];
        char        hundreds;
        uint16_t    two_digits;
    } lastThreeDigits;
} numberAs4Digits[10000];

#ifdef __cplusplus
}; /* close extern "C" scope */
#endif

/** @} */

/** @} */

#endif

/* vim: set expandtab shiftwidth=4 tabstop=4: */
