#ifndef _ORDERED_INPUT_HPP_
//! \cond
#define _ORDERED_INPUT_HPP_ "$Id: OrderedInput.hpp 455 2020-07-23 20:23:59Z geoff $"
//! \endcond

/* Copyright (C) 2015 - 2020 FARGOS Development, LLC
 */

#include <queue>
#include <vector>
#include <set>

#include <utils/logging/logging_api.hpp>
#include <utils/io/io_processor.hpp>

/* typedef only present to create forward declaration to compensate
 * for doxygen bugs
 */
//class OrderedMultipleInputFilter;

/*! \file
 * \brief Implements high-performance ordered input from multiple data streams.
 *
 * These facilities support simulation environments that consume
 * time or sequence-ordered records from multiple sources.
 * Data can come from files, sockets, memory-mapped regions,
 * or be generated on the fly.
 */

/** \addtogroup FARGOSio
 * @{
 */

// subgroup
/** \defgroup FARGOSorderedIO Ordered Input/Output Streams
 *
 * \brief The Ordered I/O facilities support simulation environments
 * that consume time or sequence-ordered records from multiple sources.
 *
 * Data can come from files, sockets, memory-mapped regions,
 * or be generated on the fly.  They can be used in conventional
 * applications that are written as filters; the \b \c mergeLogs
 * utility is a prototypical example.
 *
 * OrderedInputSource provides an interface to sources providing
 * a stream of time-ordered records.
 *
 * OrderedMultipleInputFilter implements the top-level processing logic,
 * using a OrderedInputPriorityQueue to sort between streams.
 *
 * OrderedInputBlock tracks regions of memory representing logical messages,
 * typically corresponding to a UDP packet.
 *
 * @{
 */

#ifndef FORCE_ORDERED_INPUT_BLOCK_AS_FINAL
/*! \brief Disable all virtualization support in OrderedInputBlock.
 *
 * If defined as 1, support for implementing a custom comparison
 * algorithm is disabled.
 * The normal default is 0.
 */
#define FORCE_ORDERED_INPUT_BLOCK_AS_FINAL 0
#endif

#if FORCE_ORDERED_INPUT_BLOCK_AS_FINAL == 1
#define ORDERED_INPUT_BLOCK_FINAL final
#define ORDERED_INPUT_BLOCK_VIRTUAL
#else
//! \brief Set to \b \c final if \c ORDERED_INPUT_BLOCK_AS_FINAL is set to 1
#define ORDERED_INPUT_BLOCK_FINAL
//! \brief Set to \b \c virtual if \c ORDERED_INPUT_BLOCK_AS_FINAL is set to 0
#define ORDERED_INPUT_BLOCK_VIRTUAL virtual
#endif

/*! \brief Input block record for use with OrderedMultipleInputFilter.
 *
 * The OrderedInputBlock structure is the fundamental record delivery
 * mechanism used by OrderedMultipleInputFilter objects.
 */
class OrderedInputBlock
//! \cond
ORDERED_INPUT_BLOCK_FINAL
//! \endcond
{
protected:
    enum { BLOCK_REUSE_PADDING=128 };
public:
#ifdef _MSC_VER
    typedef int64_t ssize_t;    // Visual Studio lacks definition
#endif
    typedef ssize_t BlockSize_t; // int32_t is probably more appropriate
    uint64_t                    sortKey;        //!< sort key for priority queue, typically represents nanoseconds
    class OrderedInputSource    *inputFile;     //!< source/owner of storage block
    unsigned char               *blockStart;    //!< base of storage block
    BlockSize_t                 usedLen;        //!< actively used amount within storage block
//protected: // blockLen currently used in mergeLogs
    BlockSize_t                 blockLen;       //!< size of storage block
    bool                        ownBlock;       //!< Boolean flag indicating ownership

public:
    /*! \brief Create a input block, potentially copying the source data.
     *
     * \param key specifies the value of the used to order the input.
     * \param block points to the data for the block
     * \param len specifies the length of the data
     * \param source points to the input source from which the data was obtained
     * \param makeCopy is a Boolean that indicates if a copy of the data should
     * be made.  If true, the source data can be altered or deleted immediately,
     * however, effort must be expended to duplicate the content.
     */
    OrderedInputBlock(uint64_t key, unsigned char *block, BlockSize_t len, OrderedInputSource *source, bool makeCopy=true) {
        OME_PREFETCH(this, 1, 0); // going to modify, limited use
        inputFile = source;
        sortKey = key;
        usedLen = len;
        ownBlock = makeCopy;
        if (makeCopy == false) {
            blockStart = block;
            blockLen = 0; // since we do not own the block, record available length as 0
        } else { // we allocate a new block and copy
            /* NOTE: no rounding up is done on the initial allocation,
             * but reuseBlock() will attempt resize blocks in units of pages
             */
            OME_PREFETCH(block, 0, 0); // read-only, not used again from this location
            unsigned char *newBlock = new unsigned char[len];
            blockStart = newBlock;
            blockLen = len;
            memcpy(newBlock, block, len);
        }
    }

    /*! \brief Reinitialize an existing block.
     * This routine is nearly identical to the OrderedInputBlock constructor,
     * but is called when reusing an existing record.
     *
     * \param key specifies the value of the used to order the input.
     * \param block points to the data for the block
     * \param len specifies the length of the data
     * \param source points to the input source from which the data was obtained
     * \param makeCopy is a Boolean that indicates if a copy of the data should
     * be made.  If true, the source data can be altered or deleted immediately,
     * however, effort must be expended to duplicate the content.
     */
    inline void reuseBlock(uint64_t key, unsigned char *block, BlockSize_t len, OrderedInputSource *source, bool makeCopy=true) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
        inputFile = source;
        sortKey = key;
        if (makeCopy == false) { // no copy needed, just point at it
            if (OME_EXPECT_FALSE(ownBlock == true)) { // delete prior held copy
                delete[] blockStart;
                ownBlock = false;
            }
            blockStart = block;
            blockLen = 0;
            usedLen = len;
            return;
        }
        // must make copy
        OME_PREFETCH(block, 0, 0); // read-only, not used again from this location
        OME_PREFETCH(block + CACHE_LINE_LENGTH, 0, 0); // read-only, not used again from this location
        /* NOTE: if ownBlock == false, then blockLen will be 0 */
        if (OME_EXPECT_TRUE(len <= blockLen)) { // have enough space
            memcpy(blockStart, block, len);
            usedLen = len;
            // blockLen > 0, so ownBlock was true
            return;
        }
        // need more space
        if (len <= (8192 + 4096)) { // small enough to double
            // get most significant bit
#ifndef _MSC_VER
            int b = __builtin_clz(len); // count leading zeroes
            b = ((sizeof(len) * 8) - 1) - b;
#else
            unsigned long b;
            _BitScanReverse(&b, len);
#endif
            blockLen = 2 << b;
        } else { // just round up to 4K boundary
            blockLen = (len  & ~0xfff) + 0x1000;
        }
        if (OME_EXPECT_TRUE(ownBlock == true)) { // delete prior held copy
            delete[] blockStart;
        } else {
            ownBlock = true;
        }
        unsigned char *newBlock = new unsigned char[blockLen];
        memcpy(newBlock, block, len);
        blockStart = newBlock;
        usedLen = len;
    }

//! \cond
    ORDERED_INPUT_BLOCK_VIRTUAL
//! \endcond
    ~OrderedInputBlock() {
        if (ownBlock) {
            delete[] blockStart;
        }
    }

    /*! \brief Get sort key for block
     */
    inline uint64_t getSortKey() const OME_ALWAYS_INLINE {
        return (sortKey);
    }

    /*! \brief Return maximum possible value for a sort key.
     *
     * \note This is a static routine, so the value returned is not
     * specific to a particular OrderedInputSource.
     */
    inline static CONSTEXPR uint64_t getMaximumPossibleSortKey() OME_ALWAYS_INLINE OME_CONST_FUNCTION {
        return (~static_cast<uint64_t>(0));
    }

    /*! \brief Less-than comparison between OrderedInputBlock objects.
     *
     * \param arg is a reference to the other peer object to be
     * compared. 
     *
     * \warning The unexpected is done for this specialized class by
     * having less-than act like greater-than so that the priority queue
     * returns elements in the desired order.
     *
     * \note This routine is \c virtual to allow it to be overridden.
     * This may be rare, but it does allow the logic to work with a subclass.
     * Due to the rarity of this functionality being exploited and the
     * multiple uses per input block, this benefits greatly from
     * devirtualization, so marking application classes as final is helpful.
     *
     * \sa FORCE_ORDERED_INPUT_BLOCK_AS_FINAL for a define to remove
     * all virtualization for performance.
     */
//! \cond
    ORDERED_INPUT_BLOCK_VIRTUAL
//! \endcond
     bool operator<(const OrderedInputBlock &arg) const OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
        /* NOTE: the test is reversed between the keys so that the priority
         * queue returns elements in the desired order.
         */
        if (sortKey > arg.sortKey) {
            return (true);
        }
#if 0
        if (sortKey < arg.sortKey) {
            return (false);
        }
        /* keys match...compare file object pointer to 
         * generate a strict partial order
         */
        uint64_t a1 = (uint64_t) inputFile;
        uint64_t a2 = (uint64_t) arg.inputFile;
        if (a1 < a2) {
            return (true);
        }
#endif
        return (false);
    }

}; // end class OrderedInputBlock

/*! \brief Abstract interface to be used with input sources
 * that can generate a sequence of records.
 *
 * An OrderedInputSource represents the interface to a stream of
 * events, typically ordered by time or sequence number.
 *
 * \note No actual storage is defined by this class.
 */
class OrderedInputSource {
public:
    static const struct timespec WAIT_YEAR_MAX; //!< Special \c timespec for maximum wait time
    static const struct timespec WAIT_NEVER;    //!< Special \c timespec for no wait time
#ifdef _MSC_VER
    typedef int64_t ssize_t;    // Visual Studio lacks definition
#endif

    /*! \brief Return result from getNextInputBlock().
     *
     * This structure is used to allow results to be
     * returned via registers and avoid unneeded reads
     * from memory.
     */
    struct InputBlockReturn {
        OrderedInputBlock   *block;         //!< points at retrieved record
        ssize_t             obtainedLen;    //!< length available, note that this is signed
    };

    virtual ~OrderedInputSource()   {}

    /*! \brief Abstract interface to acquire new input block
     *
     * \param forceCopy is a Boolean indicating if a copy of the data
     * should be made.
     * \param maxWaitTime specifies the maximum amount of time to wait
     * for new data to arrive as an absolute point in time.
     *
     * \return An InputBlockReturn structure is returned. Normally,
     * the \c block member points to the OrderedInputBlock being returned.
     * The \c obtainedLen member normally indicates the size of the
     * obtained block, but can
     * also have:
     * \retval 0 indicates no data was obtained.
     * \retval -2 indicates that end-of-file has not been seen,
     * but no data is currently available.
     *
     * \note A value greater than 1 indicates data was obtained.
     * Use the \c block->usedLen value for the actual length rather than
     * the \c obtainedLength field; this allows some applications to
     * avoid needing to read the block data.
     */
    /* NOTE: for now, maxWaitTime is passed as a pointer, since nothing
     * yet makes use of it.  This should probably evolve to being passed by
     * value.
     */
    virtual OrderedInputSource::InputBlockReturn getNextInputBlock(bool forceCopy=false, const struct timespec *maxWaitTime=nullptr) = 0;

    /*! \brief Return block previously obtained from getNextInputBlock().
     * Called after processInputBlock().
     *
     * \param block points to the OrderedInputBlock to be recovered.
     *
     * The default implementation deletes the block.
     */
    virtual void recoverInputBlock(OrderedInputBlock *block) {
        delete block;
    }

    /*! \brief Convenience definition to permit proxy objects to
     * forward an input block.
     *
     * \param block points to the block to be forwarded via the proxy.
     *
     * \return The value that would have been returned by the invoked
     * function is returned.  Usually this will be 0.
     * For now, no specific intent is assigned; returning the value
     * provided by the caller is partly a performance choice to avoid
     * the instructions needed to set an explicit return value.
     *
     * \sa OrderedInputProxy
     */
    /* NOTE:
     * Could be abstract, but only proxy objects need implement it.
     * More appropriately, it could/should be defined only in
     * the OrderedInputProxy subclass, but the cost of a dynamic_cast
     * is prohibitive, so this this a virtual with a default uninteresting
     * implementation.
     */
    virtual int forwardInputBlock(OrderedInputBlock *block) {
        return (0);
    }

    /*! \brief Obtain label/name for input source.
     *
     * \param outputBfr points to a buffer that can be used to hold the
     * text of the input source's label.
     * \param bfrLen specifies the length of the \a outputBfr.
     *
     * \return A pointer to the label text is returned.  This may be the
     * same as \a outputBfr, but it may also be constant text.
     */
    virtual const char *sourceLabel(char *outputBfr, uint_fast32_t bfrLen) const {
        return ("AnonInputSource");
    }

    /*! \brief User exit called when the end-of-file indication is first read.
     */
    virtual void noteEOFread() {}

    /*! \brief User exit called when the end-of-file indication is processed.
     */
    virtual void noteEOFprocessed() {}

}; // end class OrderedInputSource

/*! \brief Encapsulation class for a priority queue.
 *
 * The interface provided by this class allows the OrderedMultipleInputFilter
 * logic to be separated from the actual implementation selected
 * to realize a priority queue.
 *
 * There are 5 functions implemented, which could be abstracted as
 * an interface if a cost was not incurred.  For the time being,
 * these are not virtual functions.
 *
 * The getQueueLength() function
 * returns how many elements are in the queue.  A value of 0 serves
 * to indicate an empty queue.  The count is maintained as a variable
 * that can be retrieved using a single load instruction, so the cost
 * is O(1).
 *
 * The peekNextBlock() function gets the next element without affecting the queue,
 * whereas removeNextBlock() simply removes the next element without examining it.
 * The popNextBlock() function combines both to retrieve the next block and
 * remove it from the queue.
 * New elements are added via pushBlock().
 */
class OrderedInputPriorityQueue
#if __cplusplus >= 201100
    final /* not intended to be used as a user-extensible base class */
#endif
{
protected:
    typedef std::vector<OrderedInputBlock *> pQueueContainer_t;

    //! \brief Less-than operator for OrderedInputBlock pointers.
    struct ltOrderedInputBlockPtr {
        inline bool operator()(const OrderedInputBlock *arg1, const OrderedInputBlock *arg2) const OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
            // dereference and evaluate
            return (*arg1 < *arg2);
        }
    }; // end struct ltOrderedInputBlockPtr

    std::priority_queue<OrderedInputBlock *,pQueueContainer_t,ltOrderedInputBlockPtr>    pQueue;
    size_t  pQueueSize; // pQueue.size(), but maintained separately to avoid overhead of function call

public:
    OrderedInputPriorityQueue() {
        pQueueSize = 0;
    }

    ~OrderedInputPriorityQueue() {
        while (pQueue.empty() == false) { // there was some content left in the queue
            OrderedInputBlock *block = popNextBlock();
            OrderedInputSource *source = block->inputFile;
            source->recoverInputBlock(block); // usually deletes it
        }
    }

    /*! \brief Returns the number of entries in the queue.
     */
    size_t getQueueLength() const OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
        return (pQueueSize);
    }

    /*! \brief Removes the first element from the queue
     * and returns it.
     * 
     * \param decrementBy is an optional parameter that can be set to 0
     * if the client knows it will do an immediate pushBlock() with
     * an increment of 0.  This special scenario enables the
     * increment/decrement instructions to be optimized away.
     *
     * \warning Call only if content is in queue
     */
    OrderedInputBlock *popNextBlock(uint_fast8_t decrementBy=1) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") NONNULL_RETURN {
        OrderedInputBlock  *retBlock = pQueue.top();
        OME_PREFETCH(retBlock, 1, 0); // read/write, used immediately
        pQueue.pop();
        pQueueSize -= decrementBy;
        return (retBlock);
    }

    /*! \brief Specialized routine that does the equivalent of popNextBlock(),
     * but does not intend to examine the block.
     * 
     * \param decrementBy is an optional parameter that can be set to 0
     * if the client knows it will do an immediate pushBlock() with
     * an increment of 0.  This sequence enables the increment/decrement
     * instructions to be optimized away.
     *
     * \warning Call only if content is in queue
     */
    void removeNextBlock(uint_fast8_t decrementBy=1) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
        pQueue.pop();
        pQueueSize -= decrementBy;
    }

    /*! \brief Similar to popNextBlock() but does not remove the
     * entry from the queue.
     *
     * \warning Call only if content is in queue
     */
    OrderedInputBlock *peekNextBlock() const OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") NONNULL_RETURN {
        OrderedInputBlock *retBlock = pQueue.top();
        OME_PREFETCH(retBlock, 0, 0); // read-only, used immediately
        return (retBlock);
    }

    /*! \brief Add a new entry to the queue.
     *
     * \param block specifies the block to be added.
     * \param incrementBy is an optional parameter that can be set to 0
     * if the client knows it just did a removeNextBlock() call with
     * a decrement of 0. This usage enables the increment/decrement instructions
     * to be optimized away.
     */
    void pushBlock(OrderedInputBlock *block, uint_fast8_t incrementBy=1) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
        pQueue.push(block);
        pQueueSize += incrementBy;
    }

}; // end class OrderedInputPriorityQueue

/*! \brief Proxy class to allow state to be associated with
 * an existing OrderedInputSource, while appearing to
 * be the original OrderedInputSource.
 *
 * This class is viable as-is, but its true value is realized
 * only when it serves as the base class of an interesting
 * user subclass.
 *
 * Given an object of some potentially unknown user class \a A that
 * inherits from OrderedInputSource and a situation in which
 * the application really wanted an object of class \a B that would be
 * viewed as an extended subclass of \a A, the OrderedInputSourceProxy class can
 * be used to implement a replacement proxy object of class \a B that acts
 * upon the original object of class \a A.
 *
 * Obviously, it is optimal to just use the original object of class \a A
 * directly, so the use of a proxy intermediary should be avoided when feasible.
 */
class OrderedInputSourceProxy : public OrderedInputSource {
protected:
    OrderedInputSource  *proxyFor;  //!< points at the original input source

public: 
    /*! \brief Construct a proxy for an existing OrderedInputSource
     * object.
     *
     * \param interfaceFor specifies the existing object.
     */
    explicit OrderedInputSourceProxy(OrderedInputSource *interfaceFor) {
        OME_PREFETCH(&proxyFor, 1, 3); // going to modify, want long-term
        OME_PREFETCH(interfaceFor, 0, 3); // read-only, want long-term
        proxyFor = interfaceFor;
    }

    virtual ~OrderedInputSourceProxy() {
        delete proxyFor;
    }

    // block->inputFile is rewritten to be this
    virtual OrderedInputSource::InputBlockReturn getNextInputBlock(bool forceCopy=false, const struct timespec *maxWaitTime=nullptr) VIRTUAL_OVERRIDE {
        InputBlockReturn result = proxyFor->getNextInputBlock(forceCopy, maxWaitTime);
        if (OME_EXPECT_TRUE(result.block != nullptr)) {
            result.block->inputFile = this;
        }
        return (result);
    }

    // block->inputFile is rewritten to be the proxyFor object
    virtual void recoverInputBlock(OrderedInputBlock *block) VIRTUAL_OVERRIDE {
        OME_PREFETCH(proxyFor, 0, 3); // read-only, want long-term
        block->inputFile = proxyFor;
        proxyFor->recoverInputBlock(block);
    }

    // block->inputFile is rewritten to be the proxyFor object
    virtual int forwardInputBlock(OrderedInputBlock *block) VIRTUAL_OVERRIDE {
        OME_PREFETCH(proxyFor, 0, 3); // read-only, want long-term
        block->inputFile = proxyFor;
        int rc = proxyFor->forwardInputBlock(block);
        return (rc);
    }

    virtual const char *sourceLabel(char *outputBfr, uint_fast32_t bfrLen) const VIRTUAL_OVERRIDE {
        return (proxyFor->sourceLabel(outputBfr, bfrLen));
    }

    virtual void noteEOFread() VIRTUAL_OVERRIDE {
        proxyFor->noteEOFread();
    }

    virtual void noteEOFprocessed() VIRTUAL_OVERRIDE {
        proxyFor->noteEOFprocessed();
    }

}; // end class OrderedInputSourceProxy

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wsuggest-final-types"
/*! \brief Class that extends OrderedInputSourceProxy
 * with the ability to tolerate a certain number of out-of-order
 * messages that are provided by the source.
 *
 * This class is used by OrderedMultipleInputFilter::addInputSource() whenever
 * a value value greater than 0 is passed for the \a preloadBy parameter
 * or the OrderedMultipleInputFilter was created with a
 * nonzero \a maxReorderFromSource argument.
 *
 * It can be directly used by users to handle input sources that suffer
 * from reordered events.
 */
class OrderedInputSourceProxyWithReordering : public OrderedInputSourceProxy {
protected:
    /* Only data from a single source is maintained by this priority queue.
     * Consequently, an implementation that is heavily biased towards
     * records arriving in the correct order will perform best.
     * The number of records to be maintained simultaneously will be determined
     * by the maxReorder parameter passed to the constructor.
     */
    OrderedInputPriorityQueue   reorderQueue; //!< only holds data from single source
    bool                        sawEOF;         //!< indicates if end-of-file has been seen from this source

public:
    /*! \brief Constructor for reordering proxy.
     *
     * \param interfaceFor points at the true OrderedInputSource object.
     * \param maxReorder specifies the number of packets that can be
     * reordered from this source.
     *
     * A distance of up to \a maxReorder out-of-order input blocks can
     * be tolerated.  Normally, a very small value is sufficient
     * to yield the desired results, but a large value may be needed when
     * attempting to reorganize simulation output log files whose
     * timestamps change based on the day being simulated.
     */
    OrderedInputSourceProxyWithReordering(OrderedInputSource *interfaceFor, size_t maxReorder) :
        OrderedInputSourceProxy(interfaceFor)
    {
LOG_COMPONENT_CERR(io,trace) << "Create reorder proxy with amt=" << (uint32_t) maxReorder << LOG_ENDLINE;
        sawEOF = false;
        for(size_t i=0;i<=maxReorder;i+=1) {
            InputBlockReturn result = interfaceFor->getNextInputBlock(true, &OrderedInputSource::WAIT_NEVER);
            if (result.block == nullptr) { // EOF or error?
                if (result.obtainedLen == 0) { // EOF
                    sawEOF = true;
                    interfaceFor->noteEOFread();
                }
                // stop reading from this source
                break;
            }
            reorderQueue.pushBlock(result.block);
        }
    }
    
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wsuggest-final-methods"
    virtual ~OrderedInputSourceProxyWithReordering() {}
#pragma GCC diagnostic pop

    /*! \brief Disconnects proxy from its client
     */
    void dropProxy() {
        proxyFor = nullptr;
    }

    /*! \brief Returns total blocks already pending.
     *
     * This is not an EOF indication; more blocks may be available
     * from the source that have not yet been read.
     */
    inline size_t totalBlocksPending() const OME_ALWAYS_INLINE {
        return (reorderQueue.getQueueLength());
    }

    /*! \brief Implementation of OrderedInputSource::getNextInputBlock()
     * interface that inserts itself as a proxy between the caller
     * and the true source of data.
     *
     * It maintains a priority queue of pending blocks, which is
     * used to tolerate out-of-order arrival of blocks.
     */
    virtual OrderedInputSource::InputBlockReturn getNextInputBlock(bool forceCopy=false, const struct timespec *maxWaitTime=nullptr) VIRTUAL_OVERRIDE {
        OrderedInputSource::InputBlockReturn result;
        if (OME_EXPECT_TRUE(sawEOF == false)) {
            result = proxyFor->getNextInputBlock(forceCopy, maxWaitTime);
            if (OME_EXPECT_TRUE(result.block != nullptr)) {
                reorderQueue.pushBlock(result.block);
            } else if (OME_EXPECT_FALSE(result.obtainedLen == 0)) {
LOG_COMPONENT_CERR(io,trace) << "Reorder saw physical EOF pktsLeft=" << reorderQueue.getQueueLength() << LOG_ENDLINE;
                sawEOF = true;
                proxyFor->noteEOFread();
            }
        }
        if (OME_EXPECT_FALSE(reorderQueue.getQueueLength() == 0)) {
LOG_COMPONENT_CERR(io,trace) << "Reorder queue now empty" << LOG_ENDLINE;
            result.block = nullptr;
            result.obtainedLen = -2;
            return (result);
        }
        OrderedInputBlock *retBlock = reorderQueue.popNextBlock();
        retBlock->inputFile = this;
        result.block = retBlock;
        result.obtainedLen = retBlock->usedLen;
        return (result);
    }

}; // end class OrderedInputSourceProxyWithReordering

#pragma GCC diagnostic pop

/*! \brief Streaming filter that consumes input data from multiple sources
 * and outputs blocks in sorted order.
 *
 * \details
 * The processQueue() method processes all registered inputs
 * in order.
 * There is a processInputFiles() convenience method that 
 * calls preloadQueue() first before invoking processQueue().
 *
 * If reordered blocks are to be tolerated, the class
 * OrderedInputSourceProxyWithReordering will be used
 * an intermediary for a given input and reorders any
 * out-of-order content from a single source.
 *
 * The basic flow is to call OrderedInputSource::getNextInputBlock() on
 * an OrderedInputSource (or subclass) object 
 * to retrieve an OrderedInputBlock block and eventually return it via a
 * OrderedInputSource::recoverInputBlock() call.
 *
 * A retrieved block is processed by an ordered sequence of two
 * virtual function calls: noteNextKey() and processInputBlock().
 *
 * The noteNextKey() call is often used to note the source time of an input
 * block and set the simulated clock time.
 *
 * The processInputBlock() routine must be implemented by a subclass
 * and deals with doing something useful with the retrieved data.
 */
class OrderedMultipleInputFilter {
public:
#ifdef _MSC_VER
    typedef int64_t ssize_t;    // Visual Studio lacks definition
#endif


protected:
    /* NOTE: this priority queue is expected to have a small
     * number of elements: 1 per distinct input source.
     * While entries obtained from a given source will
     * tend to be in sequential order, they may frequently
     * be before or after existing elements in the queue.
     */
    OrderedInputPriorityQueue           priorityQueue;
    std::vector<OrderedInputSource *>   sourceFiles;
    std::set<OrderedInputSource *>      activeInputSources;
    size_t                              maxReorderedElements;
    uint_fast16_t                       activeSourceCount;
    bool                                forceCopy;
    bool                                threadingEnabled;

public:

    /*! \brief Create an OrderedMultipleInputFilter.
     *
     * \param maxReorderFromSource is an optional parameter
     * that indicates how many out-of-order blocks can be
     * tolerated and correctly reordered from a single source.
     */
    OrderedMultipleInputFilter(size_t maxReorderFromSource=0) {
        activeSourceCount = 0;
        maxReorderedElements = maxReorderFromSource;
        forceCopy = (maxReorderedElements > 0) ? true : false;
        threadingEnabled = false;
    }

    virtual ~OrderedMultipleInputFilter() {
        purgeQueue(); // if anything was left, cleanup
    }

    /*! \brief Set threading enabled flag.
     *
     * \param enabled is a Boolean specifying the flag value.
     */
    void setThreadingEnabled(bool enabled) {
        threadingEnabled = enabled;
    }

    //! \brief Retrieve threading enabled flag
    inline bool getThreadingEnabled() const OME_ALWAYS_INLINE {
        return (threadingEnabled);
    }

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wsuggest-final-methods"
    /*! \brief User exit to note next input sort key.
     *
     * Typically the \a key value represents nanoseconds.
     * This routine can be overridden by a user's subclass
     * to pick up the key to be used prior to a processInputBlock() call.
     *
     * \param key is the key for the current block
     * \param blockCount is a relative count of blocks
     * \param nextKeyFromAlternateSource specifies the next earliest
     * key from a different source.  If there is no other active source,
     * then a value of \c ~0 will be specified.
     */
    virtual void noteNextKey(uint64_t key, int64_t blockCount, uint64_t nextKeyFromAlternateSource) {}
#pragma GCC diagnostic pop

    /*! \brief Abstract interface to process block
     *
     * \param block points at the block to be processed
     *
     * \retval 1 continue processing, but do not
     * call OrderedInputSource::recoverInputBlock().
     * Ownership has been transferred and the block will be recovered later.
     * \retval  0 continue processing
     * \retval -1 stop processing this input
     * \retval -2 stop processing all input
     */
    /* NOTE: a default implementation could just output the block to
     * standard out, but it would seem to have limited applicability.
     */
    virtual int processInputBlock(OrderedInputBlock *block) = 0;

    /*! \brief User-exit called when end-of-file is detected on an input source.
     *
     * \param source specifies the input source being read
     *
     * The default implementation calls OrderedInputSource::noteEOFread()
     */
    virtual void nowAtEndOfFile(OrderedInputSource *source) {
        source->noteEOFread();
    }

    /*! \brief Return total number of registered input sources.
     */
    inline uint_fast32_t inputSourceTotal() const OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
        return (priorityQueue.getQueueLength());
    }

    /*! \brief Interface to return total amount of pending work.
     */
    virtual uint_fast32_t getTotalBlocksPending() const {
        /* TODO: would have to loop across sourceFiles and determine amount
         * of pending work, but OrderedInputSource does not have an
         * interface defined to report total queued blocks.
         */
        return (0);
    }

    /*! \brief Interface to return total amount of work currently
     * being processed.
     */
    virtual uint_fast32_t getTotalBlocksInProgress() const {
        return (0); // no background thread, so 0
    }

    /*! \brief Return Boolean indication of queue being empty.
     */
    inline bool isQueueEmpty() const OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
        return (priorityQueue.getQueueLength() == 0);
    }

    /*! \brief Completely purge any pending input blocks.
     */
    uint_fast32_t purgeQueue() {
        uint_fast32_t count = 0;
        while (priorityQueue.getQueueLength() != 0) { // there was some content left in the queue
            count += 1;
            OrderedInputBlock *block = priorityQueue.popNextBlock();
            OrderedInputSource *source = block->inputFile;
            source->recoverInputBlock(block); // usually deletes it
        }
        // queue is now totally empty...
        return (count);
    }

    /*! \brief Register a new input source
     *
     * \param source specifies the OrderedInputSource
     * \param preloadBy is an optional parameter that specifies
     * the amount of out-of-order blocks to be tolerated
     * from this source.  If negative, the default is used.
     */
    void addInputSource(OrderedInputSource *source, ssize_t preloadBy=-1) {
        size_t preloadAmt = (preloadBy >= 0) ? preloadBy : maxReorderedElements;
LOG_COMPONENT_CERR(io,trace) << "Add inputSource preloadAmt=" << preloadAmt << LOG_ENDLINE;
        if (preloadAmt > 0) { // use proxy capable of reordering
            OrderedInputSourceProxyWithReordering *reorderSource = new OrderedInputSourceProxyWithReordering(source, preloadAmt);
            size_t addedAmt = reorderSource->totalBlocksPending();
LOG_COMPONENT_CERR(io,trace) << "preloaded=" << addedAmt << LOG_ENDLINE;
            if (addedAmt > 0) {
                activeInputSources.insert(reorderSource);
                sourceFiles.push_back(reorderSource);
            } else {
                reorderSource->dropProxy();
                delete reorderSource;
                sourceFiles.push_back(source);
            }
        } else {
                sourceFiles.push_back(source);
        }
    }

    /*! \brief Preload the priority queue with the
     * initial set of blocks.  Only the first block
     * from each source is loaded, which means the
     * main priority queue size remains limited by
     * the number of distinct input sources.
     * Actual reordering within a given input stream
     * is handled by the OrderedInputSourceProxyWithReordering 
     * class.
     *
     * \return The total number of blocks loaded is returned.
     */
    int preloadQueue() {
        const uint_fast32_t n = (uint_fast32_t) sourceFiles.size();
        int blocksLoaded = 0;
        for(uint_fast32_t i=0;i<n;i+=1) { // for each file
            OrderedInputSource *f = sourceFiles[i];
            OrderedInputSource::InputBlockReturn retrievedBlock = f->getNextInputBlock(forceCopy, &OrderedInputSource::WAIT_NEVER);
            if (OME_EXPECT_TRUE(retrievedBlock.obtainedLen > 0)) {
                priorityQueue.pushBlock(retrievedBlock.block);
                activeInputSources.insert(f);
                blocksLoaded += 1;
            } else { // EOF or error
                if (retrievedBlock.obtainedLen == -2) { // temporary EOF
                    // invoke user exit to notify application
                    this->nowAtEndOfFile(f);
                }
            }
        }
        activeSourceCount = (uint_fast16_t) activeInputSources.size();
        // all preloading done
        return (blocksLoaded);
    }

    /*! \brief Process input blocks until all
     * OrderedInputSource objects are drained of
     * available data or a specified point in
     * time is reached.
     *
     * \param stopFlagPtr points to a memory region
     * which will be watched; processing will
     * stop if it is set to a nonzero value.
     * A null pointer can be passed if this is not
     * needed.
     * \param maxWaitTime specifies the maximum amount
     * of time to wait for input from a source.
     * Normally, this is specified as an absolute point
     * in time; however, if the number of seconds is less
     * than that representing a single day, the \c timespec is
     * interpreted as being an offset from the earliest time
     * in the queue.
     *
     * \return The number of total blocks processed
     * is returned.
     * 
     * EOF across all inputs can be detected by calling
     * isQueueEmpty().
     *
     * \note using a small (less than 24 hours) \a maxWaitTime value will
     * always process at least one input record (assuming any are
     * available).  In contrast, an absolute time will be honored and
     * may result in no records being processed.
     *
     * For simulation purposes, relative constraints will process
     * the data as quickly as possible.  If other events must be processed
     * exactly as would have happened in real-time, the best approach
     * is to use a relative time constraint for the first call to
     * processQueue() and absolute times for the subsequent calls.
     */
    int64_t processQueue(uint32_t *stopFlagPtr, const struct timespec *maxWaitTime=&OrderedInputSource::WAIT_YEAR_MAX) OME_ALWAYS_OPTIMIZE("-O3")
    {
        OME_PREFETCH(&priorityQueue, 1, 3); // going to be modified, long-lived
        if (OME_EXPECT_FALSE(priorityQueue.getQueueLength() == 0)) { // nothing to do...
            return (0);
        }
        // to get past here, there must have been at least 1 block queued
        uint32_t fakeStopFlag = 0; // storage for stopFlag if none provided
        uint32_t *flagPtr = (stopFlagPtr != nullptr) ? stopFlagPtr : &fakeStopFlag;
        OME_PREFETCH(flagPtr, 0, 3); // read-only, used for a long time

        int64_t totalBlocks = 0;
        // peek at initial block
        OrderedInputBlock *block = priorityQueue.peekNextBlock();
        uint64_t nextSortKey = block->sortKey; // local copy of block->sortKey
        uint64_t maxTimeNS;
        if (OME_EXPECT_FALSE(maxWaitTime == nullptr)) {
            maxTimeNS = ~static_cast<uint64_t>(0U);
        } else {
            maxTimeNS = (static_cast<uint64_t>(maxWaitTime->tv_sec) * 1000000000U) +
                maxWaitTime->tv_nsec;
            if (maxTimeNS < (static_cast<uint64_t>(24 * 60 * 60) * 1000000000U)) { // treat relative to start of next block
LOG_COMPONENT_CERR(io,trace) << " Set interval of " << maxTimeNS << " to end at " << POSIXtimeInNanoseconds(0, nextSortKey + maxTimeNS) << LOG_ENDLINE;
                maxTimeNS += nextSortKey;
            } else { // was an absolute time
                if (nextSortKey >= maxTimeNS) {
                    // return without doing anything
LOG_COMPONENT_CERR(io,trace) << "Already at end of interval " << nextSortKey << LOG_ENDLINE;
                    return (0); // do nothing
                }
            }
        }
        // from this point onwards, we are committed to at least handle block
#define _TRACK_RANGE_PROCESSED 1
#if _TRACK_RANGE_PROCESSED == 1
uint64_t firstSortKey = nextSortKey;
uint64_t lastSortKey = nextSortKey;
#endif
        priorityQueue.removeNextBlock(); // already have top in block
        const bool drainEverything = false; // TODO: currently no way to enable this feature
uint32_t queueModificationCount = 0; // this now counts queue modifications
        do {
            char label[256]; // storage for sourceLabel
            /* NOTE: "block" is the just-popped first entry in the queue, 
             * while "secondBlock" is the next apparent entry
             * in the queue.
             * For extremely intentional performance reasons, only a handful
             * of blocks are ever loaded into the priority queue:  1 + the
             * number of reordering we want to tolerate per source.
             */
            OrderedInputSource *source = block->inputFile;
            OME_PREFETCH(source, 0, 3); // read-only, should be around for a while
            // peek ahead to next block in queue
            OrderedInputBlock *secondBlock;
            uint64_t secondSortKey; // local copy of secondBlock->sortKey
            if (OME_EXPECT_TRUE(priorityQueue.getQueueLength() != 0)) {
                secondBlock = priorityQueue.peekNextBlock();
                secondSortKey = secondBlock->sortKey;
            } else {
                secondBlock = nullptr;
                secondSortKey = ~static_cast<uint64_t>(0U); // max possible value
            }
            int rc = 0;
            do { // process what we can from this source
                totalBlocks += 1;
                // process block
                noteNextKey(nextSortKey, totalBlocks, secondSortKey);
                // invoke abstract function
                rc = processInputBlock(block);
#if _TRACK_RANGE_PROCESSED == 1
                lastSortKey = nextSortKey; // last processed
#endif
                if (OME_EXPECT_TRUE(rc <= 0)) { // recover block
                    source->recoverInputBlock(block); // usually deletes it
                    // block was handled, one way or the other
                    if (OME_EXPECT_FALSE(rc < 0)) { // error or forced EOF
LOG_COMPONENT_CERR(io,trace) << "Next block now null because rc=" << rc << LOG_ENDLINE;
                        block = nullptr;
                        break;
                    }
                }
                // get next block from source
                OrderedInputSource::InputBlockReturn retrievedBlock = source->getNextInputBlock(forceCopy, maxWaitTime);
                block = retrievedBlock.block;
                if (OME_EXPECT_FALSE(block == nullptr)) { // normal EOF from this source
                    break; // this is the exit from the main loop
                }
                nextSortKey = block->sortKey;
                // NOTE: if no second block, secondSortKey is ~0, so test cannot be true
                if (OME_EXPECT_FALSE(nextSortKey > secondSortKey)) {
                    // was past earliest entries from other sources
                    queueModificationCount += 1;
                    if (OME_EXPECT_FALSE(secondSortKey >= maxTimeNS)) { // next one will be past interval
                        LOG_COMPONENT_CERR(io,debug) <<
                            "Second block would be past end of interval " << secondSortKey <<
                            " queueSkips=" << totalBlocks - queueModificationCount <<
                            " totalBlocks=" << totalBlocks <<
#if _TRACK_RANGE_PROCESSED == 1
                            " nsFromFirst=" << secondSortKey - firstSortKey <<
                            " nsFromPrior=" << secondSortKey - lastSortKey <<
#endif
                            " nsToNext=" << secondSortKey - maxTimeNS <<
                            " nextApparentTime=\"" << POSIXtimeInNanoseconds(0, secondSortKey) << "\"" <<
                            LOG_ENDLINE;
                        priorityQueue.pushBlock(block); // just save retrieved block
                        goto cleanup_and_exit; // for performance, we cheat using a goto
                    }
                    /* Blocks were either out-of-order from a given source or we
                     * need to change input sources.
                     * We'll use "secondBlock" but need to save
                     * retrieved block.
                     */
                    // This is where a bunch of push/pops are done when maxReorderedElements > 0
                    source = secondBlock->inputFile; // shift over to new source
                    OrderedInputBlock *newBlock = block; // save temporarily
                    block = secondBlock; // already had pQueue.top()
                    nextSortKey = secondSortKey;
                    OME_PREFETCH(source, 0, 3); // read-only, should be around for a while
                    /* NOTE: the order is intentional here to reduce minimize
                     * number of elements in the priority queue for
                     * performance, but requires the temporary "newBlock"
                     * to hold a pointer; however, a simple register-to-
                     * register copy is cheap.
                     */
                    priorityQueue.removeNextBlock(0); // drop first element, which we have now in block
                    priorityQueue.pushBlock(newBlock, 0); // save for processing later
                    secondBlock = priorityQueue.peekNextBlock(); // update after pop()/push()
                    secondSortKey = secondBlock->sortKey;
                } // end if new block later than earliest from all other sources
                if (OME_EXPECT_FALSE(nextSortKey >= maxTimeNS)) { // beyond interval
                    queueModificationCount += 1;
                    LOG_COMPONENT_CERR(io,debug) <<
                        "Reached end of interval " << nextSortKey <<
                        " queueSkips=" << totalBlocks - queueModificationCount <<
                        " totalBlocks=" << totalBlocks <<
#if _TRACK_RANGE_PROCESSED == 1
                        " nsFromFirst=" << nextSortKey - firstSortKey <<
                        " nsFromPrior=" << nextSortKey - lastSortKey <<
#endif
                        " nsToNext=" << nextSortKey - maxTimeNS <<
                        " nextApparentTime=\"" << POSIXtimeInNanoseconds(0, nextSortKey) << "\"" <<
                        LOG_ENDLINE;
                    priorityQueue.pushBlock(block);
                    goto cleanup_and_exit; // for performance, we cheat using a goto
                } // end if beyond interval boundary
            } while (1);
            // block is now a nullptr, we have seen an EOF
            if (activeInputSources.erase(source) != 0) { // first time
                nowAtEndOfFile(source);
                // always stop processing this input
                activeSourceCount = (uint_fast16_t) activeInputSources.size();
                LOG_COMPONENT_CERR(io,info) << "EOF seen on source=\"" <<
                    source->sourceLabel(label, sizeof(label)) <<
                    "\", activeCount=" << (uint32_t) activeSourceCount <<
                    LOG_ENDLINE;
            } else {
                LOG_COMPONENT_CERR(io,info) << "EOF on source=\"" <<
                    source->sourceLabel(label, sizeof(label)) <<
                    "\" that was already reaped" << LOG_ENDLINE;
            }
            if (OME_EXPECT_FALSE(rc == -2)) { // EOF being forced from source
                // stop processing further input from this source
                LOG_COMPONENT_CERR(io,warn) <<
                    "Ceasing all input processing from source=\"" <<
                    source->sourceLabel(label, sizeof(label)) << "\"" << LOG_ENDLINE;
                break;
            }
            if (priorityQueue.getQueueLength() == 0) { // no more data
                break;
            }
            if (maxWaitTime != nullptr) {
                if (secondSortKey >= maxTimeNS) { // next block to process is beyond interval
                    LOG_COMPONENT_CERR(io,trace) <<
                        "End of interval and reached EOF on source=\"" <<
                source->sourceLabel(label, sizeof(label)) << "\"" <<
                        " totalBlocks=" << totalBlocks <<
                        " queueSkips=" << totalBlocks - queueModificationCount <<
#if _TRACK_RANGE_PROCESSED == 1
                        " nsFromFirst=" << secondSortKey - firstSortKey <<
                        " nsFromPrior=" << secondSortKey - lastSortKey <<
#endif
                        " nsToNext=" << secondSortKey - maxTimeNS <<
                        " nextApparentTime=\"" << POSIXtimeInNanoseconds(0, secondSortKey) << "\"" <<
                        LOG_ENDLINE;
                    break;
                }
            }
            // normal EOF with more data remaining
            LOG_COMPONENT_CERR(io,info) << "Apparent EOF on source=\"" <<
                source->sourceLabel(label, sizeof(label)) << "\"" <<
                LOG_ENDLINE;
            LOG_COMPONENT_CERR(io,info) << "Next source=" << secondBlock->inputFile->sourceLabel(label, sizeof(label)) << LOG_ENDLINE;
            block = secondBlock; // get current top of queue
            nextSortKey = secondSortKey;
            OME_PREFETCH(block, 0, 0); // read-only, should be short term
// reset at top of loop           source = block->inputFile;
            priorityQueue.removeNextBlock();   // remove top entry from queue
            if (OME_EXPECT_TRUE(priorityQueue.getQueueLength() != 0)) {
                secondBlock = priorityQueue.peekNextBlock();
                secondSortKey = secondBlock->sortKey;
            } else {
                secondBlock = nullptr;
                secondSortKey = ~static_cast<uint64_t>(0U);
            }
        } while (OME_EXPECT_TRUE(*flagPtr == 0)); // loop as long as stop flag not set
cleanup_and_exit:
        if (OME_EXPECT_FALSE(drainEverything == true)) { // currently no way to enable this
            purgeQueue();
        }
        LOG_COMPONENT_CERR(io,trace) <<
            "Queue skipCount=" << totalBlocks - queueModificationCount <<
            " totalBlocks=" << totalBlocks <<
            " pQueue=" << priorityQueue.getQueueLength() <<
#if _TRACK_RANGE_PROCESSED == 1
            " duration=" << lastSortKey - firstSortKey <<
#endif
            LOG_ENDLINE;
        return (totalBlocks);
#undef _TRACK_RANGE_PROCESSED
    }

    /*! \brief Process data from the registered
     * collection of OrderedInputSource objects.
     * This is a convenience cover that invokes preloadQueue()
     * and then processQueue().  Due to the invocation
     * of preloadQueue(), it should be called only once.
     * If it returns early due to the maximum wait time
     * having been reached, processQueue() should be called
     * to continue processing.
     *
     * \param stopFlag points to a memory region
     * which will be watched; processing will
     * stop if it is set to a nonzero value.
     * A null pointer can be passed if this is not
     * needed.
     * \param maxWaitTime specifies the maximum amount
     * of time to wait for input from a source.
     * Normally, this is specified as an absolute point
     * in time; however, if the number of seconds is less
     * than that representing a single day, the \c timespec is
     * interpreted as being an offset from the earliest time
     * in the queue.
     *
     * \return The number of blocks loaded is returned.
     */
    int64_t processInputFiles(uint32_t *stopFlag, const struct timespec *maxWaitTime=&OrderedInputSource::WAIT_YEAR_MAX)
    {
        LOG_COMPONENT_CERR(io,trace) << "Do preload queue" << LOG_ENDLINE;
        preloadQueue();

        LOG_COMPONENT_CERR(io,trace) << "call processQueue with " << priorityQueue.getQueueLength() << " intial elements" << LOG_ENDLINE;
        int64_t result = processQueue(stopFlag, maxWaitTime);
        LOG_COMPONENT_CERR(io,trace) << "processQueue result=" << result << LOG_ENDLINE;
        return (result);
    }

    /*! \brief Interface to complete processing any background work that
     * was previously initiated via processQueue() but has not yet finished.
     *
     * \param stopFlagPtr points to a memory region
     * which will be watched; processing will
     * stop if it is set to a nonzero value.
     * A null pointer can be passed if this is not
     * needed.
     * \param maxWaitTime specifies the maximum amount of
     * time to wait for input from a source.
     */
    virtual int64_t processWorkInProgress(uint32_t *stopFlagPtr, const struct timespec *maxWaitTime=&OrderedInputSource::WAIT_YEAR_MAX) {
        return (0);
    }
}; // end class OrderedMultipleInputFilter

//typedef OrderedMultipleInputFilter *OrderedMultipleInputFilterP;

/** @} */

/** @} */
#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
