#ifndef _READ_BINARY_LOG_HPP_
//! \cond
#define _READ_BINARY_LOG_HPP_ "$Id: read_binary_log.hpp 428 2020-07-17 17:16:27Z geoff $"
//! \endcond

#include <utils/io/read_file.hpp>
#include <utils/logging/logging_api.hpp>

/*! \file
 * \brief Read binary log files.
 */

/** \addtogroup FARGOSio
 * @{
 */

/*! \brief Reads a binary log, expands and formats as original plain text.
 * If provided a text file, acts like 'cat'.
 */
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wsuggest-final-types"
class ReadBinaryLog : public Read_And_Process_File {
protected:
    enum { WANT_LINE_HEADER, WANT_FILENAME, WANT_FORMAT, WANT_ARG_HEADER,
        WANT_ARG_VALUE, WANT_LINE_END, CHECK_FOR_BINARY_HEADER, COPY_AS_IS };
    uint32_t                    parseState;
    uint32_t                    processedArgCount;
    uint32_t                    parseOffset;
    uint32_t                    fileNameLen;
    uint32_t                    formatStringLen;
    OS_HANDLE_TYPE              outputToFD;
    const char                  *fileName;
    const char                  *formatString;
    TimePointCache              *timePointCache;
    LogLineBinaryHeader         *lineHeader;
    LogLineBinaryValueHeader    *argHeader[LogMessageRecord::LOG_MAX_ARGUMENT_TOTAL];
    const unsigned char         *argData[LogMessageRecord::LOG_MAX_ARGUMENT_TOTAL];

    // It is possible to override this...but you'd really need to want
    // to do something extremely special...
    virtual int outputAssembledLine();
public:
    ReadBinaryLog(OS_HANDLE_TYPE fd, OS_HANDLE_TYPE outputTo=_STDOUT_FD) : Read_And_Process_File(fd)
    {
        intermediateBuffer = new File_Buffer();
        outputToFD = outputTo;
        parseState = CHECK_FOR_BINARY_HEADER;
        parseOffset = 0;
        timePointCache = 0;
    }

    ~ReadBinaryLog() {
        delete timePointCache;
    }

    OS_HANDLE_TYPE setNewOutputDescriptor(OS_HANDLE_TYPE newFD) {
        OS_HANDLE_TYPE result = outputToFD;
        outputToFD = newFD;
        return (result);
    }

    virtual int processBuffer(File_Buffer *bfrState) VIRTUAL_OVERRIDE;

    //! \brief Writes data to output descriptor.
    virtual int processLine(const unsigned char *line, size_t lineLen) VIRTUAL_OVERRIDE;

}; // end class ReadBinaryLog


/*! \brief Interface and generic implementation to traverse elements
 * in a memory-mapped region under the control of a CircularBufferManager.
 */
class ReadMappedCircularBuffer {
protected:
    CircularBufferManager   *bfrMgr;
    uint32_t                processedBlockCount;
    uint32_t                processedByteCount;
    uint32_t                usedByteCount;
    uint32_t                _pad;

    int traverseBlockChain(const SharedBufferAllocRecord *start);
public:
    ReadMappedCircularBuffer(const char *fileName);

    ReadMappedCircularBuffer(size_t regionLen, unsigned char *region);

    ~ReadMappedCircularBuffer();

    virtual int processBufferContents();

    virtual int processRegionHeader(const SharedBufferRegionHeader *hdr, size_t segmentLen);

    virtual int processEndOfBuffer();

    virtual int processBlock(const SharedBufferAllocRecord *record);

    virtual int processRecord(const unsigned char *data, size_t recLen);

}; // end class ReadMappedCircularBuffer

#pragma GCC diagnostic pop

/** @} */
#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
