#ifndef _LOGGING_API_HPP_
//! \cond
#define _LOGGING_API_HPP_ "$Id: logging_api.hpp 456 2020-07-23 20:25:49Z geoff $"
//! \endcond

/*
 * Copyright (C) 2010 - 2020 FARGOS Development, LLC
 * Author:  Geoff Carpenter - http://www.fargos.net/gcc.html
 */

/*! \file
 * \brief FARGOS Logging API
 *
 * NOTE: many of the convenience macros in this API expect the
 * <tt><b>THIS_COMPONENT</b></tt> macro to be eventually defined by the user.
 * It does not have to be set before the logging_api.hpp header
 * file is included, but it will need to be defined before the relevant
 * convenience macros are used:
 *
 * EMIT_CONDITIONAL_MESSAGE(), ALWAYS_EMIT_MESSAGE(),
 * INIT_STANDARD_LOG_FOR_THIS_COMPONENT(),
 * DEFINE_STANDARD_VARS_FOR_THIS_COMPONENT().
 * THIS_COMPONENT_LEVEL(), THIS_COMPONENT_LOG_MASK
 *
 * There is no <tt><b>DECLARE_STANDARD_VARS_FOR_THIS_COMPONENT()</b></tt>
 * because declarations are typically placed in header files,
 * which in turn may be consumed by different components.
 * As a general principle, the content of the header file is not intended to
 * change based on the definition of <tt><b>THIS_COMPONENT</b></tt>.
 *
 * The default LogManager object used by the convenience macros is specified 
 * by the \ref THIS_COMPONENT_LOG_MANAGER macro. If something other than
 * LogManager::DEFAULT_LogManager is desired, the macro should be set prior
 * to including this header file.
 *
 * The LOG_WHEN(), LOG_ALWAYS(), \ref ENDREC macros permit logging output using
 * the "<<" operator.  In the expected case, <tt><b>LOG_USING_OSTREAM</b></tt>
 * will be undefined; however, before including this file, it can be defined to
 * something like <tt><b>std::cout</b></tt> to force these macros to simply
 * output to an <tt><b>ostream</b></tt>-like object rather than use
 * a LogMessageRecord.
 *
 * \sa http://www.fargos.net/packages/LoggingAPI.html
 * \author Geoff Carpenter
 * \copyright 2010 - 2020 FARGOS Development, LLC
 */

// Include headers needed for convenience templates related to networking
#ifdef _WIN32
#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0600
#else
#if _WIN32_WINNT < 0x0600
#warning "Need to support Windows Vista or later for inet_ntop() support"
#endif
#endif
#include <winsock2.h>
#include <ws2tcpip.h>
/* end ifdef _WIN32 */
#else
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/un.h>
#endif

#include <utils/base/compiler_hints.h>
#include <stdint.h>
#include <memory.h>

#include <utils/base/circular_bfr.hpp>
#include <utils/io/io_processor.hpp>
#include <utils/shared_vars/shared_variable.hpp>
#include <utils/time/time_point.hpp>
#include <iostream>

/** \defgroup FARGOSlogging FARGOS Logging API
 * \brief High-performance multi-threaded logging APIs that support both
 * textual and binary output.
 * @{
 */


/*! \brief Permits use of \b gettid() rather than \b pthread_self() for thread
 * Id on Linux.
 *
 * \note \b gettid() is about 30 times slower than \b pthread_self(),
 * so do not enable its usage unless the external process Id is truly required.
 */
#ifndef USE_GETTID_FOR_THREADID
#define USE_GETTID_FOR_THREADID 0
#endif
#if __linux__ && USE_GETTID_FOR_THREADID == 1
#include <sys/syscall.h>
#endif

#ifndef COMPILE_DEBUG_LEVEL
#define COMPILE_DEBUG_LEVEL ~0
#endif

#define DEFAULT_LOG_DIRECTORY_ENVIRONMENT_VARIABLE "LOGDIR"

/*! \brief Magic number for shared memory log file buffer */
#define LOGFILE_BFR_MAGIC_NUMBER "LOGCBFR"

typedef uint_fast32_t LogMaskPrimitiveType_t;
typedef SMV_StandaloneNumeric<LogMaskPrimitiveType_t> LogMaskType_t;

/*! \brief Descriptive meta-data for log file-related information.
 */
struct LogSubsystemInfo {
    /* TODO: progName could be changed to a char[52], keeping this block of info
     * in a single cache line.  Downsides are 1) initialization requires
     * copying data and 2) a fixed limit on maximum program name would be
     * set.
     */
    const char      *programName;       /*!< current process name */
    uint32_t        processID;          /*!< current process Id */
    uint32_t        maxPendingLogLines; /*!< max log lines that can be buffered */
    uint32_t        defaultLogLevel;    /*!< default logging information level */
    uint32_t        defaultLogPrefix;   /*!< flags defined in LogRecord::logPrefixMask */
    unsigned char   useThreads;
    unsigned char   enableMap;
};

/*! \brief Default descriptive information for Logging subsystem
 */
extern struct LogSubsystemInfo default_LogSubsystemInfo;

/*! \brief Header record for a binary output log line.
 *
 * NOTE: these fields must be unambiguous in size and type,
 * and consistently interpreted regardless of compiler or target platform.
 */
struct LogLineBinaryValueHeader {
    // argument type has to be minimum of 16-bits
    uint32_t        argType;        //!< type of argument
    uint32_t        argLength;      //!< actual length of argument
    uint32_t        offsetNextArg;  //!< offset to the next argument
    unsigned char   headerLen;      //!< length of this header structure
    unsigned char   _reserved[3];   // alignment, available for future use
};

/*! \brief Represents a single binary log element.
 */
struct LogLineBinaryValue {
    struct LogLineBinaryValueHeader hdr;
    union LogLineBinaryValue_union {
        char            asChars[8];
        int32_t         i32;
        uint32_t        u32;
        int64_t         i64;
        uint64_t        u64;
        float           f;
        double          d;
        unsigned char   bytes[8];
    } value;
};

/*! \brief Header for encoded log line template and arguments.
 */
struct LogLineBinaryHeader {
    /*! \brief Version identifier for header layout */
    unsigned char   formatVersion;
    /* \brief A value that denotes both byte order and floating point format.
     */
    unsigned char   byteOrder;
    unsigned char   logPrefixMask;
    unsigned char   _reserved[1];
    uint32_t        logLineLength;  /*!< 0 if unknown */
    uint64_t        nanosecondTimestamp;
    uint64_t        threadId;

    uint32_t        lineNumber;
    uint32_t        importanceLevel;

    uint32_t        argumentCount;
    uint32_t        offsetFileName;

    uint32_t        offsetFormatString;
    uint32_t        offsetFirstArg;
    /* more fields can be added here, as earlier decoders will skip over
     * the content and pick up the data from offsetFileName, offsetFormatString,
     * and offsetFirstArg
     */
    /* implied field:  fileName[offsetFormatString - offsetFileName] */
    /* implied field:  formatString[offsetFirstArg - offsetFormatString] */
};

enum {
    //! \brief Denotes version of 1 of the header format
    LOG_LINE_BINARY_FORMAT_VER1 = '1',
    //! \brief Indicates little-endian, IEEE 754 floating point format
    LOG_LINE_BINARY_FORMAT_LE = 'L',
    //! \brief Indicates big-endian, IEEE 754 floating point format
    LOG_LINE_BINARY_FORMAT_BE = 'B',
    //! \brief Indicates big-endian, IBM System/360 hexadecimal floating point 
    LOG_LINE_BINARY_FORMAT_BE_S360 = 'I'
};

/*! \brief Structure defined to generate unique type name */
struct StringInROM_struct {
    char    stringData[1];
};

//! \brief Typedef to mark string data in text segment
typedef const struct StringInROM_struct *StringInROM;

/*! \brief Very short macro to encapsulate content intended
 * to be flagged as "permanent" String-in-ROM text.
 */
#define SiR(x) ((StringInROM) (x))

/*!
 * \brief Placeholder structure to reference a text fragment
 */
class TextBlock_struct {
public:
    const char      *textBfr;
    uint_fast32_t   bfrLen;

    /* \brief Scan source block and clip length at first null character
     *
     * \param bfr points to a conventional C string
     */
    inline explicit TextBlock_struct(const char *bfr) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
        // with GNU_SOURCE, could replace with rawmemchr() call
        textBfr = bfr;
        bfrLen = (uint_fast32_t) strlen(bfr);
    }

    /* Note block of known length and optionally clip output
     * length at first null character.
     *
     * \param bfr points to the string
     * \param len specifies the known length of the string
     * \param clipAtNull is a Boolean that indicates if the
     * string length should be clipped at the first null character
     * encountered before the end of the block.  If false,
     * the block length is taken from \ref len.
     */
    inline TextBlock_struct(const char *bfr, uint_fast32_t len, bool clipAtNull=false) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
        textBfr = bfr;
        if (clipAtNull == false) {
            bfrLen = len;
        } else {
            // with GNU_SOURCE, could replace with strnlen() call
            const char *end = (const char *) memchr(bfr, 0, len);
            bfrLen = (end == nullptr) ? len : end - bfr;
        }
    }
}; // end TextBlock_struct


//! \brief Default set of characters to be escaped by EscapedTextBlock_struct
#define _DEFAULT_ESCAPED_CHARACTERS "\"\n\r\t\b\\"
//! \brief Default replacement characters for EscapedTextBlock_struct
#define _DEFAULT_REPLACEMENT_CHARACTERS "\\\"\0\\n\0\\r\0\\t\0\\b\0\\\\\0"
//! \brief Default replacement for null character by EscapedTextBlock_struct
#define _DEFAULT_NULL_REPLACEMENT "\\0"

/*!
 * \brief Structure to hold table of characters that should
 * be replaced or deleted.
 *
 * By default, characters are left as-is.
 * The set of non-null characters to be modified or deleted is specified in
 * one set; the replacement for null is specified separately.
 * The replacement characters are specified as a null-separated groups of
 * characters.
 */
class Escaped_Replacement_Characters {
public:
    enum { MAX_REPLACEMENT_LENGTH = 3 };
    int8_t          escapedLen[256];
    char            escapedChar[256][MAX_REPLACEMENT_LENGTH+1];

    /*! \brief Initialize escape table
     *
     * \param quoteTheseChars specifies the set of non-null characters to
     * be modified or replaced.
     * \param escapeUsing specifies the a null-separated group of replacement
     * characters for each corresponding character in \a quoteTheseChars.
     * If the group is empty, the character is deleted.
     * \param convertNULL specifies the treatment of the null character.
     *
     * NOTE: currently, a maximum of 3 characters can be replaced for
     * a given character.
     */
    void initEscapes(const char *quoteTheseChars,
            const char *escapeUsing, const char *convertNULL) OME_ALWAYS_INLINE
    {
        OME_PREFETCH(escapeUsing, 0, 0);  // read-only, once only
        memset(escapedLen, 0xff, sizeof(escapedLen)); // initialize with -1

        const char *start = escapeUsing;
        uint_fast16_t quotableLen = 0;
        do {
            const unsigned char c = reinterpret_cast<const unsigned char *>(quoteTheseChars)[quotableLen];
            if (c != '\0') {
                uint_fast8_t i = 0;
                while (*start != '\0') {
                    escapedChar[c][i] = *start;
                    start += 1;
                    i += 1;
                }
                escapedChar[c][i] = '\0';
                escapedLen[c] = i;
                start += 1; // skip null
            }
            quotableLen += 1;
        } while (quoteTheseChars[quotableLen] != '\0');
        if (convertNULL != nullptr) {
            OME_PREFETCH(convertNULL, 0, 0);  // read-only, once only
            start = convertNULL;
            uint_fast8_t i = 0;
            while (*start != '\0') {
                escapedChar[(unsigned char) '\0'][i] = *start;
                start += 1;
                i += 1;
            }
            escapedChar[(unsigned char) '\0'][i] = '\0';
            escapedLen[(unsigned char ) '\0'] = i;
        }
    }

    /*! \brief Constructor that does no preparation.
     *
     * None of the arrays will be initialized with content.
     *
     * If intended to be used, the arrays should be populated via
     * a subsequent call to initEscapes().
     */
    explicit Escaped_Replacement_Characters(bool doNothing) {}

    /*! \brief Constructor that dynamically prepares content.
     *
     * \param quoteTheseChars specifies the characters to
     * be quoted.
     * \param escapeUsing specifies the replacement pattern.
     * \param convertNULL specifies the replacement for a null
     * character.
     */
    explicit Escaped_Replacement_Characters(
            const char *quoteTheseChars=_DEFAULT_ESCAPED_CHARACTERS,
            const char *escapeUsing=_DEFAULT_REPLACEMENT_CHARACTERS,
            const char *convertNULL=_DEFAULT_NULL_REPLACEMENT) OME_ALWAYS_INLINE {
        initEscapes(quoteTheseChars, escapeUsing, convertNULL);
    }

}; // end Escaped_Replacement_Characters

/*!
 * \brief Placeholder structure to reference a text fragment
 * that will have selected characters transformed or deleted.
 *
 * By default, characters are left as-is.
 * The set of non-null characters to be modified or deleted is specified in
 * one set; the replacement for null is specified separately.
 * The replacement characters are specified as a null-separated groups of
 * characters.
 *
 * The constructors take a pointer to a text string with an optional length
 * or character.
 * By default, they use a precomputed Escaped_Replacement_Characters block,
 * but an alternate replacement can be explicit specified or a dynamic one will
 * be populated if the replacement strings are also provided.
 */
class EscapedTextBlock_struct {
public:
    //! \brief Pre-generated default escape patterns
    static Escaped_Replacement_Characters defaultEscapePatterns;

    /* NOTE:  escapedLen is examined for EVERY character in the string, so it
     * is maintained as a dense array to maximize the amount of useful
     * content held within a cache line.
     * In contrast, escapedChar is only visited as needed to handle
     * the exceptional cases, so it is maintained separately.
     */
    const char      *textBfr;           //!< points at text to be processed
    uint_fast32_t   bfrLen;             //!< specifies length of text string
    char            singleOutputChar;   //!< storage for single character
    const Escaped_Replacement_Characters    *replacementsToUse; //!< effective replacement characters
    Escaped_Replacement_Characters          dynamicReplacements;//!< storage for dynamically assigned replacements

#if 0
protected:
    /*! \brief Initialize escape table
     *
     * \param quoteTheseChars specifies the set of non-null characters to
     * be modified or replaced.
     * \param escapeUsing specifies the a null-separated group of replacement
     * characters for each corresponding character in \ref quoteTheseChars.
     * If the group is empty, the character is deleted.
     * \param convertNULL specifies the treatment of the null character.
     *
     * NOTE: currently, a maximum of 3 characters can be replaced for
     * a given character.
     */
    void initEscapes(const char *quoteTheseChars=_DEFAULT_ESCAPED_CHARACTERS,
            const char *escapeUsing=_DEFAULT_REPLACEMENT_CHARACTERS,
            const char *convertNULL=_DEFAULT_NULL_REPLACEMENT) OME_ALWAYS_INLINE
    {
        dynamicReplacements.initEscapes(quoteTheseChars, escapeUsing, convertNULL);
    }
#endif

public:
    /*! \brief Escape string, determine string length.
     */
    inline explicit EscapedTextBlock_struct(const char *bfr,
            const char *quoteTheseChars,
            const char *escapeUsing=_DEFAULT_REPLACEMENT_CHARACTERS,
            const char *convertNULL=_DEFAULT_NULL_REPLACEMENT) OME_ALWAYS_INLINE :
        replacementsToUse(&dynamicReplacements),
        dynamicReplacements(quoteTheseChars, escapeUsing, convertNULL)
    {
        textBfr = bfr;
        bfrLen = (uint_fast32_t) strlen(bfr);
    }

    /*! \brief Escape string, determine string length.
     *
     * NOTE: this routine implements the default scenario where only a
     * const char pointer is provided.
     */
    inline explicit EscapedTextBlock_struct(const char *bfr,
            const Escaped_Replacement_Characters &useReplacements=defaultEscapePatterns) OME_ALWAYS_INLINE :
        replacementsToUse(&useReplacements),
        dynamicReplacements(false)
    {
        textBfr = bfr;
        bfrLen = (uint_fast32_t) strlen(bfr);
    }

    /*! \brief Escape string of known length
     *
     */
    inline EscapedTextBlock_struct(const char *bfr, uint_fast32_t len,
            const char *quoteTheseChars,
            const char *escapeUsing=_DEFAULT_REPLACEMENT_CHARACTERS,
            const char *convertNULL=_DEFAULT_NULL_REPLACEMENT) OME_ALWAYS_INLINE :
        replacementsToUse(&dynamicReplacements),
        dynamicReplacements(quoteTheseChars, escapeUsing, convertNULL)
    {
        textBfr = bfr;
        bfrLen = len;
    }

    /*! \brief Escape string of known length with precomputed escapes.
     *
     */
    inline EscapedTextBlock_struct(const char *bfr, uint_fast32_t len,
            const Escaped_Replacement_Characters &useReplacements=defaultEscapePatterns) OME_ALWAYS_INLINE :
        replacementsToUse(&useReplacements),
        dynamicReplacements(false)
    {
        textBfr = bfr;
        bfrLen = len;
    }

    /*! \brief Escape std::string
     *
     */
    inline explicit EscapedTextBlock_struct(const std::string &bfr,
            const char *quoteTheseChars,
            const char *escapeUsing=_DEFAULT_REPLACEMENT_CHARACTERS,
            const char *convertNULL=_DEFAULT_NULL_REPLACEMENT) OME_ALWAYS_INLINE :
        replacementsToUse(&dynamicReplacements),
        dynamicReplacements(quoteTheseChars, escapeUsing, convertNULL)
    {
        textBfr = bfr.c_str();
        bfrLen = (uint_fast32_t) bfr.size();
    }

    /*! \brief Escape std::string
     *
     */
    inline explicit EscapedTextBlock_struct(const std::string &bfr,
            const Escaped_Replacement_Characters &useReplacements=defaultEscapePatterns) OME_ALWAYS_INLINE :
        replacementsToUse(&useReplacements),
        dynamicReplacements(false)
    {
        textBfr = bfr.c_str();
        bfrLen = (uint_fast32_t) bfr.size();
    }

    /*! \brief Escape a single character
     *
     */
    inline explicit EscapedTextBlock_struct(const char justThisChar,
            const char *quoteTheseChars,
            const char *escapeUsing=_DEFAULT_REPLACEMENT_CHARACTERS,
            const char *convertNULL=_DEFAULT_NULL_REPLACEMENT) OME_ALWAYS_INLINE :
        replacementsToUse(&dynamicReplacements),
        dynamicReplacements(quoteTheseChars, escapeUsing, convertNULL)
    {
        singleOutputChar = justThisChar;
        textBfr = &singleOutputChar;
        bfrLen = 1;
    }

    inline explicit EscapedTextBlock_struct(const char justThisChar,
            const Escaped_Replacement_Characters &useReplacements=defaultEscapePatterns) OME_ALWAYS_INLINE :
        replacementsToUse(&useReplacements),
        dynamicReplacements(false)
    {
        singleOutputChar = justThisChar;
        textBfr = &singleOutputChar;
        bfrLen = 1;
    }

}; // end EscapedTextBlock_struct

/*!
 * \brief Placeholder structure to reference a block of arbitrary data.
 */
class BinaryBlock_struct {
public:
    const void      *dataBfr;
    uint_fast32_t   bfrLen;

    inline BinaryBlock_struct(const void *bfr, uint_fast32_t len) OME_ALWAYS_INLINE {
        dataBfr = bfr;
        bfrLen = len;
    }
}; // end BinaryBlock_struct

/*!
 * \brief Placeholder structure to reference a block of arbitrary data
 * to be output in hexadecimal
 */
class HexadecimalBlock_struct {
public:
    const void      *dataBfr;
    uint_fast32_t   bfrLen;
    bool            asLowercase;

    inline HexadecimalBlock_struct(const void *bfr, uint_fast32_t len,
            bool useLowercase=true) OME_ALWAYS_INLINE {
        dataBfr = bfr;
        bfrLen = len;
        asLowercase = useLowercase;
    }
}; // end HexadecimalBlock_struct

/*! \brief Recommended default amount of maximum buffered log lines. */
#define SUGGESTED_DEFAULT_MAX_LOG_LINES 1024


/*! \brief Implements multi-threaded, deferred physical I/O log file manager.
 */
class LogManager {
public:
    typedef void (* TimeAcquisitionFP)(struct timespec *, const LogManager *);

    static void defaultGetTime(struct timespec *timestamp, const LogManager *logMgr) {
        interpolated_clock_gettime(timestamp, false);
    }
private:
    //! \brief Class to maintain fileName->shortened replacement record
    class FileNameCacheEntry {
    public:
        const char          *fileName;
        TextBlock_struct    replacementText;

        //! \brief Null constructor for cache entry
        FileNameCacheEntry() : replacementText(nullptr, 0) {
            fileName = nullptr;
        }

    }; // end private FileNameCacheEntry class

protected:
    enum {
        MAX_FILE_PREFIX_ENTRIES = 8,    //!< table size for maximum distinct directory prefixes
        // file name cache should be power-of-2 for best performance
        FILENAME_CACHE_SIZE = 4         //!< max entries in prune filename cache
    };
    /* These static arrays are shared across ALL LogManager instances
     * within a process.
     */
    static uint16_t     filePrefixLen[MAX_FILE_PREFIX_ENTRIES];
    static uint16_t     filePrefix2Len[MAX_FILE_PREFIX_ENTRIES];
    static const char   *stripFilePrefix[MAX_FILE_PREFIX_ENTRIES];
    static const char   *stripFilePrefix2[MAX_FILE_PREFIX_ENTRIES];

    /*! \brief Initializes the file name prefix tables.
     *
     * Called once, either by initializeLogSubsystem(), but usually
     * by the LogManager() constructor via the static constructors
     * that create the STDOUT_LogManager and STDERR_LogManager objects.
     */
    static void initializePrefixTable();

    // TODO:  a new constructor which allows a 
    // BufferRegion to be passed can be added.
    BufferRegion            *bfrMgr;
    IO_Processor            *ioMgr;
    const char              *errorCondition;
    size_t                  blockSize;
    uint_fast32_t           logPrefixMask;
    uint32_t                stripLeadingDirectories;
    uint32_t                _explicitAlignmentPaddding;
    uint32_t                cachedFileNameCount;
    TimeAcquisitionFP       getTimestampRoutine;
    FileNameCacheEntry      fileNameReplacementCache[FILENAME_CACHE_SIZE];
public:
    enum {
        //! \brief Default maximum length of a log line.
        MAX_LOG_LINE_LENGTH=((16 * 4096) + (3 * 64)) // skewed slightly to avoid the same cache line
    };
    TimePointCache          logTimePointCache;
    TimePointCacheTable     argTimeCacheTable;
    char                    logFileName[256]; // filled in by newLogFileForComponent()

    static LogManager *DEFAULT_LogManager;  //!< Default log manager used, set by newStandardLogFile()
    static LogManager *STDOUT_LogManager;   //!< Created automatically
    static LogManager *STDERR_LogManager;   //!< Created automatically

private:
    FileNameCacheEntry findCachedFileName(const char *fileName) const;

    void addCachedFileName(const char *fileNameKey, const TextBlock_struct entry);

public:
    void killProcessingThread() { // for debug only
        ioMgr->stopThread(IO_Processor::PROCESS_THREAD);
    }

    /*! \brief Add definition of new log component
     *
     * \param componentName identifies the component
     * \param maskLocation points to the memory location which will maintain
     * the log mask.
     * \param initialMask specifies the initial log mask for the component.
     *
     * \note The effective log mask is determined by including any corresponding
     * levels set by \c +logmask command arguments and then removing any levels
     * specified by a \c -logmask command line arguments.
     **/
    static uint_fast32_t addLogComponent(const char *componentName,
        LogMaskType_t *maskLocation, LogMaskPrimitiveType_t initialMask=0);

    /*! \brief Get current log prefix mask.  It will be a combination
     * of bits composed from LogMessageRecord::LogPrefixMask enums.
     */
    uint_fast32_t getLogPrefixMask() const OME_ALWAYS_INLINE {
        return (logPrefixMask);
    }

    /*! \brief Set the log prefix mask.  The new mask is defined as
     * a combination of bits created by OR'ing together values
     * from the LogMessageRecord::LogPrefixMask enum.
     *
     * \return The original mask value will be returned as a convenience
     * so that it can be restored after a temporary change.
     *
     * NOTE:  normally the log prefix mask is set at creation time and
     * never touched, thus all log lines would have an identical prefix format.
     * In some rare circumstances, this may not be desirable, such as
     * a multi-line dump of a buffer's contents.
     */
    inline uint_fast32_t setLogPrefixMask(uint_fast32_t newMask) OME_ALWAYS_INLINE {
        uint_fast32_t original = logPrefixMask;
        logPrefixMask = newMask;
        return (original);
    }

    /* \brief Set a fixed number of directory entries to be stripped
     * from source file names when output in a log line.  Default value is
     * obtained from the GCOV_PREFIX_STRIP environment variable, if
     * present; otherwise it defaults to 0.
     */
    void setStripLeadingDirectories(uint_fast32_t dirCount) {
        stripLeadingDirectories = dirCount;
    }

    /*! \brief Initialize logging-specific parameters by passing command
     * line arguments.
     *
     * \param newArgc is a pointer to a location into which a
     * new \a argc value will be placed.
     * \param newArgv is a pointer to an array of const char pointers that
     * will be set to a subset of the argv array after any logging-related
     * arguments have been removed.
     * \param argc is a count of the arguments in argv array
     * \param argv is an array of const char pointers that provide command line
     * arguments.
     * \param componentName is an optional argument which identifies the
     * application or component name.  If not provided, the application name
     * is obtained from argv[0].
     * \param appMaskLocation is an optional argument that points to the
     * memory location which will hold the log mask for the application.
     * If provided, it overrides the always-defined "app" log level mask
     * and references to the component's active log Level mask are aliased
     * to be the same entity as that for "app".
     * \param maxLines indicates maximum number of unwritten log lines to
     * be able to buffer.  The larger the value, the more output lines that
     * can be queued up at the cost of a greater amount of permanently
     * allocated storage.  If multithreaded mode is not used, a very small
     * value, like 1, is viable.
     * \param logPrefixMask is an optional argument that specifies the log
     * line output fields that should be output for each log line.  It defaults
     * to all fields. The LogMessageRecord::LogPrefixMask enum
     * defines the bits of interest.
     *
     * The LogMessageRecord::process_commandline_log_flags() routine is invoked
     * to strip any logging-related arguments from command line argument
     * list and generate a new argc and argv[].
     *
     * \return Currently always returns 0 to indicate success.
     */
    static int initializeLogSubsystem(int *newArgc, const char **newArgv,
        const int argc, const char *argv[],
        const char *componentName=nullptr,
        LogMaskType_t *appMaskLocation=nullptr,
        const uint_fast32_t maxLines=128, const LogMaskPrimitiveType_t logPrefixMask=~0U);

    /* \brief Create a LogManager associated with an already open file descriptor.
     */
    static LogManager *newLogToFileDescriptor(SharedMemoryVariableNode *parentNode,
        OS_HANDLE_TYPE fd, LogMaskPrimitiveType_t logPrefixMask,
        uint_fast8_t useSeparateThreads, unsigned char *region, size_t bytes);

    /*! \brief Create a new file and LogManager for a component.  If desired,
     * also create an auxilary backing file for the buffer and create a
     * background thread to perform the actual output.
     *
     * \param parentNode
     * \param app specifies the application's name
     * \param filenameCreateFlags 
     * \param logPrefixMask specifies the default bit mask for enabled output
     * fields (e.g., time, thread id, file name, line number, etc.).
     * \param enableMap is a Boolean flag indicating if the in-progress work
     * buffer should be maintained in an auxillary memory-mapped file,
     * which allows the in-progress work to be recovered if the process
     * crashes.
     * \param useSeparateThreads is a Boolean flag that selects use of
     * a separate output thread.  By default, the thread that submits a
     * log message is used to make the eventual kernel call to output the
     * data, which can cause it to be blocked waiting for output to complete.
     * Using a separate output thread allows a thread to submit a log line
     * and continue working while the output thread works on processing the
     * submitted work in the background.
     * \param desiredRegionSize specifies how large the memory-mapped
     * auxillary file should be.
     * \param reserveAtEnd specifies the amount at the end of the region
     * which should be left untouched.
     * \param inDir specifies the directory into which the log file should
     * be created.
     *
     * See \ref newLogToFileDescriptor().
     */
    static LogManager *newLogFileForComponent(
        SharedMemoryVariableNode *parentNode,
        const char *app, uint_fast32_t filenameCreateFlags,
        LogMaskPrimitiveType_t logPrefixMask,
        uint_fast8_t enableMap, uint_fast8_t useSeparateThreads,
        size_t desiredRegionSize=0, size_t reserveAtEnd=0,
        const char *inDir=getenv(DEFAULT_LOG_DIRECTORY_ENVIRONMENT_VARIABLE));

    /*! \brief High-level routine to create a standard log file associated
     * with a component.
     *
     * \param namedComponent specifies the application name; if not specified,
     * it will be obtained from the process name provided to
     * initializeLogSubsystem().
     * \param filenameCreateFlags specifies the elements used to compose the log
     * file's name.
     * \param parentNode specifies the node under which the shared memory
     * variables associated with the LogManager should be parented.
     *
     * See \ref newLogFileForComponent().
     */
    static LogManager *newStandardLogFile(const char *namedComponent=nullptr,
            LogMaskPrimitiveType_t filenameCreateFlags=~0U,
            SharedMemoryVariableNode *parentNode=nullptr);

    /*! \brief Provide access to the underlying BufferRegion.
     */
    inline BufferRegion *getBufferManager() const OME_ALWAYS_INLINE {
        return (bfrMgr);
    }

    /*! \brief Provide access to underlying IO_Processor
     */
    inline IO_Processor *get_IO_Manager() const OME_ALWAYS_INLINE {
        return (ioMgr);
    }

    /*! \brief IO_processor-compatible process routine used to
     * write buffered data to an open file.
     *
     * \param rec points at the buffer to be output.
     * \param controller points at the IO_Processor driving the output
     * processing.
     */
    static int commitLogRecord(SharedBufferAllocRecord *rec,
        IO_Processor *controller);

    /*! \brief Create anonymous buffer 
     *
     * \param parentNode is a shared memory naming node under which the
     * statistics will be rooted.
     * \param output_fd defines open descriptor associated with output file.
     * \param useSeparateThread is a Boolean flag indicating if transfer to
     * the log file should be performed by directly by the thread writing
     * data to the log or if should be handled by background thread.
     * \param appName identifies the application tag associated with messages
     * emitted to the log.
     * \param logLinePrefixMask specifies the default bit mask for enabled
     * output fields (e.g., time, thread id, file name, line number, etc.).
     * \param maxLineSize identifies the maximum possible output records
     * that can be buffered before an attempt to write additional data to the
     * log will block.
     * \param region is a pointer to a pre-allocated buffer, which could
     * be a memory-mapped region.  If a null pointer is passed, a buffer
     * will be dynamically allocated.
     * \param bytes indicates the size of the buffer.  If pre-allocated,
     * then \a bytes specifies the amount of space usable in the
     * pre-allocated buffer; otherwise it indicates the amount of space to be
     * dynamically allocated.
     *
     * \note If the buffer is dynamically allocated, it will be freed
     * at the time the LogManager object is deleted.  If the buffer was
     * pre-allocated, the buffer will not be freed.
     */
    LogManager(SharedMemoryVariableNode *parentNode,
        OS_HANDLE_TYPE output_fd, uint_fast8_t useSeparateThread,
        const char *appName,
        uint_fast32_t logLinePrefixMask=~0U,
        uint_fast32_t maxLineSize=(LogManager::MAX_LOG_LINE_LENGTH - sizeof(SharedBufferAllocRecord)),
        unsigned char *region=nullptr, size_t bytes=(LogManager::MAX_LOG_LINE_LENGTH * 64));

    virtual ~LogManager();

    /*! \brief Close the open log file. */
    int closeLog();

    /*! \brief Allocate a buffer for a log record
     *
     * The SharedBufferAllocRecord describing the buffer is returned.
     * These records are special in that they can be used with
     * shared memory segments and use relative offsets rather than
     * physical addresses.
     */
    inline SharedBufferAllocRecord *allocateBuffer() OME_ALWAYS_INLINE
    {
        SharedBufferAllocRecord *result = bfrMgr->allocateBlock(blockSize);
        return (result);
    }

    /*! \brief Return a buffer previously allocated by allocateBuffer().
     *
     * \param rec points at the buffer to be returned.
     */
    inline void returnBuffer(SharedBufferAllocRecord *rec) OME_ALWAYS_INLINE {
        bfrMgr->returnBlock(rec);
    }

    /*! \brief Obtain the physical address and length of a buffer
     * from a SharedBufferAllocRecord, which are data structures
     * intended for use in shared memory segments and utilize
     * relative offsets, enabling simultaneous use by distinct
     * processes that have a segment mapped to different address regions.
     *
     * \param rec specifies the buffer.
     * \param[out] bufferLen is an optional argument that points at a location
     * into which the length of the buffer will be stored.
     */
    unsigned char *bufferAddress(SharedBufferAllocRecord *rec,
        size_t *bufferLen=nullptr) const OME_ALWAYS_INLINE {
        // use IO_Processor context in case header
        // is required before each block.  As an optimization,
        // we could assume no and use:
        //    if (bufferLen != nullptr) { *bufferLen = rec->usedLen; }
        //    return (bfrMgr->blockAddress(rec));
        if (OME_EXPECT_TRUE(ioMgr != nullptr)) {
            return (ioMgr->bufferAddress(rec, bufferLen));
        }
        // We don't have an IO_Processor associated with this
        // LogManager, as would be the case for special formatting buffers.
        if (bufferLen != nullptr) {
            *bufferLen = rec->usedLen;
        }
        return (bfrMgr->blockAddress(rec));
    }

    /*! \brief Low-level routine to add a record to log buffer.
     *
     * \param rec points at the allocation record for the buffer.
     *
     * \return the number of bytes written
     */
    ssize_t writeDataToBuffer(SharedBufferAllocRecord *rec);

    /*! \brief Low-level routine to copy data and add a record to log buffer.
     *
     * \param rec points at the allocation record for the buffer.
     * \param data points to the block of data to be copied into the buffer.
     * \param len indicates the number of bytes to be copied.
     *
     * \return the number of bytes written
     */
    ssize_t copyAndWriteDataToBuffer(SharedBufferAllocRecord *rec,
        const void *data, size_t len);

   /*! \brief Write block of data to the log buffer.
     *
     * \param data points to the block of data to be copied into the buffer.
     * \param len indicates the number of bytes to be copied.
     *
     * When this routine returns, the referenced memory block has been copied
     * and it can be reused.
     */
    ssize_t copyAndWriteData(const void *data, size_t len) {
        SharedBufferAllocRecord *rec = bfrMgr->allocateBlock(len);
        ssize_t result = copyAndWriteDataToBuffer(rec, data, len);
        return (result);
    }

    /*! \brief Return the source file name pruned of any leading
     * directory components as indicated by settings imposed by
     * setStripLeadingDirectories() and the stripFilePrefix table.
     */
    const TextBlock_struct pruneDirectoryPrefix(const char *fileName, const int_fast32_t fileNameLen);

    /*! \brief Return error condition text detected.  This is
     * normally used to obtain an explanation of errors reported
     * by the operating system when attempting to write log records.
     */
    const char *getErrorCondition() const OME_ALWAYS_INLINE {
        return (errorCondition);
    }

    /*! \brief Set error condition text.
     */
    void setErrorCondition(const char *messageText) OME_ALWAYS_INLINE {
        errorCondition = messageText;
    }

    /*! \brief Set alternate timestamp acquisition routine.
     * \param routine is the entry point of the function.  If
     * a null pointer is provided, the defaultGetTime() routine
     * is used.
     */
    void setTimeAcquisitionRoutine(TimeAcquisitionFP routine) {
        if (routine != nullptr) {
            getTimestampRoutine = routine;
        } else {
            getTimestampRoutine = defaultGetTime;
        }
    }

    /*! \brief Return pointer to the time acquistion
     * routine associated with the log.
     */
    inline TimeAcquisitionFP getTimeAcquisitionRoutine() const OME_ALWAYS_INLINE {
        return (getTimestampRoutine);
    }

    /*! \brief Return the current time using the timestamp
     * acquisition routine associated with the log.
     * \param timestamp points to the timespec structure
     * into which the current time will be stored.
     */
    inline void getTimestamp(struct timespec *timestamp) const OME_ALWAYS_INLINE {
        (*getTimestampRoutine)(timestamp, this);
    }

    /*! \brief Convenience routine to set location of a custom time
     * value for log lines.
     *
     * \param timeSource points to the memory location in which the current
     * log line time will be maintained.
     *
     * Under normal usage scenarios, a call to
     * setTimeAquisitionRoutine(LogManager::returnCustomLogTime)
     * should subsequently be made on the relevant LogManager objects.
     */
    static void setSourceForCustomLogTime(POSIXtimeInNanoseconds *timeSource);

    /*! \brief Convenience routine to retrieve a custom timestamp for log lines.
     *
     * \param timestamp points to the structure into which the current log line
     * time will be stored.
     * \param logMgr references the LogManager for the log file being processed.
     * It is ignored in this implementation, as it only supports a single time
     * source.
     */
    static void returnCustomLogTime(struct timespec *timestamp, const LogManager *logMgr);


}; // end class LogManager

/*! \brief Convenience mechanism to automatically register
 * a log component.
 *
 * Intended for use in components implemented by (shared) libraries.
 * Invokes LogManager::addLogComponent().
 */
class AutoRegisterLogComponent {
public:
    AutoRegisterLogComponent(LogMaskType_t &var,
        const char *name=nullptr, LogMaskPrimitiveType_t defaultMask=0) {
#if 0
        if (defaultMask != 0) { // if we have a mask set, set it as initial default
            var = defaultMask;
        }
#endif
        LogManager::addLogComponent(name, &var, defaultMask);
    }

    ~AutoRegisterLogComponent() {}
}; // end class AutoRegisterLogComponent


/*! \brief Log message record used to remember output format
 * and argument values, potentially deferring formatting to a later
 * point in time or by an external process.
 */
class LogMessageRecord
#if __cplusplus >= 201100L
    final // not intended to be used as a base class
#endif
{
public:
    enum {
        //! \brief Maximum number of arguments for an output line.
        LOG_MAX_ARGUMENT_TOTAL = 3600,
        //! \brief Minimum amount of string bytes worth an iovec entry
        MIN_IOV_UNIT_LEN = 64
    };

    //! \brief Log line prefix attributes
    enum LogPrefixMask {
        LOG_DATESTAMP=1,            //!< Include date on logline
        LOG_TIMESTAMP=2,            //!< Include time on logline
        LOG_THREAD_ID=4,            //!< Include thread id on logline
        LOG_FILENAME=8,             //!< Include source filename on logline
        LOG_LINE_NUMBER=16,         //!< Include source line number on logline
        LOG_ERROR_LEVEL=32,         //!< Include level indicator (INFO, ERROR, etc.)
        LOG_BINARY_FORMAT_STRING=128 //!< For binary output, include format string
    };

    /*! \brief Standard bit flags to define severity level.
     * There is a partial order of implied importance here,
     * but each level is distinct and filtering one level does not
     * imply anything about another.  User-defined masks can
     * start at LOG_SEVERITY_USER onwards. These are not normally
     * referenced directly, but instead by the aliases introduced by
     * DEFINE_COMPONENT_LOG_MASKS().  A non-standard level name
     * can be introduced by declaring something similar to:
     *   enum { COMPONENT_LEVEL(component,level) = (1 << N) };
     * where N is some unused bit position (e.g., 11 or more).
     * Example:  enum {
     *     COMPONENT_LEVEL(myComponent,incoming) = (1 << 11),
     *     COMPONENT_LEVEL(myComponent,outgoing) = (1 << 12)  };
     * See \ref DECLARE_COMPONENT_USER_LOG_MASK() and
     * \ref DEFINE_COMPONENT_USER_LOG_MASK() for related convenience macros.
     */
    enum LogLevelSeverityMask {
        // reserve lower 4 bits for flags and attributes
        LOG_SEVERITY_TRACE  = (1 << 4),
        LOG_SEVERITY_DEBUG  = (1 << 5),
        LOG_SEVERITY_INFO   = (1 << 6), 
        LOG_SEVERITY_WARN   = (1 << 7),
        LOG_SEVERITY_ERROR  = (1 << 8),
        LOG_SEVERITY_FATAL  = (1 << 9),
        LOG_SEVERITY_USER   = (1 << 10),
        AND_USE_SYSLOG =    0x1,
        AND_USE_STDERR =    0x2,
        AND_SPECIAL_MASK =  0xf
    };

    //! \brief Union to hold arbitrary argument value.
    union LOG_argument_union {
        char            asChars[sizeof(char *)];
        unsigned char   bytes[sizeof(char *)];
        unsigned char   uc;
        int32_t         i32;
        uint32_t        u32;
        int64_t         i64;
        uint64_t        u64;
        float           f;
        double          d;
        const void      *ptr;
        const char      *asCstring; // to coerce debugger into displaying content
        const unsigned char *asUCstring;
    };

    /*! \brief Argument type indicators, reuses those from the
     * SharedMemoryVariables.
     */
    enum LogArgumentType {
        LOG_ARG_TYPE_INT32=SharedMemoryVariable::SMV_TYPE_INT32,
        LOG_ARG_TYPE_UINT32=SharedMemoryVariable::SMV_TYPE_UINT32,
        LOG_ARG_TYPE_INT64=SharedMemoryVariable::SMV_TYPE_INT64,
        LOG_ARG_TYPE_UINT64=SharedMemoryVariable::SMV_TYPE_UINT64,
        LOG_ARG_TYPE_FLOAT=SharedMemoryVariable::SMV_TYPE_FLOAT,
        LOG_ARG_TYPE_DOUBLE=SharedMemoryVariable::SMV_TYPE_DOUBLE,
        LOG_ARG_TYPE_TEXT=SharedMemoryVariable::SMV_TYPE_STRING,
        LOG_ARG_TYPE_CHAR=SharedMemoryVariable::SMV_TYPE_TINY_STRING,
        LOG_ARG_TYPE_BINARY=SharedMemoryVariable::SMV_TYPE_TINY_BINARY_STRING,
        LOG_ARG_TYPE_POINTER=128,
        LOG_ARG_TYPE_ROM_TEXT = LOG_ARG_TYPE_POINTER | LOG_ARG_TYPE_TEXT,
        LOG_ARG_TYPE_TEXT_FRAGMENT = LOG_ARG_TYPE_POINTER | LOG_ARG_TYPE_CHAR,
        LOG_ARG_TYPE_HEX_FRAGMENT = LOG_ARG_TYPE_POINTER | LOG_ARG_TYPE_CHAR | LOG_ARG_TYPE_FLOAT,
        LOG_ARG_TYPE_HEX_FRAGMENT_UPPER = LOG_ARG_TYPE_POINTER | LOG_ARG_TYPE_CHAR | LOG_ARG_TYPE_DOUBLE,
        LOG_ARG_TYPE_POSIX_NANOSECONDS = 256 | LOG_ARG_TYPE_UINT64,
        LOG_ARG_TYPE_FIXED_POINT = 512 | LOG_ARG_TYPE_UINT64
    };
    /*! \brief Typedef for length of field to store argument length
     *
     * This is used within a single address space, so it can be
     * the most appropriate for the target environment.
     * This flexiblity is not present in the argLength field
     * of the \ref LogLineBinaryValueHeader structure.
     */
    typedef uint32_t LogArgumentLenType_t;

    LogManager  *logMgr;            //!< Points to LogManager used for output
    const char  *logLineFormat;     //!< Points format line for log line
    const char  *sourceFile;        //!< Points to source file, usually from __FILE__
    struct timespec timestamp;      //!< Timestamp of log event creation.
    uint64_t    threadId;           //!< Thread Id of log event creator.
    uint32_t    sourceFileNameLen;  //!< Length of sourceFile path
    uint32_t    argCount;           //!< Number of arguments for log line
    uint32_t    sourceLine;         //!< Line in source file, usually from __LINE__
    uint32_t    importanceLevel;    //!< Value from LogLevelSeverityMask enum
    uint32_t    generatedOffset;    //!< End of log line generated via << operator.
    uint32_t    argCountExceeded;
    char        generatedFormatLine[LogManager::MAX_LOG_LINE_LENGTH - 4];

    struct ArgumentRecord {
        LOG_argument_union      logArgs;
        LogArgumentType         argBaseType;
        LogArgumentLenType_t    argLen;
    } argList[LOG_MAX_ARGUMENT_TOTAL];

    /*! \brief Split the content of a null-terminate string across
     * multiple log lines, breaking the lines at appropriate points
     * (such as punctuation).
     *
     * \param context is a reference to the log record being constructed.
     * \param line points at the text to be split across multiple lines
     * \param separators specifies the characters after which the line can
     * broken.
     * \param blockLength specifies the maximum number of permitted characters
     * on a line.
     */
    static void splitLongLogLine(LogMessageRecord &context,
            const char *line, const char *separators=",;",
            size_t blockLength=LogManager::MAX_LOG_LINE_LENGTH);

    /*! \brief Define a new log level name for the command line
     * parser.
     *
     * \param flagName defines the name of the custom log level
     * that will be added to the standard names such as \b info and \b error.
     * \param mask defines the affected logging levels.
     * \param label is an optional label that provides an
     * level label to be output rather than standard labels such as "INFO"
     * and "ERROR".
     */
    static int define_commandline_flag(const char *flagName,
            LogMaskPrimitiveType_t mask,
            const char *label=nullptr);

    /*! \brief Process the standard argument list provided to an
     * application and strip out the logging-related options.
     *
     * \param info points at the LogSystemInfo object to be initialized,
     * usually default_LogsubsystemInfo.
     * \param argc specifies the number of arguments in the \b argv array.
     * \param argv is an array of pointers to the application's command line
     * arguments.
     * \param newArgc is a pointer to a variable into which the new argument
     * count should be stored.  Usually specified as the address of the
     * original \b argc.
     * \param newArgv is a pointer to a new array of pointers to character
     * strings.  Usually specified as the original \b argv.
     *
     * The most common usage is to perform this call immediately at the
     * start of a program's \b main() routine:
     *
     * \code
     * int main(int argc, const char *argv[])
     * {
     *     // parse and strip log-related arguments
     *     LogMessageRecord::process_commandline_log_flags(&default_LogSubsystemInfo,
     *         argc, argv, &argc, argv);
     *     ...
     * }
     * \endcode
     *
     * \sa PROCESS_COMMAND_LINE_FLAGS()
     */
    static int process_commandline_log_flags(LogSubsystemInfo *info,
            int argc, const char *argv[], int *newArgc, const char **newArgv);

    /*! \brief Get the text label for a logging level along with its length
     *
     * \param level specifies the logging level of interest.
     */
    static TextBlock_struct getSeverityLabelWithLength(uint_fast32_t level);

    static const char *getSeverityLabel(uint_fast32_t level);

    LogMessageRecord(LogManager *mgr, uint32_t importanceFlags,
        const char *atFileName, uint32_t atLineNumber,
        int_fast32_t fileNameLen=-1) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
        logMgr = mgr;
        argCount = 0;
        sourceFile = atFileName;
        if (OME_EXPECT_TRUE(fileNameLen != -1)) {
            sourceFileNameLen = fileNameLen;
        } else {
            if (OME_EXPECT_TRUE(atFileName != nullptr)) {
                sourceFileNameLen = strlen(atFileName);
            } else {
                sourceFileNameLen = 0;
            }
        }
        sourceLine = atLineNumber;
        importanceLevel = importanceFlags;
        logLineFormat = nullptr;
        generatedOffset = 0;
        argCountExceeded = 0;
#ifndef _WIN32
#if __linux__ && USE_GETTID_FOR_THREADID == 1
        threadId = (uint64_t) syscall(__NR_gettid); // invoke gettid()
#else
        threadId = (uint64_t) pthread_self();
#endif
#else
        threadId = (uint64_t) GetCurrentThreadId();
#endif
        logMgr->getTimestamp(&timestamp);
    }

    ~LogMessageRecord()   {}

    inline LogManager *getLogManager() const OME_ALWAYS_INLINE {
        return (logMgr);
    }

    /*! \brief Set arbitrary log line format text.
     *
     * \param f specifies the output format prototype.
     *
     * Uses printf() syntax with the following extensions.
     *
     * \arg '%P' is recognized for fixed-point output.
     * \arg '%B' is recognized for binary block output.
     * \arg '%N' is recognized for nanoseconds output as date/time.
     * \arg '%_x' is recognized for lowercase hexadecimal.
     * \arg '%_X' is recognized for uppercase hexadecimal.
     */
    inline void setLogFormat(const char *f) OME_ALWAYS_INLINE {
        logLineFormat = f;
        // TODO: set generatedOffset as strlen(logLineFormat)
    }

    // TODO:  make generateFormatString() protected rather than public
    /*! \brief Append appropriate format parameters for previously
     * added argument.
     *
     * \param fieldNum indicates the field to be appended.
     *
     * \return the length of the complete format string.
     */
    int generateFormatString(unsigned int fieldNum) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
        char *dest = generatedFormatLine + generatedOffset;

        ArgumentRecord &argRec = argList[fieldNum];

        switch (argRec.argBaseType) {
        case LOG_ARG_TYPE_POINTER:
            if (OME_EXPECT_TRUE(argRec.argLen != 0)) {
                memcpy(dest, "%B", 2);
            } else {
                memcpy(dest, "%p", 2);
            }
            dest += 2;
            break;
        case LOG_ARG_TYPE_TEXT_FRAGMENT:
            memcpy(dest, "%B", 2);
            dest += 2;
            break;
        case LOG_ARG_TYPE_HEX_FRAGMENT:
            memcpy(dest, "%x", 2);
            dest += 2;
            break;
        case LOG_ARG_TYPE_HEX_FRAGMENT_UPPER:
            memcpy(dest, "%X", 2);
            dest += 2;
            break;
        case LOG_ARG_TYPE_ROM_TEXT:
            if (OME_EXPECT_FALSE(argRec.argLen > MIN_IOV_UNIT_LEN)) { // more than threshold
                // remember its location for output later
                memcpy(dest, "%s", 2);
                dest += 2;
                break;
            }
            // Too few bytes to waste on an iovec entry
            // NOTE:  FALL THROUGH AND TREAT AS TRANSIENT TEXT
        case LOG_ARG_TYPE_TEXT: { // string is copied to format string
            char *src = (char *) argRec.logArgs.ptr;
            // honor stored length argument
            for(uint_fast32_t _c=0;_c<argRec.argLen;_c+=1) {
                if (OME_EXPECT_FALSE(*src == '%')) {      // escape formatting character ...
                    *(dest++) = '%';    // ...by adding it twice
                }
                *(dest++) = *(src++);
            }
            argCount -= 1; // delete argument
            }
            break;
        case LOG_ARG_TYPE_CHAR:
            // Too few bytes to waste on an iovec entry
            for(uint_fast32_t _c=0;_c<argRec.argLen;_c+=1) {
                char c = argRec.logArgs.asChars[_c];
                if (OME_EXPECT_FALSE(c == '%')) {         // escape formatting character ...
                    *(dest++) = '%';    // ...by adding it twice
                }
                *(dest++) = c;
            }
            argCount -= 1; // delete argument
            break;
        case LOG_ARG_TYPE_FLOAT:
            memcpy(dest, "%g", 2);
            dest += 2;
            break;
        case LOG_ARG_TYPE_DOUBLE:
            memcpy(dest, "%g", 2);
            dest += 2;
            break;
            // TODO:  [u]int_to_ascii() is fast enough that it
            // could make sense to convert to a string right here
            // but then we lose the binary element if the data was
            // assembled using the operator<<().
        case LOG_ARG_TYPE_INT32:
            memcpy(dest, "%d", 2);
            dest += 2;
            break;
        case LOG_ARG_TYPE_UINT32:
            memcpy(dest, "%u", 2);
            dest += 2;
            break;

        case LOG_ARG_TYPE_INT64:
/* we copy the null to make these even in length, which then
 * permits it to be copied via 1 instruction rather than 3.
 */
            memcpy(dest, "%ld", 4);
            dest += 3;
            break;
        case LOG_ARG_TYPE_UINT64:
            memcpy(dest, "%lu", 4);
            dest += 3;
            break;
        case LOG_ARG_TYPE_POSIX_NANOSECONDS: {
            memcpy(dest, "%N", 2); // use our special processing format marker
            dest += 2;
        }
            break;
        case LOG_ARG_TYPE_FIXED_POINT:
            memcpy(dest, "%P", 2); // use our special processing format marker
            dest += 2;
            break;
        default:
            std::cerr << "unknown field type: " << argRec.argBaseType << std::endl;
            argCount -= 1; // delete it
            break;
        } // end switch
        *dest = '\0';
        logLineFormat = generatedFormatLine;
        generatedOffset = (uint32_t) (dest - generatedFormatLine);
        return (generatedOffset); // return length
    } // end generate format string


    /*! \brief Format LogMessageRecord into text, prefixing the line with
     * the indicated attributes.
     *
     * \param prefixWithFlags is a set of bit flags from the LogPrefixMask enum.
     * \param returnLen is an optional pointer to a location into which
     * the final formatted length of the output line will be returned.
     * \param singleThreaded is a Boolean flag indicating if a background
     * output thread is not in use, which enables some optimizations that
     * are possible when a background output thread is not present.
     *
     * A SharedBufferAllocRecord is acquired from the log manager and
     * a text output line is assembled into the buffer.
     *
     * \return the pointer to an allocated SharedBufferAllocRecord is returned.
     */
    SharedBufferAllocRecord *formatRecordAsText(unsigned int prefixWithFlags,
            size_t *returnLen=nullptr, uint32_t singleThreaded=false) OME_ALWAYS_OPTIMIZE("-O3");

    /*! \brief Format the LogMessageRecord as a text output line (using
     * formatRecordAsText()) and pass the resulting buffer to the LogManager
     * via writeDataToBuffer().
     *
     * \param prefixWithFlags is a bit mask that indicates the output fields
     * that should be included on the log line (such as time, thread id,
     * file name, line number, etc.).
     *
     * Whether the output is performed immediately
     * or processed by a background thread is dependent on the mode
     * of the LogManager.
     * \return The number of bytes written is returned.
     */
    size_t writeAsTextToBuffer(unsigned int prefixWithFlags) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3") {
        IO_Processor *ioMgr = logMgr->get_IO_Manager();
        // Technically, if we use a special LogMessageRecord associated
        // with an IO_Manager lacking an IO_Processor, then we would be
        // single-threaded...but we also don't want to perform any
        // physical I/O in that scenario....hence we will indicate
        // we don't want any processing done.
        uint32_t curState = OME_EXPECT_TRUE(ioMgr != nullptr) ? ioMgr->currentThreadState : 0;
        uint32_t okToProcess = curState & IO_Processor::PROCESS_DURING_READ;

        size_t result;
        SharedBufferAllocRecord *rec = formatRecordAsText(prefixWithFlags, &result, okToProcess);
        if (OME_EXPECT_TRUE(rec != nullptr)) {
            result = logMgr->writeDataToBuffer(rec);
        }
        return (result);
    }

    /*! \brief Assemble LogMesageRecord into a suitable PDU format.
     *
     * \param prefixWithFlags is a bit mask that indicates the output fields
     * that should be included on the log line (such as time, thread id,
     * file name, line number, etc.).
     * \param[out] returnLen is an optional pointer that points to a
     * location into which
     * the length of the output record will be stored.
     * \param singleThreaded is a Boolean flag indicating if a background
     * output thread is not in use, which enables some optimizations that
     * are possible when a background output thread is not present.
     *
     * A SharedBufferAllocRecord is acquired from the log manager and
     * the output data is assembled into the buffer.
     *
     * \return the pointer to an allocated SharedBufferAllocRecord is returned.
     *
     */
    SharedBufferAllocRecord *prepareBinaryRecord(unsigned int prefixWithFlags,
            size_t *returnLen=nullptr, uint32_t singleThreaded=false);

    /*! \brief Format the LogMessageRecord as a binary record (using
     * prepareBinaryRecord()) and pass the resulting buffer to the LogManager
     * via writeDataToBuffer().
     *
     * \param prefixWithFlags is a bit mask that selects the fields
     * to be output (e.g., time, thread id, file name, line number, etc.).
     *
     * Whether the output is performed immediately
     * or processed by a background thread is dependent on the mode
     * of the LogManager.
     * \return The number of bytes written is returned.
     */
    size_t writeAsBinaryToBuffer(unsigned int prefixWithFlags) {
        IO_Processor *ioMgr = logMgr->get_IO_Manager();
        uint32_t curState = OME_EXPECT_TRUE(ioMgr != nullptr) ? ioMgr->currentThreadState : 0;
        uint32_t okToProcess = curState & IO_Processor::PROCESS_DURING_READ;

        size_t result;
        SharedBufferAllocRecord *rec = prepareBinaryRecord(prefixWithFlags, &result, okToProcess);
        if (OME_EXPECT_TRUE(rec != nullptr)) {
            result = logMgr->writeDataToBuffer(rec);
        }
        return (result);
    }

    /*! \brief Called when too many arguments are output
     *
     * \param arg references the argument that cannot be output.
     */
    template <typename T> void emitOverflowError(const T &arg) OME_COLD_ROUTINE;

}; // end class LogMessageRecord

template <typename T> inline void LogMessageRecord::emitOverflowError(const T &arg) {
    argCountExceeded += 1;
    std::cerr << "LogMessageRecord overflowEventCount=" << argCountExceeded <<
        " maxArgumentTotal=" << LogMessageRecord::LOG_MAX_ARGUMENT_TOTAL <<
        " location=\"" << sourceFile << ":" << sourceLine << "\"" <<
        " argument=" << arg <<
        std::endl;
}

// logRec and arg are predeclared arguments to function
#define _ADD_NEW_LOG_ARG(typeEnum,member) \
    LogMessageRecord::ArgumentRecord &argRec = logRec.argList[logRec.argCount]; \
    argRec.logArgs. member = arg; \
    argRec.argLen = sizeof(arg); \
    argRec.argBaseType = typeEnum; \
    logRec.argCount += 1

#define _ADD_NEW_LOG_ARG_WITH_VALUE_AND_LEN(typeEnum,member,val,len) \
    LogMessageRecord::ArgumentRecord &argRec = logRec.argList[logRec.argCount]; \
    argRec.logArgs. member = val; \
    argRec.argLen = len; \
    argRec.argBaseType = typeEnum; \
    logRec.argCount += 1

extern void addLogArg(LogMessageRecord &logRec, const int32_t arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const uint32_t arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
#if (__clang__ != 0)
extern void addLogArg(LogMessageRecord &logRec, const long arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const unsigned long arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
#endif

extern void addLogArg(LogMessageRecord &logRec, const int64_t arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const uint64_t arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const size_t arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const ssize_t arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const float arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const double arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const char arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const POSIXtimeInNanoseconds arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const char *arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, StringInROM arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const void *arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const std::string &arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, uint_fast32_t len, const char *arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const void *arg, const size_t l) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const TextBlock_struct &arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const BinaryBlock_struct &arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");
extern void addLogArg(LogMessageRecord &logRec, const HexadecimalBlock_struct &arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

/*! \brief Add log argument of indicated type.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 */
inline void addLogArg(LogMessageRecord &logRec, const int32_t arg)
{
    _ADD_NEW_LOG_ARG(LogMessageRecord::LOG_ARG_TYPE_INT32, i32);
}

/*! \brief Add log argument of indicated type.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 */
inline void addLogArg(LogMessageRecord &logRec, const uint32_t arg)
{
    _ADD_NEW_LOG_ARG(LogMessageRecord::LOG_ARG_TYPE_UINT32, u32);
}

#if (__clang__ != 0)
/*! \brief Add log argument of indicated type.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 */
inline void addLogArg(LogMessageRecord &logRec, const long arg)
{
    _ADD_NEW_LOG_ARG(LogMessageRecord::LOG_ARG_TYPE_INT32, i32);
}

/*! \brief Add log argument of indicated type.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 */
inline void addLogArg(LogMessageRecord &logRec, const unsigned long arg)
{
    _ADD_NEW_LOG_ARG(LogMessageRecord::LOG_ARG_TYPE_UINT32, u32);
}
#endif

/*! \brief Add log argument of indicated type.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 */
inline void addLogArg(LogMessageRecord &logRec, const int64_t arg)
{
    _ADD_NEW_LOG_ARG(LogMessageRecord::LOG_ARG_TYPE_INT64, i64);
}

/*! \brief Add log argument of indicated type.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 */
inline void addLogArg(LogMessageRecord &logRec, const uint64_t arg)
{
    _ADD_NEW_LOG_ARG(LogMessageRecord::LOG_ARG_TYPE_UINT64, u64);
}

/*! \brief Add log argument of indicated type.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 */
inline void addLogArg(LogMessageRecord &logRec, const float arg)
{
    _ADD_NEW_LOG_ARG(LogMessageRecord::LOG_ARG_TYPE_FLOAT, f);
}

/*! \brief Add log argument of indicated type.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 */
inline void addLogArg(LogMessageRecord &logRec, const double arg)
{
    _ADD_NEW_LOG_ARG(LogMessageRecord::LOG_ARG_TYPE_DOUBLE, d);
}

/*! \brief Add log argument of indicated type.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 */
inline void addLogArg(LogMessageRecord &logRec, const char arg)
{
    _ADD_NEW_LOG_ARG(LogMessageRecord::LOG_ARG_TYPE_CHAR, asChars[0]);
}

/*! \brief Add log argument of indicated type.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 */
inline void addLogArg(LogMessageRecord &logRec, const POSIXtimeInNanoseconds arg)
{
    _ADD_NEW_LOG_ARG_WITH_VALUE_AND_LEN(LogMessageRecord::LOG_ARG_TYPE_POSIX_NANOSECONDS, u64, arg.convertToNanosecondsSinceEpoch(), sizeof(uint64_t));
}

/*! \brief Add text string argument.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg points to the string argument be added to the log message.
 *
 * \warning The text string value is not
 * copied immediately and must remain available until either the format line
 * is generated by generateFormatString() or the output line is written.
 */
inline void addLogArg(LogMessageRecord &logRec, const char *arg)
{
    _ADD_NEW_LOG_ARG_WITH_VALUE_AND_LEN(LogMessageRecord::LOG_ARG_TYPE_TEXT, ptr, arg, (LogMessageRecord::LogArgumentLenType_t) strlen(arg));
}

/*! \brief Add text string argument with known length.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param len specifies the number of bytes to be copied from the string.
 * \param arg points to the start of the string contents.
 *
 * \warning The text string value is not
 * copied immediately and must remain available until either the format line
 * is generated by generateFormatString() or the output line is written.
 */
inline void addLogArg(LogMessageRecord &logRec, uint_fast32_t len, const char *arg)
{
    _ADD_NEW_LOG_ARG_WITH_VALUE_AND_LEN(LogMessageRecord::LOG_ARG_TYPE_TEXT_FRAGMENT, ptr, arg, (LogMessageRecord::LogArgumentLenType_t) len);
}

/*! \brief Add text string argument with known length.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 *
 * \warning The text string value is not
 * copied immediately and must remain available until either the format line
 * is generated by generateFormatString() or the output line is written.
 */
inline void addLogArg(LogMessageRecord &logRec, const TextBlock_struct &arg)
{
     if (arg.bfrLen <= 8) { // copy characters to log argument entry
        LogMessageRecord::ArgumentRecord &argRec = logRec.argList[logRec.argCount];
        for(uint_fast8_t i=0;i<arg.bfrLen;i+=1) {
            argRec.logArgs.asChars[i] = arg.textBfr[i];
        }
        argRec.argLen = arg.bfrLen;
        argRec.argBaseType = LogMessageRecord::LOG_ARG_TYPE_CHAR;
        logRec.argCount += 1;
     } else { // point to block
        _ADD_NEW_LOG_ARG_WITH_VALUE_AND_LEN(LogMessageRecord::LOG_ARG_TYPE_TEXT_FRAGMENT,
            ptr, arg.textBfr, (LogMessageRecord::LogArgumentLenType_t) arg.bfrLen);
     }
}

/*! \brief Add binary string argument with known length.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the block of binary data to be added to the log message.
 *
 * \warning The block of data is
 * not copied immediately and must remain available until the output line
 * output line is written.
 */
inline void addLogArg(LogMessageRecord &logRec, const BinaryBlock_struct &arg)
{
     if (arg.bfrLen <= 8) { // copy characters to log argument entry
        LogMessageRecord::ArgumentRecord &argRec = logRec.argList[logRec.argCount];
        for(uint_fast8_t i=0;i<arg.bfrLen;i+=1) {
            argRec.logArgs.bytes[i] = reinterpret_cast<const unsigned char *>(arg.dataBfr)[i];
        }
        argRec.argLen = arg.bfrLen;
        argRec.argBaseType = LogMessageRecord::LOG_ARG_TYPE_CHAR;
        logRec.argCount += 1;
     } else {
        // NOTE: fragment is used rather than POINTER so that a zero-length block
        // is not interpreted as a pointer value.
        _ADD_NEW_LOG_ARG_WITH_VALUE_AND_LEN(LogMessageRecord::LOG_ARG_TYPE_TEXT_FRAGMENT, ptr, arg.dataBfr, (LogMessageRecord::LogArgumentLenType_t) arg.bfrLen);
     }
}

/*! \brief Add text string data which is expected to be permanently available.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg points to the string to be added to the log message.
 *
 * \note The text string value is not copied immediately and is expected
 * to always
 * remain available for access (e.g., string constants, long-lived heap
 * objects are suitable sources).
 */
inline void addLogArg(LogMessageRecord &logRec, StringInROM arg)
{
//std::cerr << "Add ROM: " << (char *) arg << std::endl;
    _ADD_NEW_LOG_ARG_WITH_VALUE_AND_LEN(LogMessageRecord::LOG_ARG_TYPE_ROM_TEXT, ptr, arg, (LogMessageRecord::LogArgumentLenType_t) strlen((const char *) arg));
}

/*! \brief Add log argument of indicated type.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 *
 * \warning The text string value is not
 * copied immediately and must remain available until the format line
 * is generated by generateFormatString() or the output line is written.
 */
inline void addLogArg(LogMessageRecord &logRec, const std::string &arg)
{
    _ADD_NEW_LOG_ARG_WITH_VALUE_AND_LEN(LogMessageRecord::LOG_ARG_TYPE_TEXT, ptr, arg.c_str(), (LogMessageRecord::LogArgumentLenType_t) arg.size());
}

/*! \brief Add log argument of indicated type.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 */
inline void addLogArg(LogMessageRecord &logRec, const void *arg)
{
    _ADD_NEW_LOG_ARG_WITH_VALUE_AND_LEN(LogMessageRecord::LOG_ARG_TYPE_POINTER, ptr, arg, 0);
}

/*! \brief Add log argument representing a pointer.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the argument to be added to the log message.
 * \param l is indicates the length of the content.
 *
 * If the length \a l provided is 0,
 * the pointer itself is the value of interest.  If the length is non-zero, then
 * the region of memory referenced by the argument is the value of interest.
 */
inline void addLogArg(LogMessageRecord &logRec, const void *arg, const size_t l)
{
    _ADD_NEW_LOG_ARG_WITH_VALUE_AND_LEN(LogMessageRecord::LOG_ARG_TYPE_POINTER, ptr, arg, (LogMessageRecord::LogArgumentLenType_t) l);
}


/*! \brief Add binary string argument with known length.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the block of data to be added as hexadecimal data
 * to the log message.
 *
 * \warning The block of data is
 * not copied immediately and must remain available until the output line
 * output line is written.
 */
inline void addLogArg(LogMessageRecord &logRec, const HexadecimalBlock_struct &arg)
{
    // NOTE: fragment is used rather than POINTER so that a zero-length block
    // is not interpreted as a pointer value.
    LogMessageRecord::LogArgumentType t = arg.asLowercase ? LogMessageRecord::LOG_ARG_TYPE_HEX_FRAGMENT : LogMessageRecord::LOG_ARG_TYPE_HEX_FRAGMENT_UPPER;
    _ADD_NEW_LOG_ARG_WITH_VALUE_AND_LEN(t, ptr, arg.dataBfr, (LogMessageRecord::LogArgumentLenType_t) arg.bfrLen);
}

#if 0
//! \brief Templated addLogArg() routine for const char[] arrays,
// which is intended to automatically add constant text strings.
template <unsigned int N> inline void addLogArg(LogMessageRecord &logRec, const char arg[N]) { // this fits the pattern of "text" constants
    addLogArg(logRec, (StringInROM) arg);
}
#endif

/*! \brief Allow message to be built up using the \b << operator
 *
 * \todo This could be optimized by implementing an equivalent
 * to addLogArg() that also does the type-specific work of
 * generateFormatString(), like:  addLogArgWithFormat(logRec, arg, "%d")
 */
#define __EXPLICIT_LOG_MESSAGE_BODY { \
    if (OME_EXPECT_FALSE(logRec.argCount >= LogMessageRecord::LOG_MAX_ARGUMENT_TOTAL)) { \
        logRec.emitOverflowError(arg); \
        return (logRec); \
    } \
    addLogArg(logRec, arg); \
    logRec.generateFormatString(logRec.argCount - 1); \
    return (logRec); \
}

/*! \brief Output a POSIXtimeInNanoseconds into a LogMessageRecord.
 *
 * \param logRec references the LogMessageRecord being prepared.
 * \param arg is the nanoseconds to be added to the log message.
 *
 * \note This is specified
 * to take precedence over a templated version which would cause
 * the value to be immediately converted to text rather than
 * allow it to be acquired for subsequent processing.
 */
extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const POSIXtimeInNanoseconds arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const POSIXtimeInNanoseconds arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const int32_t arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const int32_t arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const uint32_t arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const uint32_t arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

#ifdef __Clang__ /*  __SIZEOF_INT__ == __SIZEOF_LONG__ */
// int and long are same size, so add routines to handle long
extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const unsigned long arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const unsigned long arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const long arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const long arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}
#endif

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const int64_t arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const int64_t arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const uint64_t arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const uint64_t arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

#if (__clang__ != 0)
extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const size_t arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const size_t arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const ssize_t arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const ssize_t arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}
#endif

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const float arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const float arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const double arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const double arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const char arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const char arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, StringInROM arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, StringInROM arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

/*! \brief Convert StringInROM type back to true <tt>const char *</tt>
 * for output on std::ostream
 *
 * \param os references the output stream
 * \param arg is the string to be output.
 */
inline std::ostream &operator<<(std::ostream &os, StringInROM arg)
{
    os << (const char *) arg;
    return (os);
}

/* Convert TextBlock_struct type back to <tt>const char *</tt>
 *
 * \param os references the output stream
 * \param arg is the text block to be output.
 */
inline std::ostream &operator<<(std::ostream &os, const TextBlock_struct &arg)
{
    for(uint_fast32_t i=0;i<arg.bfrLen;i+=1) {
        os << arg.textBfr[i];
    }
    return (os);
}

/* \brief Convert BinaryBlock_struct type back to <tt>const char *</tt> 
 *
 * \param os references the output stream
 * \param arg is the binary data to be output.
 */
inline std::ostream &operator<<(std::ostream &os, const BinaryBlock_struct &arg)
{
    const unsigned char *bfr = static_cast<const unsigned char *>(arg.dataBfr);
    for(uint_fast32_t i=0;i<arg.bfrLen;i+=1) {
        os << bfr[i];
    }
    return (os);
}

/* \brief Convert HexadecimalBlock_struct type back to <tt>const char *</tt>
 *
 * \param os references the output stream
 * \param arg is the data to be output as hexadecimal characters.
 */
inline std::ostream &operator<<(std::ostream &os, const HexadecimalBlock_struct &arg)
{
    const unsigned char *bfr = static_cast<const unsigned char *>(arg.dataBfr);
    if (OME_EXPECT_TRUE(arg.asLowercase)) {
        for(uint_fast32_t i=0;i<arg.bfrLen;i+=1) {
            os << byteAsLowercaseHexadecimal[bfr[i]][0] <<
                  byteAsLowercaseHexadecimal[bfr[i]][1];
        }
    } else {
        for(uint_fast32_t i=0;i<arg.bfrLen;i+=1) {
            os << byteAsUppercaseHexadecimal[bfr[i]][0] <<
                  byteAsUppercaseHexadecimal[bfr[i]][1];
        }
    }
    return (os);
}

/* \brief Convert EscapedTextBlock_struct type back to escaped text
 *
 * \param os references the output stream
 * \param arg is the string to be output.
 */
inline std::ostream &operator<<(std::ostream &os, const EscapedTextBlock_struct &arg)
{
    /* we don't want any characters > 128 to be treated as
     * a negative subscript
     */
    const unsigned char *bfr = reinterpret_cast<const unsigned char *>(arg.textBfr);
    for(uint_fast32_t i=0;i<arg.bfrLen;i+=1) {
        unsigned char c = bfr[i];
        uint8_t l = arg.replacementsToUse->escapedLen[c];
        if (OME_EXPECT_TRUE(l == 0xff)) {
            os << arg.textBfr[i];
        } else if (l != 0) { // some replacement text
            os << arg.replacementsToUse->escapedChar[c];
        }
    }
    return (os);
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const char *arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const char *arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const std::string &arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const std::string &arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const void *arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const void *arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const TextBlock_struct &arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const TextBlock_struct &arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const BinaryBlock_struct &arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const BinaryBlock_struct &arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const HexadecimalBlock_struct &arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const HexadecimalBlock_struct &arg)
{
    __EXPLICIT_LOG_MESSAGE_BODY
}

extern LogMessageRecord &operator<<(LogMessageRecord &logRec, const EscapedTextBlock_struct &arg) OME_ALWAYS_INLINE OME_ALWAYS_OPTIMIZE("-O3");

inline LogMessageRecord &operator<<(LogMessageRecord &logRec, const EscapedTextBlock_struct &arg)
{
    /* we don't want any characters > 128 to be treated as
     * a negative subscript
     */
    const unsigned char *bfr = reinterpret_cast<const unsigned char *>(arg.textBfr);

    uint_fast32_t startOffset = 0;
    for(uint_fast32_t i=0;i<arg.bfrLen;i+=1) {
        unsigned char c = bfr[i];
        uint8_t l = arg.replacementsToUse->escapedLen[c];
        if (OME_EXPECT_TRUE(l != 0xff)) { // some replacement to be done
            uint_fast32_t priorBlockLen = i - startOffset;
            if (priorBlockLen != 0) { // output data before
                BinaryBlock_struct block(arg.textBfr + startOffset, priorBlockLen);
                logRec << block;
            }
            if (l != 0) { // output alternate
                BinaryBlock_struct block(arg.replacementsToUse->escapedChar[c], l);
                logRec << block;
            }
            startOffset = i + 1;
        }
    }
    uint_fast32_t blockLen = arg.bfrLen - startOffset;
    if (blockLen > 0) {
        BinaryBlock_struct block(arg.textBfr + startOffset, blockLen);
        logRec << block;
    }
    return (logRec);
}

/*! \brief Convenience label to enable passing text with known length
 * to output operator<<().
 *
 * \sa TextBlock_struct
 */
#define AS_TEXT_BUFFER(s,...) TextBlock_struct(s,__VA_ARGS__)

/*! \brief Convenience label to enable passing text with known length
 * to output operator<<().
 *
 * \arg buffer points to the text to be output
 * \arg bufferLen
 * \arg quotedCharacters
 * \arg escapeCharacter
 * \arg convertNullCharacter
 *
 * \sa EscapedTextBlock_struct
 */
#define AS_ESCAPED_TEXT_BUFFER(...) EscapedTextBlock_struct(__VA_ARGS__)

/*! \brief Convenience label to enable passing a single character
 * to output operator<<().
 *
 * \arg character
 * \arg quotedCharacters
 * \arg escapeCharacter
 * \arg convertNullCharacter
 *
 * \sa EscapedTextBlock_struct
 */
#define AS_ESCAPED_TEXT_CHARACTER(...) EscapedTextBlock_struct(__VA_ARGS__)

/*! \brief Convenience label to enable passing buffer with known length
 * to output operator<<().
 *
 * \param d points to the start of the data block.
 * \param l specifies the length of the data block.
 *
 * \sa BinaryBlock_struct
 */
#define AS_BINARY_BUFFER(d,l) BinaryBlock_struct(d,l)

/*! \brief Convenience label to enable passing buffer with known length
 * to output operator<<() but output the bytes as hexadecimal characters.
 *
 * \param d points to the start of the data block.
 *
 * \arg Second parameter is length of block.
 * \arg Third parameter is an optional Boolean that indicates if lowercase 
 * is to be used.
 *
 * \sa HexadecimalBlock_struct
 */
#define AS_HEXADECIMAL_BUFFER(d,...) HexadecimalBlock_struct(d,__VA_ARGS__)


/*! \brief Convenience mechanism to automatically register
 * a log level command line flag definition.
 *
 * Intended for use in components implemented by (shared) libraries.
 * Invokes LogMessageRecord::define_commandline_flag().
 */
class AutoRegisterLogLevelName {
public:
/*! \brief Automatically register a log level name with the command line
 * option parser.
 *
 * \param name points to the null-terminated string representing the level name
 * or alias.
 * \param mask is the mask that indicates the levels associated with the
 * level name.
 * \param label is an optional text label like "INFO", "DEBUG", "TRACE", etc.
 *
 * Arguments are passed to LogMessageRecord::define_commandline_flag() as-is.
 */
    AutoRegisterLogLevelName(const char *name, uint32_t mask,
            const char *label=nullptr)
    {
        LogMessageRecord::define_commandline_flag(name, mask, label);
    }

    ~AutoRegisterLogLevelName() {}
}; // end class AutoRegisterLogLevelName

#if 1 
/* NOTE:  this component only works 100% on POSIX systems,
 * but can provide viable results on Microsoft Windows using
 * the standard handles.
 */

/*! \brief Intercept output to an existing file descriptor by taking ownership
 * of its destination, examining all data written to that descriptor, and
 * possibly modifying it before sending it on to the originally intended output
 * destination.
 */
class ReplumbAndLog {
protected:
	OS_HANDLE_TYPE	source_fd;
	OS_HANDLE_TYPE	output_fd;
	LogManager      *logMgr;
	SharedMemoryVariableNode namingNode;
	unsigned char bfrBlock[0x2000];
public:
	static int acceptData(SharedBufferAllocRecord *rec, IO_Processor *controller);

	ReplumbAndLog(OS_HANDLE_TYPE existing_fd, IO_processBlockFP rout=acceptData,
        const char *label = "replumb");

	~ReplumbAndLog()	{}
};

#endif // end ReplumbAndLog

/* NOTE:  "component" must be suitable for a variable name
 * and is not a string
 */

// \brief Concatenate two tokens together to create new variable name.
#define _LOGAPI_APPEND_TOKEN(component,suffix) component ## suffix

/*! \brief Concatenate two tokens together to create new variable name.
 *
 *  \param component specifies the first token
 *  \param suffix specifies the second token
 */
#define LOGAPI_APPEND_TOKEN(component,suffix) _LOGAPI_APPEND_TOKEN(component,suffix)

/*! \brief Construct variable name for a component's active log mask.
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 */
#define COMPONENT_LOG_MASK(component) LOGAPI_APPEND_TOKEN(component,_logLevelMask)

//! \brief Construct variable name for the default component's active log mask
#define THIS_COMPONENT_LOG_MASK COMPONENT_LOG_MASK(THIS_COMPONENT)

/*! \brief Master compile-time log-message disable mask.
 *
 * If not defined, it will default to 0.
 * Command line usage is \c -DISABLE_ALL_LOG_MASK=n
 */
#ifndef ISABLE_ALL_LOG_MASK
#define ISABLE_ALL_LOG_MASK 0
#endif

/*! \brief Defines the name of a component-specific compile-time log message
 * disable mask.
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 *
 * Command line usage is intended to be:
 * \code
 * -DISABLE_component_logMask=n
 * \endcode
 *
 * If defined in source:
 * \code
 * #define ISABLE_component_logMask n
 * \endcode
 * where \a n is a decimal number and \a component is the component id.
 * If undefined, nothing is inhibited.
 * \note As implemented, the value must be expressed as an
 * unadorned decimal number. Symbolic names or parentheses cannot be used;
 * however, the \b setLogMask utility can be used with the \c --value option
 * to generate the mask value from a shell script.
 */
#define DISABLE_COMPONENT_LOG_MASK(component) LOGAPI_APPEND_TOKEN(ISABLE_, LOGAPI_APPEND_TOKEN(component,_logMask))

/*! \brief Boolean expression to determine if component's active log mask
 * has enabled at least one of the indicated levels.
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 * \param mask is the log level mask to be tested
 *
 * While \a mask is not required to be a constant value,
 * this is the expected case.
 * The comparison against component_disableMask is expected to take place at
 * compile-time using constant values and either is a constant false, which
 * causes the entire block to be dropped, or a constant true, in which case
 * just the remaining comparison against the logMask is performed at runtime.
 * The subset of the bit mask that excludes the special mode flags
 * is also determined at compile time.
 * When supported by the underlying compiler, a bias is placed on the code
 * layout with the expectation that trace and debug messages will not normally
 * be emitted.
 */
#define COMPONENT_LOG_ENABLED(component, mask) OME_EXPECT( ( ((mask & LOGAPI_APPEND_TOKEN(component,_disableMask)) == 0 ) && (((uint32_t)(COMPONENT_LOG_MASK(component)) & (mask & ~LogMessageRecord::AND_SPECIAL_MASK)) == (mask & ~LogMessageRecord::AND_SPECIAL_MASK)) ), (static_cast<uint_fast32_t>(mask) & static_cast<uint_fast32_t>(LogMessageRecord::LOG_SEVERITY_DEBUG | LogMessageRecord::LOG_SEVERITY_TRACE | LogMessageRecord::LOG_SEVERITY_USER)) )


/*  \brief Compose symbolic log mask name for a component and level,
 * providing one level of macro expansion.
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 * \param level names the log level (e.g., info, error, trace, etc.)
 */
#define _COMPONENT_LEVEL(component, level) component ## _ ## level

/*! \brief Compose a symbolic log mask name for a component and level
 *
 * \param component specifies the component name
 * \param level names the log level (e.g., info, error, trace, etc.)
 */
#define COMPONENT_LEVEL(component, level) _COMPONENT_LEVEL(component, level)

/*! \brief Compose a symbolic log mask name for the default component and level
 *
 * \param level names the log level (e.g., info, error, trace, etc.)
 */
#define THIS_COMPONENT_LEVEL(level) COMPONENT_LEVEL(THIS_COMPONENT, level)

/*! \brief Enable a symbolic log level for a component
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 * \param level names the log level (e.g., info, error, trace, etc.)
 */
#define ENABLE_COMPONENT_LOG_LEVEL(component, level) COMPONENT_LOG_MASK(component) = (uint32_t) COMPONENT_LOG_MASK(component) | COMPONENT_LEVEL(component,level)

/*! \brief Disable a symbolic log level for a component
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 * \param level names the log level (e.g., info, error, trace, etc.)
 */
#define DISABLE_COMPONENT_LOG_LEVEL(component, level) COMPONENT_LOG_MASK(component) = (uint32_t) COMPONENT_LOG_MASK(component) & ~ COMPONENT_LEVEL(component,level)

/*! \brief Generate compile-time constant representing
 * a user-specific log level for a component.
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 * \param baseLevel identifies a base log level for which this would be a
 * subset, such as \c debug or \c error.  If defining a completely new level,
 * then the keyword \c none should be specified.
 * \param N specifies the relative user-specific mask instance, starting at 1.
 */
#define COMPONENT_USER_LEVEL(component, baseLevel, N) (COMPONENT_LEVEL(component, baseLevel) | (COMPONENT_LEVEL(component, user) * (1 << N)))

/*! \brief Convenience macro that uses \ref COMPONENT_USER_LEVEL()
 * with \ref THIS_COMPONENT as the \a component argument.
 *
 * \param baseLevel identifies a base log level for which this would be a
 * subset, such as \c debug or \c error.  If defining a completely new level,
 * then the keyword \c none should be specified.
 * \param N specifies the relative user-specific mask instance, starting at 1.
 */
#define THIS_COMPONENT_USER_LEVEL(baseLevel,N) COMPONENT_USER_LEVEL(THIS_COMPONENT, baseLevel, N)

/*! \brief Declare a user-specific log mask name.
 *
 * Arguments are the same as for \ref COMPONENT_USER_LEVEL().
 * A new component-specific log level, \a name, will be declared.
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 * \param name specifies the name to be used for the new log level.
 * \param baseLevel identifies a base log level for which this would be a
 * subset, such as \c debug or \c error.  If defining a completely new level,
 * then the keyword \c none should be specified.
 * \param N specifies the relative user-specific mask instance, starting at 1.
 */
#define DECLARE_COMPONENT_USER_LOG_MASK(component,name,baseLevel,N) enum { COMPONENT_LEVEL(component, name) = COMPONENT_USER_LEVEL(component,baseLevel,N) }

/* NOTE:  there are two cases to be handled for treating the compile-time
 * log-message disable mask:  1) the mask was already -Defined, which means
 * it will get expanded to the interesting number or 2) it wasn't, in which
 * case the token is left as-is and looks like a identifier name.
 * That's a big problem for a macro.  We'd like to be able to do:
 *     #ifndef DISABLE_COMPONENT_LOG_MASK(component)
 *     #define DISABLE_COMPONENT_LOG_MASK(component) 0
 *     #endif
 * and have the define declare an object token rather than a function macro,
 * but that's not how it works.
 * This is solved by declaring an identifier almost exactly like the
 * macro name that could be defined, but with a 0 appended to it.  This has one 
 * obvious effect:  it makes a unique name that won't be altered regardless
 * of the definition of the log mask macro.  The choice of 0 is crucial, though,
 * because we understand the mathemetical property of appending a zero to a
 * decimal number: we can simply divide by 10 to obtain the original value.
 * So in the case where the log mask is not defined, we emit code like:
 *   ISABLE_component_logMask0 = 0,
 *   component_disableMask = ISABLE_component_logMask0 / 10,
 * In this case, the prior enum value introduces the identifier we need
 * to allow this to compile.  If the macro had defined as 1234, we'd instead
 * see the last line become:
 *   component_disableMask = 12340 / 10,
 * The compiler does the division at compile time to restore the desired
 * value of 1234. 
 */

/*! \brief Define default symbolic log mask names for a component,
 * providing one level of macro expansion
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not provided as a string.
 */
#define _DEFINE_COMPONENT_LOG_MASKS(component) enum { \
    component ## _none = 0, \
    component ## _trace = LogMessageRecord::LOG_SEVERITY_TRACE, \
    component ## _debug = LogMessageRecord::LOG_SEVERITY_DEBUG, \
    component ## _info = LogMessageRecord::LOG_SEVERITY_INFO, \
    component ## _information = LogMessageRecord::LOG_SEVERITY_INFO, \
    component ## _warn = LogMessageRecord::LOG_SEVERITY_WARN, \
    component ## _warning = LogMessageRecord::LOG_SEVERITY_WARN, \
    component ## _warningAndSyslog = LogMessageRecord::LOG_SEVERITY_WARN | LogMessageRecord::AND_USE_SYSLOG, \
    component ## _warningAndStderr = LogMessageRecord::LOG_SEVERITY_WARN | LogMessageRecord::AND_USE_STDERR, \
    component ## _warningAndBroadcast = LogMessageRecord::LOG_SEVERITY_WARN | LogMessageRecord::AND_USE_SYSLOG | LogMessageRecord::AND_USE_STDERR, \
    component ## _error = LogMessageRecord::LOG_SEVERITY_ERROR, \
    component ## _errorAndSyslog = LogMessageRecord::LOG_SEVERITY_ERROR | LogMessageRecord::AND_USE_SYSLOG, \
    component ## _errorAndStderr = LogMessageRecord::LOG_SEVERITY_ERROR | LogMessageRecord::AND_USE_STDERR, \
    component ## _errorAndBroadcast = LogMessageRecord::LOG_SEVERITY_ERROR | LogMessageRecord::AND_USE_SYSLOG | LogMessageRecord::AND_USE_STDERR, \
    component ## _fatal = LogMessageRecord::LOG_SEVERITY_FATAL, \
    component ## _fatalAndSyslog = LogMessageRecord::LOG_SEVERITY_FATAL | LogMessageRecord::AND_USE_SYSLOG, \
    component ## _fatalAndStderr = LogMessageRecord::LOG_SEVERITY_FATAL | LogMessageRecord::AND_USE_STDERR, \
    component ## _fatalAndBroadcast = LogMessageRecord::LOG_SEVERITY_FATAL | LogMessageRecord::AND_USE_SYSLOG | LogMessageRecord::AND_USE_STDERR, \
    component ## _user = LogMessageRecord::LOG_SEVERITY_USER, \
    component ## _userInfo1 = COMPONENT_USER_LEVEL(component, info, 1), \
    component ## _userInfo2 = COMPONENT_USER_LEVEL(component, info, 2), \
    component ## _userInfo3 = COMPONENT_USER_LEVEL(component, info, 3), \
    component ## _userInfo4 = COMPONENT_USER_LEVEL(component, info, 4), \
    component ## _userInfo5 = COMPONENT_USER_LEVEL(component, info, 5), \
    component ## _userInfo6 = COMPONENT_USER_LEVEL(component, info, 6), \
    component ## _userDebug1 = COMPONENT_USER_LEVEL(component, debug, 1), \
    component ## _userDebug2 = COMPONENT_USER_LEVEL(component, debug, 2), \
    component ## _userDebug3 = COMPONENT_USER_LEVEL(component, debug, 3), \
    component ## _userDebug4 = COMPONENT_USER_LEVEL(component, debug, 4), \
    component ## _userDebug5 = COMPONENT_USER_LEVEL(component, debug, 5), \
    component ## _userDebug6 = COMPONENT_USER_LEVEL(component, debug, 6), \
    component ## _userTrace1 = COMPONENT_USER_LEVEL(component, trace, 1), \
    component ## _userTrace2 = COMPONENT_USER_LEVEL(component, trace, 2), \
    component ## _userTrace3 = COMPONENT_USER_LEVEL(component, trace, 3), \
    component ## _userTrace4 = COMPONENT_USER_LEVEL(component, trace, 4), \
    component ## _userTrace5 = COMPONENT_USER_LEVEL(component, trace, 5), \
    component ## _userTrace6 = COMPONENT_USER_LEVEL(component, trace, 6), \
    component ## _userLevel1 = COMPONENT_USER_LEVEL(component, none, 1), \
    component ## _userLevel2 = COMPONENT_USER_LEVEL(component, none, 2), \
    component ## _userLevel3 = COMPONENT_USER_LEVEL(component, none, 3), \
    component ## _userLevel4 = COMPONENT_USER_LEVEL(component, none, 4), \
    component ## _userLevel5 = COMPONENT_USER_LEVEL(component, none, 5), \
    component ## _userLevel6 = COMPONENT_USER_LEVEL(component, none, 6), \
    component ## _basicLevels = component ## _trace | component ## _debug | \
            component ## _info | component ## _warn | component ## _error | \
            component ## _fatal, \
    component ## _defaultMask = LogMessageRecord::LOG_SEVERITY_INFO | \
            LogMessageRecord::LOG_SEVERITY_WARN | \
            LogMessageRecord::LOG_SEVERITY_ERROR | \
            LogMessageRecord::LOG_SEVERITY_FATAL, \
    ISABLE_ ## component ## _logMask0 = 0, \
    component ## _disableMask = ( LOGAPI_APPEND_TOKEN(DISABLE_COMPONENT_LOG_MASK(component),0) / 10 ) | ( ISABLE_ALL_LOG_MASK ), \
    component ## _all = 0x7fffffff }

/*! \brief Define default symbolic log mask names for a component.
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 *
 * \note The standard names are: \c trace, \c debug, \c info,
 * \c warn, \c error, \c fatal.
 */
#define DEFINE_COMPONENT_LOG_MASKS(component) _DEFINE_COMPONENT_LOG_MASKS(component)



/*! \brief Automatically register a user-specific log mask name with the
 * command line parser.
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 * \param name specifies the name of the log level.
 * \param label is a character pointer string that specifies
 * the message type label to be emitted in the log if the log
 * level is not a subset of the basic levels,
 * such as \c info, \c warn and \c error.
 */
#define AUTO_REGISTER_USER_LOG_MASK(component,name,label) static AutoRegisterLogLevelName LOGAPI_APPEND_TOKEN(autoRegFlag,LOGAPI_APPEND_TOKEN(component,name))(__LOGAPI_EXPAND_STRING2(name),COMPONENT_LEVEL(component,name),((COMPONENT_LEVEL(component,name) & COMPONENT_LEVEL(component,basicLevels)) ? 0 : label))

/*! \brief Define and automatically register a user-specific
 * log mask for a component using DECLARE_COMPONENT_USER_LOG_MASK(),
 * and AUTO_REGISTER_USER_LOG_MASK().
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 * \param name specifies the name of the log level.
 * \param baseLevel identifies a base log level for which this would be a
 * subset, such as \c debug or \c error.  If defining a completely new level,
 * then the keyword \c none should be specified.
 * \param N specifies the relative user-specific mask instance, starting at 1.
 * \param label is a character pointer string that specifies
 * the message type label to be emitted in the log if the log
 * level is not a subset of the basic levels,
 * such as \c info, \c warn and \c error.
 *
 * This is a convenient mechanism to declare and register new
 * log flags specific to a single file.  If the user-specific
 * log level will be used in several files, then an appropriate
 * DECLARE_COMPONENT_USER_LOG_MASK() invocation should be placed into
 * a header file included by the relevant C++ source files and
 * AUTO_REGISTER_USER_LOG_MASK() should be placed into a
 * single C++ source file.
 */
#define DEFINE_COMPONENT_USER_LOG_MASK(component,name,baseLevel,N,label) DECLARE_COMPONENT_USER_LOG_MASK(component,name,baseLevel,N); AUTO_REGISTER_USER_LOG_MASK(component,name,label)

/*! \brief Declare active log mask variable for a component and 
 * standard symbolic log level names, providing one level of macro expansion.
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 */
#define _DECLARE_STANDARD_COMPONENT_VARS(component) \
    DEFINE_COMPONENT_LOG_MASKS(component); \
    extern SharedMemoryVariableNode component ## _namingNode ; \
    extern LogMaskType_t COMPONENT_LOG_MASK(component)

/*! \brief Declare active log mask variable for a component and its standard
 * symbolic log level names.
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 */
#define DECLARE_STANDARD_COMPONENT_VARS(component) _DECLARE_STANDARD_COMPONENT_VARS(component)

/*! \brief Define active log mask for a component,
 * providing one level of macro expansion
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 * \param maskValue specifies the integer value of the bit mask for
 * the log mask variable's initial value.
 */
#define _DEFINE_STANDARD_COMPONENT_VARS_AND_MASK(component,maskValue) \
    SharedMemoryVariableNode component ## _namingNode( #component); \
    LogMaskType_t COMPONENT_LOG_MASK(component)( #component "_logMask", & component ## _namingNode, maskValue)

/*! \brief Define the active log mask variable for a component and its default
 * log mask value.
 * Used in situations where the bitmask value itself is provided,
 * rather than a symbolic log level name.
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 * \param maskValue specifies the integer value of the bit mask for
 * the log mask variable's initial value.
 */
#define DEFINE_STANDARD_COMPONENT_VARS_AND_MASK(component,maskValue) _DEFINE_STANDARD_COMPONENT_VARS_AND_MASK(component,maskValue) 

/*! \brief Define the active log mask variable for a component and its
 * default mask value using a symbolic log level name.
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 * \param defaultMaskName is a token that names the initial log mask
 * using the named level associated with the component.
 * It is not an integer.
 *
 * Referenced by \ref DEFINE_STANDARD_COMPONENT_VARS(), which is the macro
 * normally exploited by users.
 * If no symbolic log level name is appropriate, the actual bitmask value can
 * be used with \ref DEFINE_STANDARD_COMPONENT_VARS_AND_MASK().
 */
#define DEFINE_STANDARD_COMPONENT_VARS_AND_DEFAULT(component,defaultMaskName) \
    _DEFINE_STANDARD_COMPONENT_VARS_AND_MASK(component,COMPONENT_LEVEL(component,defaultMaskName))

/*! \brief Define active log mask variable for a component.  The default
 * mask value is determined by the component's \b defaultMask symbolic log
 * level name.
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 *
 * If a nonstandard default is required, the
 * \ref DEFINE_STANDARD_COMPONENT_VARS_AND_DEFAULT() macro can be used directly.
 */
#define DEFINE_STANDARD_COMPONENT_VARS(component) DEFINE_STANDARD_COMPONENT_VARS_AND_DEFAULT(component,defaultMask)

//! \brief Define the active log mask variable for the default component.
#define DEFINE_STANDARD_VARS_FOR_THIS_COMPONENT() DEFINE_STANDARD_COMPONENT_VARS(THIS_COMPONENT)

/* Internal macros to expand a macro and convert the contents to a string */
#define __LOGAPI_EXPAND_STRING1(x) #x
#define __LOGAPI_EXPAND_STRING2(x) __LOGAPI_EXPAND_STRING1(x)

/*! \brief Provides the value of THIS_COMPONENT as a text string or
 * const char[] value.
 */
#define THIS_COMPONENT_AS_STRING __LOGAPI_EXPAND_STRING2(THIS_COMPONENT)

/*! \brief Automatically register a component's active log mask variable 
 * with the command line parser.
 * \param componentName specifies the name of the component
 */
#define AUTO_REGISTER_COMPONENT(componentName) static AutoRegisterLogComponent LOGAPI_APPEND_TOKEN(autoReg_,componentName) (COMPONENT_LOG_MASK(componentName), __LOGAPI_EXPAND_STRING2(componentName), LOGAPI_APPEND_TOKEN(componentName,_defaultMask));

/*! \brief Automatically register default component's active log mask variable
 * with the command line parser by invoking \ref AUTO_REGISTER_COMPONENT()
 * with THIS_COMPONENT.
 */
#define AUTO_REGISTER_THIS_COMPONENT() AUTO_REGISTER_COMPONENT(THIS_COMPONENT);

/*! \brief Standard mechanism to process logging-related command line
 * arguments.
 *
 * \param argc specifies the total number of arguments.
 * \param argv is an array of char pointers corresponding to command
 * line arguments.
 *
 * \note This is the standard mechanism to process logging-related arguments
 * to an application.  It is typically used as the first statement
 * in \b \c main().
 * Any discovered logging-related arguments are processed and removed from
 * the \a argv array and \a argc is set accordingly.
 */
#define PROCESS_COMMANDLINE_LOG_FLAGS(argc, argv) LogMessageRecord::process_commandline_log_flags(&default_LogSubsystemInfo, argc, argv, &argc, argv)



/*! \brief Initialize and open standard log file for a component,
 * providing one level of macro expansion
 *
 * \param argc provides the count of the number of arguments
 * \param argv is an array of const char pointers holding arguments to the
 * program.
 * \param component is the name of the component
 * \param newArgc is an lvalue into which the new argument count will be stored.
 * \param newArgv is an array of const char pointers into which a duplicate
 * of argv will be placed after any log-related options are stripped.
 * If the initial contents of argc and argv do not need to be preserved,
 * they can be specified as the newArgc and newArgv arguments.
 */
#define _INIT_AND_OPEN_STANDARD_LOG(argc,argv,component,newArgc,newArgv) LogManager::initializeLogSubsystem(newArgc,newArgv,argc, argv, #component , & COMPONENT_LOG_MASK(component)); LogManager::newStandardLogFile()

#ifndef _WIN32
#define _STDIN_FD 0     //!< Platform-independent reference to standard in
#define _STDOUT_FD 1    //!< Platform-independent reference to standard out
#define _STDERR_FD 2    //!< Platform-independent reference to standard error
#define _INVALID_DESCRIPTOR -1 //!< Platform-independent reference to invalid descriptor
#else
#define _STDIN_FD GetStdHandle(STD_INPUT_HANDLE)
#define _STDOUT_FD GetStdHandle(STD_OUTPUT_HANDLE)
#define _STDERR_FD GetStdHandle(STD_ERROR_HANDLE)
#define _INVALID_DESCRIPTOR INVALID_HANDLE_VALUE //!< Platform-independent reference to invalid descriptor
#endif

#ifndef THIS_COMPONENT_LOG_MANAGER
/*! \brief Defines the default log manager object for the component.
 *
 * \note Normally, only THIS_COMPONENT is defined by user source code,
 * but a custom LogManager object for the component can be utilized by
 * defining \ref THIS_COMPONENT_LOG_MANAGER appropriately.
 *
 * If left undefined, which is the normal case, it will default to
 * LogManager::DEFAULT_LogManager.
 *
 * For scenarios in which multiple logs must be manipulated within a
 * single source file, the ALWAYS_EMIT_MESSAGE_TO_LOG() and
 * EMIT_CONDITIONAL_MESSAGE_FOR_COMPONENT_TO_LOG() macros provide
 * the necessary interfaces.
 */
#define THIS_COMPONENT_LOG_MANAGER LogManager::DEFAULT_LogManager
#endif

/* Logic: 1) Create buffer  _logBfrN[MAX_LOG_LINE_LENGTH * 8] on the stack,
 * 2) Create new LogManager _logMgrN on the stack that uses _logBfrN for
 * its workspace.  No parentNode for naming is provided and
 * _INVALID_DESCRIPTOR is used as a fake descriptor (no I/O will be performed).
 * No threading is enabled and an appName of "logstream" is used only for a
 * label in a never-should-happen core dump.
 * The logLinePrefixMask is forced to 0 to prevent any extra data such as
 * date, time, thread id, etc. being placed into the generated output line.
 * The \ref LOG_APPEND_BUFFER() macro also uses a mask of 0 when it makes
 * the formatRecordAsText() call, so a 0 here is not required.
 * 3) Create LogMessageRecord logObjN on the stack (as is always the case)
 * that is associated with the _logMgrN LogManager object.
 * 4) In order to force initialization of the LogMessageRecord,
 * use the << operator to append a null string. Since the null string will
 * add no data, it has no effect on the output line that would be generated;
 * its only purpose is to ensure that at least some << usage was invoked against
 * the logObjN variable before its contents are retrieved using
 * \c LOG_APPEND_BUFFER().
 */
/*! \brief Instantiates a LogMessageRecord object suitable for use as a
 * ostringstream-like buffer.
 *
 * \param n specifies the label to be used to name the generated
 * LogMessageRecord object.  The label specified by \a n will be appended to
 * the prefix \c logObj to construct the variable name.  The user can thus
 * have multiple variables in existence simultaneously by using distinct labels.
 *
 * The constructed content is expected to be retrieved and copied into
 * the destination log using \c LOG_APPEND_BUFFER().
 */
#define LOG_STREAM_OBJECT(n) \
    unsigned char LOGAPI_APPEND_TOKEN(_logBfr,n)[LogManager::MAX_LOG_LINE_LENGTH*8]; \
    LogManager LOGAPI_APPEND_TOKEN(_logMgr,n)(nullptr,_INVALID_DESCRIPTOR,false,"logstream",0,LogManager::MAX_LOG_LINE_LENGTH,LOGAPI_APPEND_TOKEN(_logBfr,n),sizeof( LOGAPI_APPEND_TOKEN(_logBfr,n) )); \
    LogMessageRecord LOGAPI_APPEND_TOKEN(logObj,n)(& LOGAPI_APPEND_TOKEN(_logMgr,n), 0, __FILE__, __LINE__, sizeof(__FILE__) - 1); LOGAPI_APPEND_TOKEN(logObj,n) << ""


/*! \brief Format a log buffer and return its address
 *
 * \param _var specifies a previously declared char pointer into
 * which the address of the formatted buffer will be assigned.
 * \param src specifies the LogMessageRecord currently holding the log message
 *
 * \return A char pointer is returned to the base of the
 * formatted output.  A hidden variable _bRec is introduced into a new scope,
 * which must be closed by \c RELEASE_LOG_BUFFER().
 */
#define FORMAT_LOG_BUFFER(_var,src) do { \
    SharedBufferAllocRecord *_bRec = src.formatRecordAsText(0); \
    _var = (char *) src.getLogManager()->getBufferManager()->blockAddress(_bRec)

/*! \brief Release a log buffer formatted by FORMAT_LOG_BUFFER()
 */
#define RELEASE_LOG_BUFFER(src) src.getLogManager()->getBufferManager()->returnBlock(_bRec); } while (0)

/*! \brief Append log line data constructed from one LogMessageRecord object
 * into another.
 *
 * \param dest specifies the LogMessageRecord object into which the generated
 * message should be appended.
 * \param src specifies the LogMessageRecord currently holding the log message
 * data to be appended.
 *
 * Note that the normal logging-related APIs use the
 * \c ALWAYS_EMIT_MESSAGE_TO_LOG() macro, which declares its LogMessageRecord
 * variable as \c logObj.
 */
#define APPEND_LOG_BUFFER(dest,src) do { char *_addr; \
        FORMAT_LOG_BUFFER(_addr, src); \
        dest << _addr; \
        RELEASE_LOG_BUFFER(src); \
    } while (0)


/*! \brief Initialize and use standard out as log device for a component,
 * providing one level of macro expansion
 *
 * \param argc provides the count of the number of arguments
 * \param argv is an array of const char pointers holding arguments to the
 * program.
 * \param component is the name of the component
 * \param newArgc is an lvalue into which the new argument count will be stored.
 * \param newArgv is an array of const char pointers into which a duplicate
 * of argv will be placed after any log-related options are stripped.
 * If the initial contents of argc and argv do not need to be preserved,
 * they can be specified as the newArgc and newArgv arguments.
 */
#define _INIT_WITH_STANDARD_OUT(argc,argv,component,newArgc,newArgv) LogManager::initializeLogSubsystem(newArgc,newArgv,argc, argv, #component , & COMPONENT_LOG_MASK(component)); SharedMemoryVariableNode _logNamingRoot(#component); THIS_COMPONENT_LOG_MANAGER = LogManager::newLogToFileDescriptor(&_logNamingRoot, _STDOUT_FD, default_LogSubsystemInfo.defaultLogPrefix, default_LogSubsystemInfo.useThreads, 0, default_LogSubsystemInfo.maxPendingLogLines * LogManager::MAX_LOG_LINE_LENGTH)

/*! \brief Create log manager for STDOUT or STDERR.
 *
 * \param output is either \c STDOUT or \c STDERR
 */
#define _SET_LOGMGR_FOR(output) SharedMemoryVariableNode _logNamingRoot ## output(#output); LogManager::  output ## _LogManager = LogManager::newLogToFileDescriptor(&_logNamingRoot ## output, _ ## output ## _FD, default_LogSubsystemInfo.defaultLogPrefix, default_LogSubsystemInfo.useThreads, 0, default_LogSubsystemInfo.maxPendingLogLines * LogManager::MAX_LOG_LINE_LENGTH);


/*! \brief Initialize the logging subsystem and create the default log file
 * for an application.
 *
 * \param argc provides the count of the number of arguments
 * \param argv is an array of const char pointers holding arguments to the
 * program.
 * \param appName is the name of the component
 * \param newArgc is an lvalue into which the new argument count will be stored.
 * \param newArgv is an array of const char pointers into which a duplicate
 * of argv will be placed after any log-related options are stripped.
 * If the initial contents of argc and argv do not need to be preserved,
 * they can be specified as the newArgc and newArgv arguments.
 */
#define INIT_STANDARD_LOG_FOR_COMPONENT(argc,argv,appName,newArgc,newArgv) _INIT_AND_OPEN_STANDARD_LOG(argc,argv,appName,newArgc,newArgv)

/*! \brief Initialize the logging subsystem and create the default log file
 * using the component name provided by \c THIS_COMPONENT.
 *
 * \param argc provides the count of the number of arguments
 * \param argv is an array of const char pointers holding arguments to the
 * program.
 * \param newArgc is an lvalue into which the new argument count will be stored.
 * \param newArgv is an array of const char pointers into which a duplicate
 * of argv will be placed after any log-related options are stripped.
 * If the initial contents of argc and argv do not need to be preserved,
 * they can be specified as the newArgc and newArgv arguments.
 */
#define INIT_STANDARD_LOG_FOR_THIS_COMPONENT(argc,argv,newArgc,newArgv) INIT_STANDARD_LOG_FOR_COMPONENT(argc,argv,THIS_COMPONENT,newArgc,newArgv)


/*! \brief Create LogMessageRecord context for content that will always be
 * emitted to indicated log manager.  The active log mask variable
 * has no control over such content.
 *
 * \param logHandle is the LogManager object to be used for the output.
 * \param mask is the mask that indicates the levels associated with the
 * level name.
 */
#define ALWAYS_EMIT_MESSAGE_TO_LOG(logHandle, mask) do { \
  LogMessageRecord logObj(logHandle, mask, __FILE__, __LINE__, sizeof(__FILE__) - 1)

/*! \brief Create LogMessageRecord context for content that will always be
 * emitted to default log manager.  The active log mask variable
 * has no control over such content.
 *
 * \param lvl is a symbolic name for the component's log level.
 */
#define ALWAYS_EMIT_MESSAGE(lvl) ALWAYS_EMIT_MESSAGE_TO_LOG(THIS_COMPONENT_LOG_MANAGER, THIS_COMPONENT_LEVEL(lvl))

/*! \brief Create LogMessageRecord context for content that will be
 * emitted to the indicated log manager only if enabled by the component's
 * active log mask variable.
 *
 * \param component specifies the component name.  It must be a legal
 * variable name; it is not a string.
 * \param mask is the mask that indicates the levels associated with the
 * level name.
 * \param log is the LogManager object to be used for the output.
 */
#define EMIT_CONDITIONAL_MESSAGE_FOR_COMPONENT_TO_LOG(component,mask,log) if (COMPONENT_LOG_ENABLED(component,mask)) ALWAYS_EMIT_MESSAGE_TO_LOG(log,mask)

/*! \brief Create LogMessageRecord context for content that will be
 * emitted to the default log manager only if enabled by the corresponding
 * component's active log mask variable.
 *
 * \param component specifies the component name.
 * level name.
 * \param lvl is a symbolic name for the component's log level.
 */
#define EMIT_CONDITIONAL_MESSAGE_FOR_COMPONENT(component,lvl) EMIT_CONDITIONAL_MESSAGE_FOR_COMPONENT_TO_LOG(component,COMPONENT_LEVEL(component,lvl),THIS_COMPONENT_LOG_MANAGER)

/*! \brief Create LogMessageRecord context for content that will be
 * emitted to the default log manager only if enabled by the default component's
 * active log mask variable.
 *
 * \param lvl is a symbolic name for the component's log level.
 */
#define EMIT_CONDITIONAL_MESSAGE(lvl) EMIT_CONDITIONAL_MESSAGE_FOR_COMPONENT(THIS_COMPONENT,lvl)

/*! \brief Used to denote the log message format.
 *
 * \param s specifies the log message format.
 */
#define LOG_MESSAGE_FORMAT(s) logObj.setLogFormat(s);

/*! \brief Used to denote an argument for the log message.
 *
 * \param arg specifies an argument for the log message.
 */
#define LOG_ARG(arg) addLogArg(logObj, arg);

/*! \brief Used to denote an binary buffer argument for the log message.
 *
 * \param arg points to the start of the buffer.
 * \param len specifies the length of the buffer.
 */
#define LOG_BINARY_ARG(arg,len) addLogArg(logObj, arg,len);

/*! \brief Used to denote an argument that requires a temporary buffer to
 * permit the content to be formatted.
 *
 * \param arg specifies the argument to be 
 * \param bfr points to the base of the temporary buffer
 * \param bfrLen specifies the length of the temporary buffer.
 */
#define LOG_ARG_USING_BUFFER(arg,bfr,bfrLen) addLogArg(logObj, arg, bfr, bfrLen)

/*! \brief Closing clause to emit log message record as unadorned text.
 *
 * NOTE: in general, AS_TIMESTAMPED_TEXT is preferred as it provides
 * a lot more information, but there are scenarios where only plain
 * text is desired.
 */
#define AS_PLAIN_TEXT logObj.writeAsTextToBuffer(0); } while (0)

/*! \brief Closing clause to emit log message record as text prefixed with
 * an appropriate timestamp and other information.  This is the preferred
 * mechanism.
 */
#define AS_TIMESTAMPED_TEXT logObj.writeAsTextToBuffer(logObj.getLogManager()->getLogPrefixMask()); } while (0)

/*! \brief Closing clause to emit log message record as text prefixed with
 * arbitrarily selected fields.
 *
 * \param selectionMask specifies the bit mask that selects the initial fields
 * to be output on the log line (such as time, thread, file name, line number,
 * etc.)
 *
 * NOTE: use caution when changing line layouts.  Any information not logged is
 * lost forever.  Some simple parsers can be confused by the absence of expected
 * fields.
 */
#define AS_PREFIXED_TEXT(selectionMask) logObj.writeAsTextToBuffer(selectionMask); } while (0)

//! \brief Closing clause to emit log message record as a binary record.
#define AS_BINARY_RECORD logObj.writeAsBinaryToBuffer(logObj.getLogManager()->getLogPrefixMask()); } while (0)

//! log as $date_time $log_level $msg
#define WITH_DATE_TIME_LEVEL logObj.writeAsTextToBuffer( LogMessageRecord::LOG_DATESTAMP | LogMessageRecord::LOG_TIMESTAMP  | LogMessageRecord::LOG_ERROR_LEVEL); } while (0)

//! log as $time $log_level $msg
#define WITH_TIME_LEVEL logObj.writeAsTextToBuffer(LogMessageRecord::LOG_TIMESTAMP | LogMessageRecord::LOG_ERROR_LEVEL); } while (0)

/*! \brief Convenience macro for conditional log output using a \c std::ostream
 * or allomorphic equivalent.
 *
 * \param stream specifies the output stream.
 * \param lvl is a symbolic name for the component's log level.
 */
#define LOG_WHEN_OSTREAM(stream,lvl) if (COMPONENT_LOG_ENABLED(THIS_COMPONENT,THIS_COMPONENT_LEVEL(lvl))) do { stream 

//! End-of-Line fragment for use with \c LOG_WHEN_OSTREAM.
#define OSTREAM_ENDLINE "\n"; } while (0)

/*! \brief Convenience macro to emit conditional message for THIS_COMPONENT
 * via a LogManager associated with standard output or error.
 *
 * \param output specifies the name of responsible log manager and is
 * intended to be \c STDOUT, \c STDERR, or \c DEFAULT.
 * \param lvl is a symbolic name for the component's log level.
 *
 * \note  This will work with arbitrary LogManager objects if their variable is
 * named with a "_LogManager" suffix.
 */
#define LOG_TO_DESCRIPTOR(output,lvl) EMIT_CONDITIONAL_MESSAGE_FOR_COMPONENT_TO_LOG(THIS_COMPONENT,lvl,output ## _LogManager) 


/* In the expected case, LOG_USING_OSTREAM will be undefined;
 * however, it can be defined to something like std::cout to
 * force output to an ostream-like object.
 */
#ifndef LOG_USING_OSTREAM

/*! \brief Convenience declaration for log line that will be always be
 * emitted for the default component to the default log manager.
 *
 * \param lvl is a symbolic name for the component's log level.
 */
#define LOG_ALWAYS(lvl) ALWAYS_EMIT_MESSAGE(lvl); logObj

/*! \brief Convenience declaration for log line that will always be
 * emitted for the default component if the default log manager
 * has been initialized.
 *
 * \param lvl is a symbolic name for the component's log level.
 */
#define IFLOG_ALWAYS(lvl) if (THIS_COMPONENT_LOG_MANAGER != nullptr) ALWAYS_EMIT_MESSAGE(lvl); logObj

/*! \brief Convenience declaration for log line that will be conditionally
 * emitted for the default component to the default log manager if
 * the indicated log level is enabled.
 *
 * \param lvl is a symbolic name for the component's log level.
 */
#define LOG_WHEN(lvl) EMIT_CONDITIONAL_MESSAGE(lvl); logObj

/*! \brief Convenience declaration for log line that will be conditionally
 * emitted for the default component if the default log manager
 * has been initialized and the indicate log level is enabled.
 *
 * \param lvl is a symbolic name for the component's log level.
 */
#define IFLOG_WHEN(lvl) if (THIS_COMPONENT_LOG_MANAGER != nullptr) EMIT_CONDITIONAL_MESSAGE(lvl); logObj

/*! \brief Closing attribute to complete a LOG_ALWAYS() or LOG_WHEN() statement
 * and emit the log message as a text line. The result will be subject to the
 * LogManager object's current prefix mask, which might inhibit the output
 * of elements such as the date, thread id, file name, source line number, etc.
 */
#define ENDREC "\n"; logObj.writeAsTextToBuffer(logObj.getLogManager()->getLogPrefixMask()); } while (0)

/*! \brief Closing attribute to complete LOG_ALWAYS() or LOG_WHEN statement
 * and emit the log message as a binary record.
 */
#define BINARY_ENDREC "\n"; logObj.writeAsBinaryToBuffer(logObj.getLogManager()->getLogPrefixMask()); } while (0)

//! \brief Closing clause for text line output using << operators.
#define LOG_ENDLINE ENDREC

//! \brief Closing close for binary record output using << operators.
#define BINARY_LOG_ENDLINE BINARY_ENDREC

/*! \brief Conditionally emits a log message constructed using the << operator.
 * The end of line should be terminated with either \ref LOG_ENDLINE or
 * \ref BINARY_LOG_ENDLINE.
 * \param mgrPtr is a pointer to the LogManager object.
 * \param component identifies the relevant component.
 * \param lvl is a symbolic name for the component's log level.
 *
 * The bulk of the work is provided by the low-level macro
 * \ref EMIT_CONDITIONAL_MESSAGE_FOR_COMPONENT_TO_LOG().
 */
#define LOG_COMPONENT_INTO(mgrPtr,component,lvl) EMIT_CONDITIONAL_MESSAGE_FOR_COMPONENT_TO_LOG(component,COMPONENT_LEVEL(component,lvl),mgrPtr); logObj

/*! \brief Always emits a log message constructed using the << operator.
 * The end of line should be terminated with either LOG_ENDLINE or
 * BINARY_LOG_ENDLINE.
 *
 * \arg mgrPtr is a pointer to the LogManager object.
 * \arg component identifies the relevant component.
 * \arg lvl is a symbolic name for the component's log level.
 *
 * The bulk of the work is provided by the low-level macro
 * ALWAYS_EMIT_MESSAGE_TO_LOG().
 */
#define LOG_COMPONENT_ALWAYS_INTO(mgrPtr,component,lvl) ALWAYS_EMIT_MESSAGE_TO_LOG(mgrPtr,COMPONENT_LEVEL(component,lvl)); logObj

/*! \brief Convenience macro that uses \ref LOG_COMPONENT_INTO to 
 * conditionally emit a message associated with THIS_COMPONENT.
 *
 * \param mgrPtr specifies the LogManager object to be used.
 * \param lvl is a symbolic name for the component's log level.
 */
#define LOG_INTO(mgrPtr,lvl) LOG_COMPONENT_INTO(mgrPtr,THIS_COMPONENT,lvl)

/*! \brief Convenience macro that uses \ref LOG_COMPONENT_INTO to 
 * emit a message associated with THIS_COMPONENT.
 *
 * \param mgrPtr specifies the LogManager object to be used.
 * \param lvl is a symbolic name for the component's log level.
 */
#define LOG_ALWAYS_INTO(mgrPtr,lvl) LOG_COMPONENT_ALWAYS_INTO(mgrPtr,THIS_COMPONENT,lvl)

/*! \brief Convenience macro that uses \ref LOG_INTO() to conditionally
 * log a message to standard output.
 *
 * \param lvl is a symbolic name for the component's log level.
 */
#define LOG_COUT(lvl) LOG_INTO(LogManager::STDOUT_LogManager, lvl)

/*! \brief Convenience macro that uses \ref LOG_INTO() to conditionally
 * log a message to standard error.
 *
 * \param lvl is a symbolic name for the component's log level.
 */
#define LOG_CERR(lvl) LOG_INTO(LogManager::STDERR_LogManager, lvl)

/*! \brief Convenience macro that uses \ref LOG_COMPONENT_INTO to conditionally
 * log a message to standard output.
 *
 * \param component identifies the relevant component.
 * \param lvl is a symbolic name for the component's log level.
 */
#define LOG_COMPONENT_COUT(component,lvl) LOG_COMPONENT_INTO(LogManager::STDOUT_LogManager,component,lvl)

/*! \brief Convenience macro that uses \ref LOG_COMPONENT_INTO to conditionally
 * log a message to standard error.
 *
 * \param component identifies the relevant component.
 * \param lvl is a symbolic name for the component's log level.
 */
#define LOG_COMPONENT_CERR(component,lvl) LOG_COMPONENT_INTO(LogManager::STDERR_LogManager,component,lvl)

/*! \brief Convenience macro that uses \ref LOG_COMPONENT_INTO to conditionally
 * log a message to the default log manager, which is in turn specified
 * by the \ref THIS_COMPONENT_LOG_MANAGER macro.
 *
 * \param component identifies the relevant component.
 * \param lvl is a symbolic name for the component's log level.
 */
#define LOG_COMPONENT(component,lvl) LOG_COMPONENT_INTO(THIS_COMPONENT_LOG_MANAGER,component,lvl)

/*! \brief Convenience macro that uses \ref LOG_COMPONENT_ALWAYS_INTO to
 * log a message to the default log manager, which is in turn specified
 * by the \ref THIS_COMPONENT_LOG_MANAGER macro.
 */
#define LOG_COMPONENT_ALWAYS(component,lvl) LOG_COMPONENT_ALWAYS_INTO(THIS_COMPONENT_LOG_MANAGER,component,lvl)

/*! \brief Only if the default log manager has been created, uses the
 * LogManager specified by the THIS_COMPONENT_LOG_MANAGER macro to conditionally
 * emit a log message.
 *
 * \param component identifies the relevant component.
 * \param lvl is a symbolic name for the component's log level.
 * 
 * \note This macro can be safely used in library routines that cannot
 * assume that there will be a default LogManager in existence.  There are
 * two downsides.  The major one is that there is an integer comparison
 * performed on every invocation to check for a non-null pointer.  This
 * is not a high computational load and modern CPUs will cache the result
 * of the test, but it is overhead.  The other downside is that no message
 * will be emitted if there was no log file created, which makes this
 * mechanism unsuitable for critical messages that must be emitted no matter
 * what.  In those scenarios, consider using the STDERR_LogManager as
 * both the STDOUT_LogManger and STDERR_LogManager will always be present.
 */
#define IFLOG_COMPONENT(component,lvl) if (OME_EXPECT_TRUE(THIS_COMPONENT_LOG_MANAGER != nullptr)) LOG_COMPONENT_INTO(THIS_COMPONENT_LOG_MANAGER,component,lvl)

/*! \brief Similiar to IFLOG_COMPONENT(), but the message cannot be disabled
 * using the component's logLevel mask.
 *
 * \param component identifies the relevant component.
 * \param lvl is a symbolic name for the component's log level.
 */
#define IFLOG_COMPONENT_ALWAYS(component,lvl) if (OME_EXPECT_TRUE(THIS_COMPONENT_LOG_MANAGER != nullptr)) ALWAYS_EMIT_MESSAGE_TO_LOG(THIS_COMPONENT_LOG_MANAGER,COMPONENT_LEVEL(component,lvl)); logObj

#else /* LOG_USING_OSTREAM was defined */

#define LOG_ALWAYS(lvl) do { LOG_USING_OSTREAM
#define IFLOG_ALWAYS(level) if (THIS_COMPONENT_LOG_MANAGER != nullptr) { LOG_USING_OSTREAM

#define LOG_WHEN(lvl) LOG_WHEN_OSTREAM(LOG_USING_OSTREAM,lvl)
#define IFLOG_WHEN(lvl) if (THIS_COMPONENT_LOG_MANAGER != nullptr) LOG_WHEN_OSTREAM(LOG_USING_OSTREAM,lvl)

#define LOG_COUT(lvl) LOG_WHEN_OSTREAM(std::cout,lvl)
#define LOG_CERR(lvl) LOG_WHEN_OSTREAM(std::cerr,lvl)
#define LOG_ENDLINE std::endl; } while (0)
#define ENDREC LOG_ENDLINE /* tolerate ENDREC usage */

#endif // LOG_USING_STREAM not defined (expected case)

DEFINE_COMPONENT_LOG_MASKS(app);
DEFINE_COMPONENT_LOG_MASKS(io);

//! \brief Default application log mask; will be aliased to component
extern LogMaskType_t COMPONENT_LOG_MASK(app);
//! \brief Default io-or-library-related log mask
extern LogMaskType_t COMPONENT_LOG_MASK(io);


/*! \brief Label a block of text that is split across multiple log lines.
 *
 * \param label specifies the text label to be used to identify the block.
 *
 * The LogMessageRecord::splitLongLogLine() routine is used to perform
 * the actual split across multiple lines.
 */
#define SPLIT_ACROSS_LINES(label,...) "BEGIN " << label << "\n"; \
    logObj.writeAsTextToBuffer(logObj.getLogManager()->getLogPrefixMask()); \
    LogMessageRecord::splitLongLogLine(logObj, __VA_ARGS__); \
    logObj << "END " << label

/* Support native output of sockaddr structures in ASCII readable text
 *
 * \param os is a reference to the output stream or object
 * \param addr is the address to be output as text.
 *
 * For performance, one would normally NOT use a const reference on these
 * simple structures, but because an address is required for the inet_ntop()
 * call, a reference is appropriate: we were forced onto the stack already.
 */
template <typename STREAM> inline STREAM & operator<<(STREAM& os, const struct in_addr &addr)
{
    char buffer[16];
    const char *result;
#ifndef _WIN32
    // from <arpa/inet.h>
    result = inet_ntop(AF_INET, &addr, buffer, sizeof(buffer));
#else
    // from Ws2tcpip.h
    /* Windows has broken definition that declares second argument as void *
     * rather than const void *.
     */
    result = InetNtop(AF_INET, const_cast<struct in_addr *>(&addr), buffer, sizeof(buffer));
#endif
    if (result == nullptr) {
        result = "[badipv4]";
    }
    os << result;
    return (os);
}

template <typename STREAM> inline STREAM & operator<<(STREAM& os, const struct in6_addr &addr)
{
    char buffer[64];
    const char *result;
#ifndef _WIN32
    // from <arpa/inet.h>
    result = inet_ntop(AF_INET6, &addr, buffer, sizeof(buffer));
#else
    // from Ws2tcpip.h
    /* Windows has broken definition that declares second argument as void *
     * rather than const void *.
     */
    result = InetNtop(AF_INET6, const_cast<struct in6_addr *>(&addr), buffer, sizeof(buffer));
#endif
    if (result == nullptr) {
        result = "[badipv6]";
    }
    os << result;
    return (os);
}

template <typename STREAM> inline STREAM & operator<<(STREAM& os, const struct sockaddr_in &s)
{
    os << "ipv4://";
    os << s.sin_addr;
    os << ":";
    os << ntohs(s.sin_port);
    return (os);
}

template <typename STREAM> inline STREAM & operator<<(STREAM& os, const struct sockaddr_in6 &s)
{
    os << "ipv6://";
    os << s.sin6_addr;
    os << ":";
    os << ntohs(s.sin6_port);
    return (os);
}

#ifndef _WIN32
template <typename STREAM> inline STREAM & operator<<(STREAM& os, const struct sockaddr_un &s)
{
    os << "file://";
    os << s.sun_path;
    return (os);
}
#endif

template <typename STREAM> inline STREAM & operator<<(STREAM& os, const struct sockaddr &s)
{
    switch (s.sa_family) {
    case AF_INET: {
        const struct sockaddr_in *ipv4 = reinterpret_cast<const struct sockaddr_in *>(&s);
        os << *ipv4;
                  }
        break;
    case AF_INET6: {
        const struct sockaddr_in6 *ipv6 = reinterpret_cast<const struct sockaddr_in6 *>(&s);
        os << *ipv6;
                   }
        break;
#ifndef _WIN32
    case AF_LOCAL: {
        const struct sockaddr_un *u = reinterpret_cast<const struct sockaddr_un *>(&s);
        os << *u;
                  }
        break;
#endif
    default:
        os << "[UnknownAF: " << s.sa_family << "]";
        break;
    }
    return (os);
}

/*! \brief Convenience macro to output a member field 
 *
 * \param s is a reference to the structure from which the field
 * should be obtained
 * \param f is the name of the field.
 */
#define _OUTPUT_MEMBER_FIELD(s,f) << " " #f "=" << (s). ## f

/*! \brief Convenience macro to output a member field as a string 
 *
 * \param s is a reference to the structure from which the field
 * should be obtained
 * \param f is the name of the string field.
 * \param l is the length of the string contents desired.
 */
#define _OUTPUT_MEMBER_FIELD_AS_STRING_BLOCK(s,f,l) << " " #f "=\"" << AS_ESCAPED_TEXT_BUFFER((s). ## f, l) << "\""

/*! \brief Convenience macro to output a member field as a string,
 * with the length determined at runtime.
 *
 * \param s is a reference to the structure from which the field
 * should be obtained
 * \param f is the name of the string field.
 */
#define _OUTPUT_MEMBER_FIELD_AS_STRING(s,f) _OUTPUT_MEMBER_FIELD_AS_STRING_BLOCK(s,f,strlen((s). ## f))

/** @} */

#endif
/* vim: set expandtab shiftwidth=4 tabstop=4: */
